@@ -38,6 +38,7 @@
 #include "cairo-dock-desktop-file-factory.h"
 #include "cairo-dock-file-manager.h"
 #include "cairo-dock-applications-manager.h"
+#include "cairo-dock-keybinder.h"
 #include "cairo-dock-X-manager.h"
 #include "cairo-dock-gui-manager.h"
 #include "cairo-dock-gui-commons.h"
@@ -45,11 +46,11 @@
 #include "cairo-dock-gui-items.h"
 #include "cairo-dock-gui-main.h"
 
-#define CAIRO_DOCK_GROUP_ICON_SIZE 32
-#define CAIRO_DOCK_CATEGORY_ICON_SIZE 32
+#define CAIRO_DOCK_GROUP_ICON_SIZE 28  // 32
+#define CAIRO_DOCK_CATEGORY_ICON_SIZE 28  // 32
 #define CAIRO_DOCK_NB_BUTTONS_BY_ROW 4
 #define CAIRO_DOCK_NB_BUTTONS_BY_ROW_MIN 3
-#define CAIRO_DOCK_TABLE_MARGIN 12
+#define CAIRO_DOCK_TABLE_MARGIN 6
 #define CAIRO_DOCK_CONF_PANEL_WIDTH 1250
 #define CAIRO_DOCK_CONF_PANEL_WIDTH_MIN 800
 #define CAIRO_DOCK_CONF_PANEL_HEIGHT 700
@@ -57,11 +58,9 @@
 #define CAIRO_DOCK_PREVIEW_WIDTH_MIN 100
 #define CAIRO_DOCK_PREVIEW_HEIGHT 250
 #define CAIRO_DOCK_ICON_MARGIN 6
-#define CAIRO_DOCK_TAB_ICON_SIZE 32
-#define CAIRO_DOCK_PLUGINS_EXTRAS_URL "http://extras.glx-dock.org"
+#define CAIRO_DOCK_TAB_ICON_SIZE 24
 
 extern CairoDockDesktopGeometry g_desktopGeometry;
-extern int g_iMajorVersion, g_iMinorVersion, g_iMicroVersion;
 
 extern gchar *g_cConfFile;
 extern CairoContainer *g_pPrimaryContainer;
@@ -107,6 +106,7 @@ static CairoDockCategoryWidgetTable s_pCategoryWidgetTables[CAIRO_DOCK_NB_CATEGO
 GSList *s_pCurrentWidgetList;  // liste des widgets du module courant.
 GSList *s_pExtraCurrentWidgetList;  // liste des widgets des eventuels modules lies.
 static GList *s_pGroupDescriptionList = NULL;
+static GtkWidget *s_pPreviewBox = NULL;
 static GtkWidget *s_pPreviewImage = NULL;
 static GtkWidget *s_pOkButton = NULL;
 static GtkWidget *s_pApplyButton = NULL;
@@ -862,9 +862,11 @@ static gboolean _show_group_dialog (CairoDockGroupDescription *pGroupDescription
 				1,
 				1);
 		}
+		else 
+			gtk_widget_show (s_pPreviewBox);
+
 		gtk_image_set_from_pixbuf (GTK_IMAGE (pPreviewImage), pPreviewPixbuf);
 		gdk_pixbuf_unref (pPreviewPixbuf);
-		gtk_widget_show (pPreviewImage);
 	}
 	
 	if (s_pDialog != NULL)
@@ -914,7 +916,7 @@ static void on_leave_group_button (GtkButton *button, gpointer *data)
 
 	int iPreviewWidgetWidth = s_iPreviewWidth;
 	GtkWidget *pPreviewImage = s_pPreviewImage;
-	gtk_widget_hide (pPreviewImage);
+	gtk_widget_hide (s_pPreviewBox);
 	
 	if (! cairo_dock_dialog_unreference (s_pDialog))
 		cairo_dock_dialog_unreference (s_pDialog);
@@ -976,22 +978,14 @@ static void cairo_dock_free_categories (void)
 	s_path = NULL;
 }
 
-static gboolean on_delete_main_gui (GtkWidget *pWidget, GdkEvent *event, GMainLoop *pBlockingLoop)
+static gboolean on_delete_main_gui (GtkWidget *pWidget, gpointer data)
 {
-	cd_debug ("%s (%ld)\n", __func__, pBlockingLoop);
-	if (pBlockingLoop != NULL)
-	{
-		cd_debug ("dialogue detruit, on sort de la boucle");
-		//if (g_main_loop_is_running (pBlockingLoop))
-		//	g_main_loop_quit (pBlockingLoop);
-	}
 	cairo_dock_free_categories ();
 	if (s_iSidShowGroupDialog != 0)
 	{
 		g_source_remove (s_iSidShowGroupDialog);
 		s_iSidShowGroupDialog = 0;
 	}
-	
 	return FALSE;
 }
 
@@ -1097,15 +1091,9 @@ static void on_click_apply (GtkButton *button, GtkWidget *pWindow)
 
 static void on_click_quit (GtkButton *button, GtkWidget *pWindow)
 {
-	//g_print ("%s ()\n", __func__);
-	GMainLoop *pBlockingLoop = g_object_get_data (G_OBJECT (pWindow), "loop");
-	
-	gboolean bReturn;
-	g_signal_emit_by_name (pWindow, "delete-event", NULL, &bReturn);
-	///on_delete_main_gui (pWindow, NULL, pBlockingLoop);
-	
-	if (pBlockingLoop == NULL)
-		gtk_widget_destroy (pWindow);
+	///gboolean bReturn;
+	///g_signal_emit_by_name (pWindow, "delete-event", NULL, &bReturn);
+	gtk_widget_destroy (pWindow);
 }
 
 static void on_click_ok (GtkButton *button, GtkWidget *pWindow)
@@ -1282,14 +1270,6 @@ static inline GtkWidget *_make_image (const gchar *cImage, int iSize)
 	}
 	return pImage;
 }
-static void _add_image_on_button (GtkWidget *pButton, const gchar *cImage, int iSize)
-{
-	if (cImage == NULL)
-		return ;
-	GtkWidget *pImage = _make_image (cImage, iSize);
-	if (pImage != NULL)
-		gtk_button_set_image (GTK_BUTTON (pButton), pImage);  /// unref l'image ?...
-}
 static GtkToolItem *_make_toolbutton (const gchar *cLabel, const gchar *cImage, int iSize)
 {
 	if (cImage == NULL)
@@ -1305,7 +1285,7 @@ static GtkToolItem *_make_toolbutton (const gchar *cLabel, const gchar *cImage,
 		return pWidget;
 	
 	GtkWidget *pLabel = gtk_label_new (NULL);
-	gchar *cLabel2 = g_strdup_printf ("<span font_desc=\"Sans 12\"><b>%s</b></span>", cLabel);
+	gchar *cLabel2 = g_strdup_printf ("<span size='large' weight='800'>%s</span>", cLabel);
 	gtk_label_set_markup (GTK_LABEL (pLabel), cLabel2);
 	g_free (cLabel2);
 	
@@ -1364,7 +1344,7 @@ static inline CairoDockGroupDescription *_add_group_button (const gchar *cGroupN
 		return pGroupDescription;
 	
 	//\____________ On construit le bouton du groupe.
-	GtkWidget *pGroupHBox = gtk_hbox_new (FALSE, CAIRO_DOCK_FRAME_MARGIN);
+	GtkWidget *pGroupHBox = _gtk_hbox_new (CAIRO_DOCK_FRAME_MARGIN);
 	pGroupDescription->pGroupHBox = pGroupHBox;
 	
 	pGroupDescription->pActivateButton = gtk_check_button_new ();
@@ -1386,8 +1366,8 @@ static inline CairoDockGroupDescription *_add_group_button (const gchar *cGroupN
 		gtk_widget_set_sensitive (pGroupButton, FALSE);
 	g_signal_connect (G_OBJECT (pGroupButton), "enter", G_CALLBACK(on_enter_group_button), pGroupDescription);
 	g_signal_connect (G_OBJECT (pGroupButton), "leave", G_CALLBACK(on_leave_group_button), NULL);
-	
-	GtkWidget *pButtonHBox = gtk_hbox_new (FALSE, CAIRO_DOCK_FRAME_MARGIN);
+
+	GtkWidget *pButtonHBox = _gtk_hbox_new (CAIRO_DOCK_FRAME_MARGIN);
 	GtkWidget *pImage = _make_image (cIconPath, CAIRO_DOCK_GROUP_ICON_SIZE);
 	gtk_box_pack_start (GTK_BOX (pButtonHBox), pImage, FALSE, FALSE, 0);
 	pGroupDescription->pLabel = gtk_label_new (pGroupDescription->cTitle);
@@ -1458,6 +1438,24 @@ static gboolean _cairo_dock_add_one_module_widget (CairoDockModule *pModule, con
 	return TRUE;  // continue.
 }
 
+static void _load_shortkeys_widget (CairoDockModuleInstance *pInstance, GKeyFile *pKeyFile)
+{
+	//\_____________ On recupere notre emplacement perso dans la fenetre.
+	CairoDockGroupKeyWidget *myWidget = cairo_dock_gui_find_group_key_widget_in_list (s_pCurrentWidgetList, "Shortkeys", "shortkeys");
+	g_return_if_fail (myWidget != NULL);
+	
+	//\_____________ On construit le tree-view.
+	GtkWidget *pOneWidget = cairo_dock_build_shortkeys_widget ();
+	
+	//\_____________ On l'ajoute a la fenetre.
+	GtkWidget *pScrolledWindow = gtk_scrolled_window_new (NULL, NULL);
+	g_object_set (pScrolledWindow, "height-request", MIN (2*CAIRO_DOCK_PREVIEW_HEIGHT, g_desktopGeometry.iXScreenHeight[CAIRO_DOCK_HORIZONTAL] - 175), NULL);
+	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (pScrolledWindow), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+	gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (pScrolledWindow), pOneWidget);
+	myWidget->pSubWidgetList = g_slist_append (myWidget->pSubWidgetList, pOneWidget);  // on le met dans la liste, non pas pour recuperer sa valeur, mais pour pouvoir y acceder facilement plus tard.
+	gtk_box_pack_start (GTK_BOX (myWidget->pKeyBox), pScrolledWindow, FALSE, FALSE, 0);
+}
+
 #define _add_one_main_group_button(cGroupName, cIcon, iCategory, cDescription, cTitle) \
 _add_group_button (cGroupName,\
 		cIcon,\
@@ -1497,6 +1495,13 @@ static void _add_main_groups_buttons (void)
 		_("Taskbar"));
 	pGroupDescription->pManagers = g_list_prepend (NULL, (gchar*)"Taskbar");
 	
+	pGroupDescription = _add_one_main_group_button ("Shortkeys",
+		"gtk-select-font",  /// TODO: trouver une meilleure icone, et l'utiliser aussi pour le backend "simple"...
+		CAIRO_DOCK_CATEGORY_BEHAVIOR,
+		N_("Define all the keyboard shortcuts currently available."),
+		_("Shortkeys"));
+	pGroupDescription->load_custom_widget = _load_shortkeys_widget;
+	
 	pGroupDescription = _add_one_main_group_button ("System",
 		"icon-system.svg",
 		CAIRO_DOCK_CATEGORY_BEHAVIOR,
@@ -1561,8 +1566,24 @@ static void _add_main_groups_buttons (void)
 	pGroupDescription->pManagers = g_list_prepend (NULL, (gchar*)"Icons");
 }
 
+static GtkWidget *cairo_dock_build_main_ihm_left_frame (const gchar *cText)
+{
+	// frame
+	GtkWidget *pFrame = gtk_frame_new (NULL);
+	//gtk_container_set_border_width (GTK_CONTAINER (pFrame), CAIRO_DOCK_FRAME_MARGIN);
+	gtk_frame_set_shadow_type (GTK_FRAME (pFrame), GTK_SHADOW_NONE);
+	
+	// label
+	gchar *cLabel = g_strdup_printf ("<span size='x-large' weight='800' color=\"#81728C\">%s</span>", cText);
+	GtkWidget *pLabel = gtk_label_new (NULL);
+	gtk_label_set_markup (GTK_LABEL (pLabel), cLabel);
+	g_free (cLabel);
+	gtk_frame_set_label_widget (GTK_FRAME (pFrame), pLabel);
+	
+	return pFrame;
+}
 
-static GtkWidget *cairo_dock_build_main_ihm (const gchar *cConfFilePath, gboolean bMaintenanceMode)
+static GtkWidget *cairo_dock_build_main_ihm (const gchar *cConfFilePath)
 {
 	//\_____________ On construit la fenetre.
 	if (s_pMainWindow != NULL)
@@ -1575,8 +1596,8 @@ static GtkWidget *cairo_dock_build_main_ihm (const gchar *cConfFilePath, gboolea
 	gchar *cIconPath = g_strdup_printf ("%s/%s", CAIRO_DOCK_SHARE_DATA_DIR, CAIRO_DOCK_ICON);
 	gtk_window_set_icon_from_file (GTK_WINDOW (s_pMainWindow), cIconPath, NULL);
 	g_free (cIconPath);
-	
-	GtkWidget *pMainHBox = gtk_hbox_new (FALSE, 0);
+
+	GtkWidget *pMainHBox = _gtk_hbox_new (0);
 	gtk_container_add (GTK_CONTAINER (s_pMainWindow), pMainHBox);
 	
 	if (g_desktopGeometry.iXScreenWidth[CAIRO_DOCK_HORIZONTAL] > CAIRO_DOCK_CONF_PANEL_WIDTH)
@@ -1596,42 +1617,107 @@ static GtkWidget *cairo_dock_build_main_ihm (const gchar *cConfFilePath, gboolea
 		s_iPreviewWidth = CAIRO_DOCK_PREVIEW_WIDTH_MIN;
 		s_iNbButtonsByRow = CAIRO_DOCK_NB_BUTTONS_BY_ROW_MIN;
 	}
-	GtkWidget *pCategoriesVBox = gtk_vbox_new (FALSE, CAIRO_DOCK_FRAME_MARGIN);
+
+	GtkWidget *pCategoriesVBox = _gtk_vbox_new (CAIRO_DOCK_FRAME_MARGIN);
 	gtk_widget_set_size_request (pCategoriesVBox, s_iPreviewWidth+2*CAIRO_DOCK_FRAME_MARGIN, CAIRO_DOCK_PREVIEW_HEIGHT);
 	gtk_box_pack_start (GTK_BOX (pMainHBox),
 		pCategoriesVBox,
 		FALSE,
 		FALSE,
 		0);
-	
-	GtkWidget *pVBox = gtk_vbox_new (FALSE, 0);
+
+	GtkWidget *pVBox = _gtk_vbox_new (0);
 	gtk_box_pack_start (GTK_BOX (pMainHBox),
 		pVBox,
 		TRUE,
 		TRUE,
 		0);
-	s_pGroupsVBox = gtk_vbox_new (FALSE, CAIRO_DOCK_TABLE_MARGIN);
+	s_pGroupsVBox = _gtk_vbox_new (CAIRO_DOCK_TABLE_MARGIN);
 	GtkWidget *pScrolledWindow = gtk_scrolled_window_new (NULL, NULL);
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (pScrolledWindow), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-	gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (pScrolledWindow), s_pGroupsVBox);
+	GtkWidget *pViewport = gtk_viewport_new( NULL, NULL );
+	gtk_viewport_set_shadow_type ( GTK_VIEWPORT (pViewport), GTK_SHADOW_NONE);
+	gtk_container_add (GTK_CONTAINER( pViewport ), s_pGroupsVBox );
+	gtk_container_add (GTK_CONTAINER( pScrolledWindow ), pViewport );
+
 	gtk_box_pack_start (GTK_BOX (pVBox),
 		pScrolledWindow,
 		TRUE,
 		TRUE,
 		0);
 	
+	// Empty box to get some space between window border and filter label
+	gtk_box_pack_start (GTK_BOX (pCategoriesVBox),
+		_gtk_hbox_new (CAIRO_DOCK_FRAME_MARGIN / 2),
+		FALSE,
+		FALSE,
+		0);
+
+	//\_____________ Filter.
+	GtkWidget *pFilterFrame = cairo_dock_build_main_ihm_left_frame (_("Filter"));
+	gtk_box_pack_start (GTK_BOX (pCategoriesVBox),
+		pFilterFrame,
+		FALSE,
+		FALSE,
+		0);
+
+	// text entry
+	GtkWidget *pFilterBox = _gtk_hbox_new (0);
+	gtk_container_add (GTK_CONTAINER (pFilterFrame), pFilterBox);
+
+	s_pFilterEntry = gtk_entry_new ();
+	g_signal_connect (s_pFilterEntry, "activate", G_CALLBACK (on_activate_filter), NULL);
+	gtk_box_pack_start (GTK_BOX (pFilterBox),
+		s_pFilterEntry,
+		TRUE,
+		FALSE,
+		0);
+	//~ gtk_container_set_focus_child (GTK_CONTAINER (s_pMainWindow), pFilterBox); /// set focus to filter box
+	
+	#if (GTK_MAJOR_VERSION > 2 || GTK_MINOR_VERSION >= 16)
+	gtk_entry_set_icon_activatable (GTK_ENTRY (s_pFilterEntry), GTK_ENTRY_ICON_SECONDARY, TRUE);
+	gtk_entry_set_icon_from_stock (GTK_ENTRY (s_pFilterEntry), GTK_ENTRY_ICON_SECONDARY, GTK_STOCK_CLEAR);
+	g_signal_connect (s_pFilterEntry, "icon-press", G_CALLBACK (on_clear_filter), NULL);
+	#endif
+	
+	// Filter options
+	_reset_filter_state ();
+	
+	GtkWidget *pMenuBar = gtk_menu_bar_new ();
+	GtkWidget *pMenuItem = gtk_image_menu_item_new_from_stock (GTK_STOCK_PREFERENCES, NULL);
+	gtk_menu_item_set_label (GTK_MENU_ITEM (pMenuItem), NULL);
+	gtk_menu_shell_append (GTK_MENU_SHELL (pMenuBar), pMenuItem);
+	gtk_box_pack_start (GTK_BOX (pFilterBox),
+		pMenuBar,
+		FALSE,
+		FALSE,
+		0);
+	GtkWidget *pMenu = gtk_menu_new ();
+	gtk_menu_item_set_submenu (GTK_MENU_ITEM (pMenuItem), pMenu);
+	
+	
+	pMenuItem = gtk_check_menu_item_new_with_label (_("All words"));
+	gtk_menu_shell_append (GTK_MENU_SHELL (pMenu), pMenuItem);
+	g_signal_connect (pMenuItem, "toggled", G_CALLBACK (on_toggle_all_words), NULL);
+	
+	pMenuItem = gtk_check_menu_item_new_with_label (_("Highlighted words"));
+	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (pMenuItem), TRUE);
+	gtk_menu_shell_append (GTK_MENU_SHELL (pMenu), pMenuItem);
+	g_signal_connect (pMenuItem, "toggled", G_CALLBACK (on_toggle_highlight_words), NULL);
+	
+	pMenuItem = gtk_check_menu_item_new_with_label (_("Hide others"));
+	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (pMenuItem), TRUE);
+	gtk_menu_shell_append (GTK_MENU_SHELL (pMenu), pMenuItem);
+	g_signal_connect (pMenuItem, "toggled", G_CALLBACK (on_toggle_hide_others), NULL);
+	
+	pMenuItem = gtk_check_menu_item_new_with_label (_("Search in description"));
+	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (pMenuItem), TRUE);
+	gtk_menu_shell_append (GTK_MENU_SHELL (pMenu), pMenuItem);
+	g_signal_connect (pMenuItem, "toggled", G_CALLBACK (on_toggle_search_in_tooltip), NULL);
+	
+
 	//\_____________ On construit les boutons de chaque categorie.
-	GtkWidget *pCategoriesFrame = gtk_frame_new (NULL);
-	gtk_container_set_border_width (GTK_CONTAINER (pCategoriesFrame), CAIRO_DOCK_FRAME_MARGIN);
-	gtk_frame_set_shadow_type (GTK_FRAME (pCategoriesFrame), GTK_SHADOW_OUT);
-	
-	GtkWidget *pLabel;
-	gchar *cLabel = g_strdup_printf ("<span font_desc=\"Sans 12\" color=\"#81728C\"><b><u>%s :</u></b></span>", _("Categories"));
-	pLabel = gtk_label_new (NULL);
-	gtk_label_set_markup (GTK_LABEL (pLabel), cLabel);
-	g_free (cLabel);
-	gtk_frame_set_label_widget (GTK_FRAME (pCategoriesFrame), pLabel);
-	
+	GtkWidget *pCategoriesFrame = cairo_dock_build_main_ihm_left_frame (_("Categories"));
 	gtk_box_pack_start (GTK_BOX (pCategoriesVBox),
 		pCategoriesFrame,
 		TRUE,  /// FALSE
@@ -1639,7 +1725,11 @@ static GtkWidget *cairo_dock_build_main_ihm (const gchar *cConfFilePath, gboolea
 		0);
 	
 	s_pToolBar = gtk_toolbar_new ();
+	#if (GTK_MAJOR_VERSION < 3 && GTK_MINOR_VERSION < 16)
 	gtk_toolbar_set_orientation (GTK_TOOLBAR (s_pToolBar), GTK_ORIENTATION_VERTICAL);
+	#else
+	gtk_orientable_set_orientation (GTK_ORIENTABLE (s_pToolBar), GTK_ORIENTATION_VERTICAL);
+	#endif
 	gtk_toolbar_set_style (GTK_TOOLBAR (s_pToolBar), GTK_TOOLBAR_BOTH_HORIZ);
 	gtk_toolbar_set_show_arrow (GTK_TOOLBAR (s_pToolBar), TRUE);  /// FALSE
 	//gtk_widget_set (s_pToolBar, "height-request", 300, NULL);
@@ -1653,9 +1743,10 @@ static GtkWidget *cairo_dock_build_main_ihm (const gchar *cConfFilePath, gboolea
 	pCategoryButton = _make_toolbutton (_("All"),
 		"icon-all.svg",
 		CAIRO_DOCK_CATEGORY_ICON_SIZE);
-	g_signal_connect (G_OBJECT (pCategoryButton), "clicked", G_CALLBACK(on_click_all_button), NULL);
 	gtk_toolbar_insert (GTK_TOOLBAR (s_pToolBar) , pCategoryButton, -1);
 	pCategoryWidget->pCategoryButton = pCategoryButton;
+	gtk_toggle_tool_button_set_active (GTK_TOGGLE_TOOL_BUTTON (pCategoryButton), TRUE);
+	g_signal_connect (G_OBJECT (pCategoryButton), "clicked", G_CALLBACK(on_click_all_button), NULL);
 	
 	guint i;
 	for (i = 0; i < CAIRO_DOCK_NB_CATEGORY; i ++)
@@ -1678,8 +1769,8 @@ static GtkWidget *cairo_dock_build_main_ihm (const gchar *cConfFilePath, gboolea
 		gtk_container_set_border_width (GTK_CONTAINER (pCategoryWidget->pFrame), CAIRO_DOCK_FRAME_MARGIN);
 		gtk_frame_set_shadow_type (GTK_FRAME (pCategoryWidget->pFrame), GTK_SHADOW_OUT);
 		
-		pLabel = gtk_label_new (NULL);
-		cLabel = g_strdup_printf ("<span font_desc=\"Sans 12\"><b>%s</b></span>", gettext (s_cCategoriesDescription[2*i]));
+		GtkWidget *pLabel = gtk_label_new (NULL);
+		gchar *cLabel = g_strdup_printf ("<span size='large' weight='800'>%s</span>", gettext (s_cCategoriesDescription[2*i]));
 		gtk_label_set_markup (GTK_LABEL (pLabel), cLabel);
 		g_free (cLabel);
 		gtk_frame_set_label_widget (GTK_FRAME (pCategoryWidget->pFrame), pLabel);
@@ -1761,89 +1852,9 @@ static GtkWidget *cairo_dock_build_main_ihm (const gchar *cConfFilePath, gboolea
 	cairo_dock_foreach_module_in_alphabetical_order ((GCompareFunc) _cairo_dock_add_one_module_widget, cActiveModules);
 	g_free (cActiveModules);
 	
-	//\_____________ On ajoute le filtre.
-	// frame
-	GtkWidget *pFilterFrame = gtk_frame_new (NULL);
-	cLabel = g_strdup_printf ("<span font_desc=\"Sans 12\" color=\"#81728C\"><b><u>%s :</u></b></span>", _("Filter"));
-	GtkWidget *pFilterLabelContainer = gtk_hbox_new (FALSE, CAIRO_DOCK_FRAME_MARGIN);
-	GtkWidget *pImage = gtk_image_new_from_stock (GTK_STOCK_FIND, GTK_ICON_SIZE_MENU);
-	gtk_container_add (GTK_CONTAINER (pFilterLabelContainer), pImage);
-	
-	pLabel = gtk_label_new (NULL);
-	gtk_label_set_markup (GTK_LABEL (pLabel), cLabel);
-	g_free (cLabel);
-	gtk_container_add (GTK_CONTAINER (pFilterLabelContainer), pLabel);
-	
-	gtk_frame_set_label_widget (GTK_FRAME (pFilterFrame), pFilterLabelContainer);
-	gtk_container_set_border_width (GTK_CONTAINER (pFilterFrame), CAIRO_DOCK_FRAME_MARGIN);
-	gtk_frame_set_shadow_type (GTK_FRAME (pFilterFrame), GTK_SHADOW_OUT);
-	gtk_box_pack_start (GTK_BOX (pCategoriesVBox),
-		pFilterFrame,
-		FALSE,
-		FALSE,
-		0);
-	
-	GtkWidget *pOptionVBox = gtk_vbox_new (FALSE, CAIRO_DOCK_FRAME_MARGIN);
-	gtk_container_add (GTK_CONTAINER (pFilterFrame), pOptionVBox);
-	
-	// entree de texte
-	GtkWidget *pFilterBox = gtk_hbox_new (FALSE, CAIRO_DOCK_FRAME_MARGIN);
-	gtk_box_pack_start (GTK_BOX (pOptionVBox),
-		pFilterBox,
-		FALSE,
-		FALSE,
-		0);
-	s_pFilterEntry = gtk_entry_new ();
-	g_signal_connect (s_pFilterEntry, "activate", G_CALLBACK (on_activate_filter), NULL);
-	gtk_box_pack_start (GTK_BOX (pFilterBox),
-		s_pFilterEntry,
-		FALSE,
-		FALSE,
-		0);
-	
-	#if (GTK_MAJOR_VERSION > 2 || GTK_MINOR_VERSION >= 16)
-	gtk_entry_set_icon_activatable (GTK_ENTRY (s_pFilterEntry), GTK_ENTRY_ICON_SECONDARY, TRUE);
-	gtk_entry_set_icon_from_stock (GTK_ENTRY (s_pFilterEntry), GTK_ENTRY_ICON_SECONDARY, GTK_STOCK_CLEAR);
-	g_signal_connect (s_pFilterEntry, "icon-press", G_CALLBACK (on_clear_filter), NULL);
-	#endif
-	
-	// options
-	_reset_filter_state ();
-	
-	GtkWidget *pMenuBar = gtk_menu_bar_new ();
-	GtkWidget *pMenuItem = gtk_menu_item_new_with_label (_("Options"));
-	gtk_menu_shell_append (GTK_MENU_SHELL (pMenuBar), pMenuItem);
-	gtk_box_pack_start (GTK_BOX (pOptionVBox),
-		pMenuBar,
-		FALSE,
-		FALSE,
-		0);
-	GtkWidget *pMenu = gtk_menu_new ();
-	gtk_menu_item_set_submenu (GTK_MENU_ITEM (pMenuItem), pMenu);
-	
-	pMenuItem = gtk_check_menu_item_new_with_label (_("All words"));
-	gtk_menu_shell_append (GTK_MENU_SHELL (pMenu), pMenuItem);
-	g_signal_connect (pMenuItem, "toggled", G_CALLBACK (on_toggle_all_words), NULL);
-	
-	pMenuItem = gtk_check_menu_item_new_with_label (_("Highlighted words"));
-	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (pMenuItem), TRUE);
-	gtk_menu_shell_append (GTK_MENU_SHELL (pMenu), pMenuItem);
-	g_signal_connect (pMenuItem, "toggled", G_CALLBACK (on_toggle_highlight_words), NULL);
-	
-	pMenuItem = gtk_check_menu_item_new_with_label (_("Hide others"));
-	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (pMenuItem), TRUE);
-	gtk_menu_shell_append (GTK_MENU_SHELL (pMenu), pMenuItem);
-	g_signal_connect (pMenuItem, "toggled", G_CALLBACK (on_toggle_hide_others), NULL);
-	
-	pMenuItem = gtk_check_menu_item_new_with_label (_("Search in description"));
-	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (pMenuItem), TRUE);
-	gtk_menu_shell_append (GTK_MENU_SHELL (pMenu), pMenuItem);
-	g_signal_connect (pMenuItem, "toggled", G_CALLBACK (on_toggle_search_in_tooltip), NULL);
-	
 	//\_____________ On ajoute le cadre d'activation du module.
 	s_pGroupFrame = gtk_frame_new ("pouet");
-	gtk_container_set_border_width (GTK_CONTAINER (s_pGroupFrame), CAIRO_DOCK_FRAME_MARGIN);
-	gtk_frame_set_shadow_type (GTK_FRAME (s_pGroupFrame), GTK_SHADOW_OUT);
+	gtk_frame_set_shadow_type (GTK_FRAME (s_pGroupFrame), GTK_SHADOW_NONE);
 	gtk_box_pack_start (GTK_BOX (pCategoriesVBox),
 		s_pGroupFrame,
 		FALSE,
@@ -1855,18 +1866,18 @@ static GtkWidget *cairo_dock_build_main_ihm (const gchar *cConfFilePath, gboolea
 	gtk_widget_show_all (s_pActivateButton);
 	
 	//\_____________ On ajoute la zone de prevue.
-	GtkWidget *pInfoVBox = gtk_vbox_new (FALSE, CAIRO_DOCK_FRAME_MARGIN);
+	s_pPreviewBox = _gtk_vbox_new (CAIRO_DOCK_FRAME_MARGIN);
 	gtk_box_pack_start (GTK_BOX (pCategoriesVBox),
-		pInfoVBox,
+		s_pPreviewBox,
 		FALSE,
 		FALSE,
 		0);
 	
 	s_pPreviewImage = gtk_image_new_from_pixbuf (NULL);
-	gtk_container_add (GTK_CONTAINER (pInfoVBox), s_pPreviewImage);
+	gtk_container_add (GTK_CONTAINER (s_pPreviewBox), s_pPreviewImage);
 	
 	//\_____________ On ajoute les boutons.
-	GtkWidget *pButtonsHBox = gtk_hbox_new (FALSE, CAIRO_DOCK_FRAME_MARGIN);
+	GtkWidget *pButtonsHBox = _gtk_hbox_new (CAIRO_DOCK_FRAME_MARGIN);
 	gtk_box_pack_end (GTK_BOX (pVBox),
 		pButtonsHBox,
 		FALSE,
@@ -1912,12 +1923,12 @@ static GtkWidget *cairo_dock_build_main_ihm (const gchar *cConfFilePath, gboolea
 		FALSE,
 		0);
 	
-	gchar *cAdress = g_strdup_printf (CAIRO_DOCK_PLUGINS_EXTRAS_URL"/%d.%d.%d", g_iMajorVersion, g_iMinorVersion, g_iMicroVersion);
-	GtkWidget *pThirdPartyButton = gtk_link_button_new_with_label (cAdress, _("More applets"));
+	gchar *cLink = cairo_dock_get_third_party_applets_link ();
+	GtkWidget *pThirdPartyButton = gtk_link_button_new_with_label (cLink, _("More applets"));
 	gtk_widget_set_tooltip_text (pThirdPartyButton, _("Get more applets online !"));
-	pImage = gtk_image_new_from_stock (GTK_STOCK_ADD, GTK_ICON_SIZE_BUTTON);
+	GtkWidget *pImage = gtk_image_new_from_stock (GTK_STOCK_ADD, GTK_ICON_SIZE_BUTTON);
 	gtk_button_set_image (GTK_BUTTON (pThirdPartyButton), pImage);
-	g_free (cAdress);
+	g_free (cLink);
 	gtk_box_pack_start (GTK_BOX (pButtonsHBox),
 		pThirdPartyButton,
 		FALSE,
@@ -1926,7 +1937,9 @@ static GtkWidget *cairo_dock_build_main_ihm (const gchar *cConfFilePath, gboolea
 	
 	//\_____________ On ajoute la barre de status a la fin.
 	s_pStatusBar = gtk_statusbar_new ();
+	#if (GTK_MAJOR_VERSION < 3)
 	gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (s_pStatusBar), FALSE);
+	#endif
 	gtk_box_pack_start (GTK_BOX (pButtonsHBox),
 		s_pStatusBar,
 		TRUE,
@@ -1944,35 +1957,12 @@ static GtkWidget *cairo_dock_build_main_ihm (const gchar *cConfFilePath, gboolea
 	gtk_widget_hide (s_pApplyButton);
 	gtk_widget_hide (s_pOkButton);
 	gtk_widget_hide (s_pGroupFrame);
-	gtk_widget_hide (s_pPreviewImage);
+	gtk_widget_hide (s_pPreviewBox);
 	
-	if (bMaintenanceMode)
-	{
-		gtk_window_set_title (GTK_WINDOW (s_pMainWindow), _("< Maintenance mode >"));
-		GMainLoop *pBlockingLoop = g_main_loop_new (NULL, FALSE);
-		g_object_set_data (G_OBJECT (s_pMainWindow), "loop", pBlockingLoop);
-		g_signal_connect (s_pMainWindow,
-			"delete-event",
-			G_CALLBACK (on_delete_main_gui),
-			pBlockingLoop);
-		
-		cd_debug ("debut de boucle bloquante ...\n");
-		GDK_THREADS_LEAVE ();
-		g_main_loop_run (pBlockingLoop);
-		GDK_THREADS_ENTER ();
-		cd_debug ("fin de boucle bloquante\n");
-		
-		g_main_loop_unref (pBlockingLoop);
-	}
-	else
-	{
-		//gtk_window_set_title (GTK_WINDOW (s_pMainWindow), _("Cairo-Dock configuration"));
-		///cairo_dock_show_one_category (0);
-		g_signal_connect (G_OBJECT (s_pMainWindow),
-			"delete-event",
-			G_CALLBACK (on_delete_main_gui),
-			NULL);
-	}
+	g_signal_connect (G_OBJECT (s_pMainWindow),
+		"destroy",
+		G_CALLBACK (on_delete_main_gui),
+		NULL);
 	return s_pMainWindow;
 }
 
@@ -2298,19 +2288,29 @@ static GtkWidget *cairo_dock_present_group_widget (const gchar *cConfFilePath, C
 			GtkWidget *pAlign = NULL;
 			if (pGroupDescription->cIcon != NULL && *pGroupDescription->cIcon != '\0')
 			{
-				pLabelContainer = gtk_hbox_new (FALSE, CAIRO_DOCK_ICON_MARGIN);
+				pLabelContainer = _gtk_hbox_new (CAIRO_DOCK_ICON_MARGIN);
 				pAlign = gtk_alignment_new (0., 0.5, 0., 0.);
 				gtk_container_add (GTK_CONTAINER (pAlign), pLabelContainer);
 
 				GtkWidget *pImage = gtk_image_new ();
 				GdkPixbuf *pixbuf;
 				if (*pGroupDescription->cIcon != '/')
+				{
+					#if (GTK_MAJOR_VERSION < 3)
 					pixbuf = gtk_widget_render_icon (pImage,
 						pGroupDescription->cIcon ,
 						GTK_ICON_SIZE_BUTTON,
 						NULL);
+					#else
+					pixbuf = gtk_widget_render_icon_pixbuf (pImage,
+						pGroupDescription->cIcon ,
+						GTK_ICON_SIZE_BUTTON);
+					#endif
+				}
 				else
+				{
 					pixbuf = gdk_pixbuf_new_from_file_at_size (pGroupDescription->cIcon, CAIRO_DOCK_TAB_ICON_SIZE, CAIRO_DOCK_TAB_ICON_SIZE, NULL);
+				}
 				if (pixbuf != NULL)
 				{
 					gtk_image_set_from_pixbuf (GTK_IMAGE (pImage), pixbuf);
@@ -2338,7 +2338,7 @@ static GtkWidget *cairo_dock_present_group_widget (const gchar *cConfFilePath, C
 	
 	gtk_window_set_title (GTK_WINDOW (s_pMainWindow), pGroupDescription->cTitle);
 	
-	if (pInstance != NULL && pGroupDescription->load_custom_widget != NULL)
+	if (/**pInstance != NULL && */pGroupDescription->load_custom_widget != NULL)
 		pGroupDescription->load_custom_widget (pInstance, pKeyFile);
 	
 	cairo_dock_insert_extern_widget_in_gui (pWidget);  // devient le widget courant.
@@ -2347,7 +2347,7 @@ static GtkWidget *cairo_dock_present_group_widget (const gchar *cConfFilePath, C
 	
 	//\_______________ On met a jour la frame du groupe (label + check-button).
 	GtkWidget *pLabel = gtk_label_new (NULL);
-	gchar *cLabel = g_strdup_printf ("<span font_desc=\"Sans 12\" color=\"#81728C\"><u><b>%s</b></u></span>", pGroupDescription->cTitle);
+	gchar *cLabel = g_strdup_printf ("<span size='x-large' weight='800' color=\"#81728C\">%s</span>", pGroupDescription->cTitle);
 	gtk_label_set_markup (GTK_LABEL (pLabel), cLabel);
 	g_free (cLabel);
 	gtk_frame_set_label_widget (GTK_FRAME (s_pGroupFrame), pLabel);
@@ -2487,7 +2487,7 @@ static void cairo_dock_show_group (CairoDockGroupDescription *pGroupDescription)
 	else  // c'est un module, on recupere son fichier de conf en entier.
 	{
 		cConfFilePath = _get_valid_module_conf_file (pModule);
-		
+		pModuleInstance = (pModule->pInstancesList != NULL ? pModule->pInstancesList->data : NULL);
 		bSingleGroup = FALSE;
 	}
 	
@@ -2520,7 +2520,7 @@ static void cairo_dock_apply_current_filter (gchar **pKeyWords, gboolean bAllWor
 
 static GtkWidget * show_main_gui (void)
 {
-	GtkWidget *pWindow = cairo_dock_build_main_ihm (g_cConfFile, FALSE);
+	GtkWidget *pWindow = cairo_dock_build_main_ihm (g_cConfFile);
 	return pWindow;
 }
 
@@ -2533,7 +2533,7 @@ static void show_module_gui (const gchar *cModuleName)
 	CairoDockGroupDescription *pGroupDescription = cairo_dock_find_module_description (cModuleName);
 	if (pGroupDescription == NULL)
 	{
-		cairo_dock_build_main_ihm (g_cConfFile, FALSE);
+		cairo_dock_build_main_ihm (g_cConfFile);
 		pGroupDescription = cairo_dock_find_module_description (cModuleName);
 		g_return_if_fail (pGroupDescription != NULL);
 	}
@@ -2549,7 +2549,7 @@ static void show_module_instance_gui (CairoDockModuleInstance *pModuleInstance,
 			gtk_notebook_get_current_page (GTK_NOTEBOOK (s_pCurrentGroupWidget))) :
 		-1);
 	
-	cairo_dock_build_main_ihm (g_cConfFile, FALSE);
+	cairo_dock_build_main_ihm (g_cConfFile);
 	cairo_dock_present_module_instance_gui (pModuleInstance);
 	cairo_dock_toggle_category_button (pModuleInstance->pModule->pVisitCard->iCategory);  // on active la categorie du module.
 	
@@ -2703,6 +2703,24 @@ static void update_modules_list (void)
 	gtk_widget_show_all (s_pMainWindow);
 }
 
+static void update_shortkeys (void)
+{
+	if (s_pMainWindow == NULL
+	|| s_pCurrentGroup == NULL
+	|| s_pCurrentGroup->cGroupName == NULL
+	|| strcmp (s_pCurrentGroup->cGroupName, "Shortkeys") != 0)  // the Shortkeys widget is not currently displayed => nothing to do.
+		return ;
+	
+	CairoDockGroupKeyWidget *pGroupKeyWidget = cairo_dock_gui_find_group_key_widget_in_list (s_pCurrentWidgetList, "Shortkeys", "shortkeys");
+	g_return_if_fail (pGroupKeyWidget != NULL && pGroupKeyWidget->pSubWidgetList != NULL);
+	
+	GtkWidget *pOneWidget = pGroupKeyWidget->pSubWidgetList->data;
+	GtkTreeModel *pModel = gtk_tree_view_get_model (GTK_TREE_VIEW (pOneWidget));
+	g_return_if_fail (pModel != NULL);
+	gtk_list_store_clear (GTK_LIST_STORE (pModel));
+	cd_keybinder_foreach ((GFunc) cairo_dock_add_shortkey_to_model, pModel);
+}
+
 static void set_status_message_on_gui (const gchar *cMessage)
 {
 	if (s_pStatusBar == NULL)
@@ -2746,6 +2764,7 @@ void cairo_dock_register_main_gui_backend (void)
 	pBackend->update_desklet_visibility_params 	= update_desklet_visibility_params;
 	pBackend->update_module_instance_container 	= update_module_instance_container;
 	pBackend->update_modules_list 				= update_modules_list;
+	pBackend->update_shortkeys 					= update_shortkeys;
 	pBackend->bCanManageThemes 					= FALSE;
 	pBackend->cDisplayedName 					= _("Simple Mode");
 	pBackend->cTooltip 							= NULL;
