@@ -41,6 +41,7 @@
 #include "cairo-dock-themes-manager.h"
 #include "cairo-dock-applications-manager.h"
 #include "cairo-dock-backends-manager.h"
+#include "cairo-dock-keybinder.h"
 #include "cairo-dock-X-manager.h"
 #include "cairo-dock-gui-manager.h"
 #include "cairo-dock-gui-factory.h"
@@ -54,21 +55,14 @@
 #define CAIRO_DOCK_SIMPLE_PANEL_WIDTH 1024
 #define CAIRO_DOCK_SIMPLE_PANEL_HEIGHT 700
 #define CAIRO_DOCK_SIMPLE_CONF_FILE "cairo-dock-simple.conf"
-#define ICON_HUGE 60
-#define ICON_BIG 56
-#define ICON_MEDIUM 48
-#define ICON_SMALL 42
-#define ICON_TINY 36
-#define CAIRO_DOCK_PLUGINS_EXTRAS_URL "http://extras.glx-dock.org"
-#define CAIRO_DOCK_THEMES_PAGE 3
+#define CAIRO_DOCK_SHORTKEY_PAGE 2
+#define CAIRO_DOCK_ADDONS_PAGE 3
+#define CAIRO_DOCK_THEMES_PAGE 4
 
 static GtkWidget *s_pSimpleConfigWindow = NULL;
 static const  gchar *s_cCurrentModuleName = NULL;
-// GSList *s_pCurrentWidgetList;  // liste des widgets du main panel.
-// GSList *s_pCurrentModuleWidgetList;  // liste des widgets du module courant.
 static int s_iIconSize;
 static int s_iTaskbarType;
-static gboolean s_bSeparateIcons;
 static gchar *s_cHoverAnim = NULL;
 static gchar *s_cHoverEffect = NULL;
 static gchar *s_cClickAnim = NULL;
@@ -80,15 +74,8 @@ extern gchar *g_cConfFile;
 extern gchar *g_cCurrentThemePath;
 extern gchar *g_cCairoDockDataDir;
 extern gboolean g_bUseOpenGL;
-extern int g_iMajorVersion, g_iMinorVersion, g_iMicroVersion;
 extern CairoDockDesktopGeometry g_desktopGeometry;
 
-/*#define cd_reload(module_name) do {\
-	pInternalModule = cairo_dock_find_internal_module_from_name (module_name);\
-	if (pInternalModule != NULL) \
-		cairo_dock_reload_internal_module_from_keyfile (pInternalModule, pKeyFile);\
-	} while (0)*/
-
 #define cd_reload(module_name) do {\
 	pManager = gldi_get_manager (module_name);\
 	if (pManager != NULL)\
@@ -192,7 +179,8 @@ static gchar * _make_simple_conf_file (void)
 	//\_____________ On actualise le fichier de conf simple.
 	// on cree le fichier au besoin, et on l'ouvre.
 	gchar *cConfFilePath = g_strdup_printf ("%s/%s", g_cCurrentThemePath, CAIRO_DOCK_SIMPLE_CONF_FILE);
-	///if (! g_file_test (cConfFilePath, G_FILE_TEST_EXISTS))  // since we remove the opengl effects params, we need to overwrite the file each time. TODO: fix this ...
+	/// at the moment, always copy the conf file, because when applied, it's modified with the themes keys. this will not occur with the next version ;-)
+	///if (! g_file_test (cConfFilePath, G_FILE_TEST_EXISTS))
 	{
 		cairo_dock_copy_file (CAIRO_DOCK_SHARE_DATA_DIR"/"CAIRO_DOCK_SIMPLE_CONF_FILE, cConfFilePath);
 	}
@@ -202,7 +190,6 @@ static gchar * _make_simple_conf_file (void)
 	
 	if (cairo_dock_conf_file_needs_update (pSimpleKeyFile, CAIRO_DOCK_VERSION))
 	{
-		///cairo_dock_flush_conf_file (pSimpleKeyFile, cConfFilePath, CAIRO_DOCK_SHARE_DATA_DIR, CAIRO_DOCK_SIMPLE_CONF_FILE);
 		cairo_dock_upgrade_conf_file (cConfFilePath, pSimpleKeyFile, CAIRO_DOCK_SHARE_DATA_DIR"/"CAIRO_DOCK_SIMPLE_CONF_FILE);
 		
 		g_key_file_free (pSimpleKeyFile);
@@ -230,6 +217,8 @@ static gchar * _make_simple_conf_file (void)
 		iTaskbarType = 3;
 	s_iTaskbarType = iTaskbarType;
 	g_key_file_set_integer (pSimpleKeyFile, "Behavior", "taskbar", iTaskbarType);
+
+	g_key_file_set_integer (pSimpleKeyFile, "Behavior", "place icons", myTaskbarParam.iIconPlacement);
 	
 	// animations
 	CairoDockModule *pModule;
@@ -312,15 +301,15 @@ static gchar * _make_simple_conf_file (void)
 	
 	if (g_bUseOpenGL)
 		g_key_file_set_string_list (pSimpleKeyFile, "Behavior", "anim_hover", cOnMouseHover, 2);
-	else
+	/**else
 	{
 		g_key_file_remove_comment (pSimpleKeyFile, "Behavior", "anim_hover", NULL);
 		g_key_file_remove_key (pSimpleKeyFile, "Behavior", "anim_hover", NULL);
-	}
+	}*/
 	g_key_file_set_string_list (pSimpleKeyFile, "Behavior", "anim_click", cOnClick, 2);
 	if (g_bUseOpenGL)
 		g_key_file_set_integer (pSimpleKeyFile, "Behavior", "anim_disappear", s_iEffectOnDisappearance);
-	else
+	/**else
 	{
 		g_key_file_remove_comment (pSimpleKeyFile, "Behavior", "anim_disappear", NULL);
 		g_key_file_remove_key (pSimpleKeyFile, "Behavior", "anim_disappear", NULL);
@@ -336,43 +325,25 @@ static gchar * _make_simple_conf_file (void)
 			g_key_file_set_comment (pSimpleKeyFile, "Behavior", "anim_disappear", cComment, NULL);
 			g_free (cComment);
 		}
-	}
+	}*/
 	
 	// apparence
 	g_key_file_set_string (pSimpleKeyFile, "Appearance", "default icon directory", myIconsParam.cIconTheme);
 	
-	int iIconSize;
-	int s = myIconsParam.tIconAuthorizedWidth[CAIRO_DOCK_LAUNCHER];
-	if (s <= ICON_TINY+2)  // icones toutes petites.
-		iIconSize = 0;
-	else if (s >= ICON_HUGE-2)  // icones tres grandes.
-		iIconSize = 4;
-	else if (s <= ICON_MEDIUM)
-	{
-		if (myIconsParam.fAmplitude >= 2 || s <= ICON_SMALL)  // icones petites.
-			iIconSize = 1;
-		else
-			iIconSize = 2;  // moyennes.
-	}
-	else  // grandes.
-		iIconSize = 3;
+	int iIconSize = cairo_dock_convert_icon_size_to_enum (myIconsParam.iIconWidth);
+	iIconSize --;  // skip the "default" enum
 	
 	g_key_file_set_integer (pSimpleKeyFile, "Appearance", "icon size", iIconSize);
 	s_iIconSize = iIconSize;
 	
-	g_key_file_set_boolean (pSimpleKeyFile, "Appearance", "separate icons", myIconsParam.iSeparateIcons);
-	s_bSeparateIcons = myIconsParam.iSeparateIcons;
-	
-	g_key_file_set_integer_list (pSimpleKeyFile, "Appearance", "icon's type order", myIconsParam.iIconsTypesList, 3);
-	
 	g_key_file_set_string (pSimpleKeyFile, "Appearance", "main dock view", myBackendsParam.cMainDockDefaultRendererName);
 	
 	g_key_file_set_string (pSimpleKeyFile, "Appearance", "sub-dock view", myBackendsParam.cSubDockDefaultRendererName);
 	
 	// applets
-	gchar *cAdress = g_strdup_printf (CAIRO_DOCK_PLUGINS_EXTRAS_URL"/%d.%d.%d", g_iMajorVersion, g_iMinorVersion, g_iMicroVersion);
-	g_key_file_set_string (pSimpleKeyFile, "Add-ons", "third party", cAdress);
-	g_free (cAdress);
+	gchar *cLink = cairo_dock_get_third_party_applets_link ();
+	g_key_file_set_string (pSimpleKeyFile, "Add-ons", "third party", cLink);
+	g_free (cLink);
 	
 	cairo_dock_write_keys_to_file (pSimpleKeyFile, cConfFilePath);
 	
@@ -463,12 +434,17 @@ static gboolean on_apply_config_simple (gpointer data)
 	GList *children = gtk_container_get_children (GTK_CONTAINER (pMainVBox));
 	GtkWidget *pGroupWidget = children->data;
 	g_list_free (children);
-	if (gtk_notebook_get_current_page (GTK_NOTEBOOK (pGroupWidget)) == CAIRO_DOCK_THEMES_PAGE)
+	int iNumPage = gtk_notebook_get_current_page (GTK_NOTEBOOK (pGroupWidget));
+	if (iNumPage == CAIRO_DOCK_THEMES_PAGE)
 	{
 		CairoDockGroupKeyWidget *myWidget = cairo_dock_gui_find_group_key_widget (s_pSimpleConfigWindow, "Themes", "notebook");
 		g_return_val_if_fail (myWidget != NULL && myWidget->pSubWidgetList != NULL, TRUE);
 		return on_apply_theme_simple (myWidget->pSubWidgetList->data, data);
 	}
+	else if (iNumPage == CAIRO_DOCK_SHORTKEY_PAGE || iNumPage == CAIRO_DOCK_ADDONS_PAGE)
+	{
+		return TRUE;
+	}
 	
 	//\_____________ On actualise le fichier de conf principal.
 	// on ouvre les 2 fichiers (l'un en lecture, l'autre en ecriture).
@@ -496,7 +472,6 @@ static gboolean on_apply_config_simple (gpointer data)
 	int iTaskbarType = g_key_file_get_integer (pSimpleKeyFile, "Behavior", "taskbar", NULL);
 	if (iTaskbarType != s_iTaskbarType)
 	{
-		cd_debug ("s_iTaskbarType : %d / %d", s_iTaskbarType, iTaskbarType);
 		gboolean bShowAppli = TRUE, bHideVisible, bCurrentDesktopOnly, bMixLauncherAppli, bGroupAppliByClass;
 		switch (iTaskbarType)
 		{
@@ -530,17 +505,20 @@ static gboolean on_apply_config_simple (gpointer data)
 			g_key_file_set_boolean (pKeyFile, "TaskBar", "current desktop only", bCurrentDesktopOnly);
 			g_key_file_set_boolean (pKeyFile, "TaskBar", "mix launcher appli", bMixLauncherAppli);
 			g_key_file_set_boolean (pKeyFile, "TaskBar", "group by class", bGroupAppliByClass);
-			cd_debug (" taskbar : %d; %d; %d; %d", bHideVisible, bCurrentDesktopOnly, bMixLauncherAppli, bGroupAppliByClass);
 		}
 		s_iTaskbarType = iTaskbarType;
 	}
+
+	int iPlaceIcons = g_key_file_get_integer (pSimpleKeyFile, "Behavior", "place icons", NULL);
+	g_key_file_set_integer (pKeyFile, "TaskBar", "place icons", iPlaceIcons);
 	
 	// animations
 	gsize length;
 	gchar **cOnMouseHover = g_key_file_get_string_list (pSimpleKeyFile, "Behavior", "anim_hover", &length, NULL);
 	gchar **cOnClick = g_key_file_get_string_list (pSimpleKeyFile, "Behavior", "anim_click", &length, NULL);
 	int iEffectOnDisappearance = -1;
-	if (g_key_file_has_key (pSimpleKeyFile, "Behavior", "anim_disappear", NULL))
+	///if (g_key_file_has_key (pSimpleKeyFile, "Behavior", "anim_disappear", NULL))
+	if (g_bUseOpenGL)
 		iEffectOnDisappearance = g_key_file_get_integer (pSimpleKeyFile, "Behavior", "anim_disappear", NULL);
 	
 	CairoDockModule *pModule;
@@ -639,65 +617,19 @@ static gboolean on_apply_config_simple (gpointer data)
 	{
 		int iLauncherSize, iIconGap;
 		double fMaxScale, fReflectSize;
-		switch (iIconSize)
-		{
-			case 0:  // tres petites
-				iLauncherSize = ICON_TINY;
-				fMaxScale = 2.2;
-				iIconGap = 5;
-				fReflectSize = .4;
-			break;
-			case 1:  // petites
-				iLauncherSize = ICON_SMALL;
-				fMaxScale = 2.;
-				iIconGap = 4;
-				fReflectSize = .4;
-			break;
-			case 2:  // moyennes
-				iLauncherSize = ICON_MEDIUM;
-				fMaxScale = 1.7;
-				iIconGap = 2;
-				fReflectSize = .5;
-			break;
-			case 3:  // grandes
-			default:
-				iLauncherSize = ICON_BIG;
-				fMaxScale = 1.5;
-				iIconGap = 2;
-				fReflectSize = .6;
-			break;
-			case 4:  // tres grandes
-				iLauncherSize = ICON_HUGE;
-				fMaxScale = 1.3;
-				iIconGap = 2;
-				fReflectSize = .6;
-			break;
-			
-		}
+		iLauncherSize = cairo_dock_convert_icon_size_to_pixels (iIconSize+1, &fMaxScale, &fReflectSize, &iIconGap);  // +1 to skip the "default" enum
+		
 		gint tab[2] = {iLauncherSize, iLauncherSize};
 		g_key_file_set_integer_list (pKeyFile, "Icons", "launcher size", tab, 2);
-		g_key_file_set_integer_list (pKeyFile, "Icons", "appli size", tab, 2);
-		g_key_file_set_integer_list (pKeyFile, "Icons", "applet size", tab, 2);
-		tab[0] = myIconsParam.tIconAuthorizedWidth[CAIRO_DOCK_SEPARATOR12];
+		tab[0] = myIconsParam.iSeparatorWidth;
 		g_key_file_set_integer_list (pKeyFile, "Icons", "separator size", tab, 2);
-		
 		g_key_file_set_double (pKeyFile, "Icons", "zoom max", fMaxScale);
 		g_key_file_set_double (pKeyFile, "Icons", "field depth", fReflectSize);
 		g_key_file_set_integer (pKeyFile, "Icons", "icon gap", iIconGap);
+		
 		s_iIconSize = iIconSize;
 	}
 	
-	gboolean bSeparateIcons = g_key_file_get_boolean (pSimpleKeyFile, "Appearance", "separate icons", NULL);
-	if (bSeparateIcons != s_bSeparateIcons)
-	{
-		g_key_file_set_integer (pKeyFile, "Icons", "separate_icons", (bSeparateIcons ? 3 : 0));
-		s_bSeparateIcons = bSeparateIcons;
-	}
-	
-	gchar *cIconOrder = g_key_file_get_string (pSimpleKeyFile, "Appearance", "icon's type order", NULL);
-	g_key_file_set_string (pKeyFile, "Icons", "icon's type order", cIconOrder);
-	g_free (cIconOrder);
-	
 	gchar *cMainDockDefaultRendererName = g_key_file_get_string (pSimpleKeyFile, "Appearance", "main dock view", NULL);
 	g_key_file_set_string (pKeyFile, "Views", "main dock view", cMainDockDefaultRendererName);
 	g_free (cMainDockDefaultRendererName);
@@ -769,8 +701,8 @@ static void _make_double_anim_widget (GtkWidget *pSimpleConfigWindow, GKeyFile *
 	
 	gsize length = 0;
 	gchar **cValues = g_key_file_get_string_list (pKeyFile, cGroupName, cKeyName, &length, NULL);
-	
-	GtkWidget *box = gtk_hbox_new (FALSE, CAIRO_DOCK_GUI_MARGIN);
+
+	GtkWidget *box = _gtk_hbox_new (CAIRO_DOCK_GUI_MARGIN);
 	gtk_box_pack_end (GTK_BOX (myWidget->pKeyBox), box, FALSE, FALSE, 0);
 	
 	GtkWidget *pLabel = gtk_label_new (_("Animation:"));
@@ -786,7 +718,11 @@ static void _make_double_anim_widget (GtkWidget *pSimpleConfigWindow, GKeyFile *
 	
 	if (g_bUseOpenGL)
 	{
+		#if (GTK_MAJOR_VERSION < 3)
 		pLabel = gtk_vseparator_new ();
+		#else
+		pLabel = gtk_separator_new (GTK_ORIENTATION_VERTICAL);
+		#endif
 		gtk_widget_set_size_request (pLabel, 20, 1);
 		gtk_box_pack_start (GTK_BOX (box), pLabel, FALSE, FALSE, 0);
 		
@@ -828,7 +764,7 @@ static void _make_theme_manager_widget (GtkWidget *pSimpleConfigWindow)
 		gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (pScrolledWindow), GTK_POLICY_NEVER, GTK_POLICY_NEVER);
 	}
 	
-	gtk_widget_set (pThemeNotebook, "height-request", MIN (CAIRO_DOCK_SIMPLE_PANEL_HEIGHT, g_desktopGeometry.iXScreenHeight[CAIRO_DOCK_HORIZONTAL] - 100), NULL);  // sinon le notebook est tout petit :-/
+	g_object_set (pThemeNotebook, "height-request", MIN (CAIRO_DOCK_SIMPLE_PANEL_HEIGHT, g_desktopGeometry.iXScreenHeight[CAIRO_DOCK_HORIZONTAL] - 100), NULL);  // sinon le notebook est tout petit :-/
 	gtk_box_pack_start (GTK_BOX (myWidget->pKeyBox),
 		pThemeNotebook,
 		TRUE,
@@ -851,7 +787,7 @@ static void _make_modules_widget (GtkWidget *pSimpleConfigWindow)
 	
 	//\_____________ On l'ajoute a la fenetre.
 	GtkWidget *pScrolledWindow = gtk_scrolled_window_new (NULL, NULL);
-	gtk_widget_set (pScrolledWindow, "height-request", MIN (2*CAIRO_DOCK_PREVIEW_HEIGHT, g_desktopGeometry.iXScreenHeight[CAIRO_DOCK_HORIZONTAL] - 175), NULL);
+	g_object_set (pScrolledWindow, "height-request", MIN (2*CAIRO_DOCK_PREVIEW_HEIGHT, g_desktopGeometry.iXScreenHeight[CAIRO_DOCK_HORIZONTAL] - 175), NULL);
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (pScrolledWindow), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
 	gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (pScrolledWindow), pOneWidget);
 	myWidget->pSubWidgetList = g_slist_append (myWidget->pSubWidgetList, pOneWidget);  // on le met dans la liste, non pas pour recuperer sa valeur, mais pour pouvoir y acceder facilement plus tard.
@@ -864,6 +800,23 @@ static void _make_modules_widget (GtkWidget *pSimpleConfigWindow)
 	gtk_box_pack_start (GTK_BOX (myWidget->pKeyBox), pPreviewBox, FALSE, FALSE, 0);
 	g_free (cDefaultMessage);
 }
+static void _make_shortkeys_widget (GtkWidget *pSimpleConfigWindow)
+{
+	//\_____________ On recupere notre emplacement perso dans la fenetre.
+	CairoDockGroupKeyWidget *myWidget = cairo_dock_gui_find_group_key_widget (pSimpleConfigWindow, "Shortkeys", "shortkeys");
+	g_return_if_fail (myWidget != NULL);
+	
+	//\_____________ On construit le tree-view.
+	GtkWidget *pOneWidget = cairo_dock_build_shortkeys_widget ();
+	
+	//\_____________ On l'ajoute a la fenetre.
+	GtkWidget *pScrolledWindow = gtk_scrolled_window_new (NULL, NULL);
+	g_object_set (pScrolledWindow, "height-request", MIN (2*CAIRO_DOCK_PREVIEW_HEIGHT, g_desktopGeometry.iXScreenHeight[CAIRO_DOCK_HORIZONTAL] - 175), NULL);
+	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (pScrolledWindow), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+	gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (pScrolledWindow), pOneWidget);
+	myWidget->pSubWidgetList = g_slist_append (myWidget->pSubWidgetList, pOneWidget);  // on le met dans la liste, non pas pour recuperer sa valeur, mais pour pouvoir y acceder facilement plus tard.
+	gtk_box_pack_start (GTK_BOX (myWidget->pKeyBox), pScrolledWindow, FALSE, FALSE, 0);
+}
 static void _make_widgets (GtkWidget *pSimpleConfigWindow, GKeyFile *pKeyFile)
 {
 	//\_____________ On ajoute le widget des animations au survol.
@@ -874,6 +827,8 @@ static void _make_widgets (GtkWidget *pSimpleConfigWindow, GKeyFile *pKeyFile)
 	_make_theme_manager_widget (pSimpleConfigWindow);
 	//\_____________ On ajoute le widget des modules.
 	_make_modules_widget (pSimpleConfigWindow);
+	//\_____________ On ajoute le widget des shortkeys.
+	_make_shortkeys_widget (pSimpleConfigWindow);
 	//\_____________ On montre la page des themes si necessaire.
 	if (s_bShowThemePage)
 	{
@@ -1030,10 +985,27 @@ static void update_modules_list (void)
 	GtkTreeModel *pModel = gtk_tree_view_get_model (GTK_TREE_VIEW (pOneWidget));
 	g_return_if_fail (pModel != NULL);
 	gtk_list_store_clear (GTK_LIST_STORE (pModel));
+	
 	cairo_dock_foreach_module ((GHRFunc) cairo_dock_add_module_to_modele, pModel);
 }
 
 
+static void update_shortkeys (void)
+{
+	if (s_pSimpleConfigWindow == NULL)
+		return;
+	
+	CairoDockGroupKeyWidget *pGroupKeyWidget = cairo_dock_gui_find_group_key_widget (s_pSimpleConfigWindow, "Shortkeys", "shortkeys");
+	g_return_if_fail (pGroupKeyWidget != NULL && pGroupKeyWidget->pSubWidgetList != NULL);
+	
+	GtkWidget *pOneWidget = pGroupKeyWidget->pSubWidgetList->data;
+	GtkTreeModel *pModel = gtk_tree_view_get_model (GTK_TREE_VIEW (pOneWidget));
+	g_return_if_fail (pModel != NULL);
+	gtk_list_store_clear (GTK_LIST_STORE (pModel));
+	cd_keybinder_foreach ((GFunc) cairo_dock_add_shortkey_to_model, pModel);
+}
+
+
 static void set_status_message_on_gui (const gchar *cMessage)
 {
 	GtkWidget *pStatusBar = NULL;
@@ -1057,13 +1029,13 @@ void cairo_dock_register_simple_gui_backend (void)
 	
 	pBackend->show_main_gui 				= show_main_gui;
 	pBackend->show_module_gui 				= show_module_gui;
-	//pBackend->show_module_instance_gui 		= show_module_instance_gui;
 	pBackend->close_gui 					= close_gui;
 	pBackend->update_module_state 			= update_module_state;
 	pBackend->update_module_instance_container = cairo_dock_gui_items_update_module_instance_container;
 	pBackend->update_desklet_params 		= cairo_dock_gui_items_update_desklet_params;
 	pBackend->update_desklet_visibility_params = cairo_dock_update_desklet_visibility_params;
 	pBackend->update_modules_list 			= update_modules_list;
+	pBackend->update_shortkeys 				= update_shortkeys;
 	pBackend->bCanManageThemes 				= TRUE;
 	pBackend->cDisplayedName 				= _("Advanced Mode");
 	pBackend->cTooltip 						= _("The advanced mode lets you tweak every single parameter of the dock. It is a powerful tool to customise your current theme.");
