@@ -41,7 +41,7 @@
 #include "cairo-dock-file-manager.h"  // cairo_dock_copy_file
 #include "cairo-dock-module-factory.h"
 #include "cairo-dock-log.h"
-#include "cairo-dock-config.h"
+#include "cairo-dock-keyfile-utilities.h"
 #include "cairo-dock-dock-manager.h"
 #include "cairo-dock-keybinder.h"
 #include "cairo-dock-animations.h"
@@ -154,6 +154,19 @@ static void _cairo_dock_close_all_in_class_subdock (Icon *icon)
 	}
 }
 
+static void _show_all_windows (GList *pIcons)
+{
+	Icon *pIcon;
+	GList *ic;
+	for (ic = pIcons; ic != NULL; ic = ic->next)
+	{
+		pIcon = ic->data;
+		if (pIcon->Xid != 0 && pIcon->bIsHidden)  // a window is hidden...
+		{
+			cairo_dock_show_xwindow (pIcon->Xid);
+		}
+	}
+}
 
 static gboolean _launch_icon_command (Icon *icon, CairoDock *pDock)
 {
@@ -170,7 +183,7 @@ static gboolean _launch_icon_command (Icon *icon, CairoDock *pDock)
 	gboolean bSuccess = FALSE;
 	if (*icon->cCommand == '<')  // shortkey
 	{
-		bSuccess = cairo_dock_simulate_key_sequence (icon->cCommand);
+		bSuccess = cairo_dock_trigger_shortkey (icon->cCommand);
 		if (!bSuccess)
 			bSuccess = cairo_dock_launch_command_full (icon->cCommand, icon->cWorkingDirectory);
 	}
@@ -178,7 +191,7 @@ static gboolean _launch_icon_command (Icon *icon, CairoDock *pDock)
 	{
 		bSuccess = cairo_dock_launch_command_full (icon->cCommand, icon->cWorkingDirectory);
 		if (! bSuccess)
-			bSuccess = cairo_dock_simulate_key_sequence (icon->cCommand);
+			bSuccess = cairo_dock_trigger_shortkey (icon->cCommand);
 	}
 	if (! bSuccess)
 	{
@@ -188,7 +201,6 @@ static gboolean _launch_icon_command (Icon *icon, CairoDock *pDock)
 }
 gboolean cairo_dock_notification_click_icon (gpointer pUserData, Icon *icon, CairoContainer *pContainer, guint iButtonState)
 {
-	//g_print ("+ %s (%s, %d)\n", __func__, icon ? icon->cCommand : "no icon", CAIRO_DOCK_ICON_TYPE_IS_LAUNCHER (icon));
 	if (icon == NULL || ! CAIRO_DOCK_IS_DOCK (pContainer))
 		return CAIRO_DOCK_LET_PASS_NOTIFICATION;
 	CairoDock *pDock = CAIRO_DOCK (pContainer);
@@ -208,19 +220,20 @@ gboolean cairo_dock_notification_click_icon (gpointer pUserData, Icon *icon, Cai
 	// scale on an icon holding a class sub-dock.
 	if (CAIRO_DOCK_IS_MULTI_APPLI(icon))
 	{
-		if ( (!myDocksParam.bShowSubDockOnClick  // if sub-docks are shown on mouse over
-		|| GTK_WIDGET_VISIBLE (icon->pSubDock->container.pWidget)  // or this sub-dock is already visible
-		|| myTaskbarParam.bPresentClassOnClick)  // or we explicitely allowed to skip the sub-dock
+		if (myTaskbarParam.bPresentClassOnClick // if we want to use this feature
+		&& (!myDocksParam.bShowSubDockOnClick  // if sub-docks are shown on mouse over
+			|| gldi_container_is_visible (CAIRO_CONTAINER (icon->pSubDock)))  // or this sub-dock is already visible
 		&& cairo_dock_wm_present_class (icon->cClass)) // we use the scale plugin if it's possible
 		{
-			if (icon->pSubDock)  // in case the dock is visible or about to be visible, hide it, as it would confuse the user to have both.
-				cairo_dock_emit_leave_signal (CAIRO_CONTAINER (icon->pSubDock));
+			_show_all_windows (icon->pSubDock->icons); // show all windows
+			// in case the dock is visible or about to be visible, hide it, as it would confuse the user to have both.
+			cairo_dock_emit_leave_signal (CAIRO_CONTAINER (icon->pSubDock));
 			return CAIRO_DOCK_INTERCEPT_NOTIFICATION;
 		}
 	}
 	
 	// else handle sub-docks showing on click, applis and launchers (not applets).
-	if (icon->pSubDock != NULL && (myDocksParam.bShowSubDockOnClick || !GTK_WIDGET_VISIBLE (icon->pSubDock->container.pWidget)))  // icon pointing to a sub-dock with either "sub-dock activation on click" option enabled, or sub-dock not visible -> open the sub-dock
+	if (icon->pSubDock != NULL && (myDocksParam.bShowSubDockOnClick || !gldi_container_is_visible (CAIRO_CONTAINER (icon->pSubDock))))  // icon pointing to a sub-dock with either "sub-dock activation on click" option enabled, or sub-dock not visible -> open the sub-dock
 	{
 		cairo_dock_show_subdock (icon, pDock);
 		return CAIRO_DOCK_INTERCEPT_NOTIFICATION;
@@ -275,7 +288,7 @@ gboolean cairo_dock_notification_middle_click_icon (gpointer pUserData, Icon *ic
 			case 3:  // launch new
 				if (icon->cCommand != NULL)
 				{
-					cairo_dock_notify_on_object (&myContainersMgr, NOTIFICATION_CLICK_ICON, icon, pDock, GDK_SHIFT_MASK);  // on emule un shift+clic gauche .
+					///cairo_dock_notify_on_object (&myContainersMgr, NOTIFICATION_CLICK_ICON, icon, pDock, GDK_SHIFT_MASK);  // on emule un shift+clic gauche .
 					cairo_dock_notify_on_object (CAIRO_CONTAINER (pDock), NOTIFICATION_CLICK_ICON, icon, pDock, GDK_SHIFT_MASK);  // on emule un shift+clic gauche .
 				}
 			break;
@@ -295,7 +308,7 @@ gboolean cairo_dock_notification_middle_click_icon (gpointer pUserData, Icon *ic
 			case 3:  // launch new
 				if (icon->cCommand != NULL)
 				{
-					cairo_dock_notify_on_object (&myContainersMgr, NOTIFICATION_CLICK_ICON, icon, pDock, GDK_SHIFT_MASK);  // on emule un shift+clic gauche .
+					///cairo_dock_notify_on_object (&myContainersMgr, NOTIFICATION_CLICK_ICON, icon, pDock, GDK_SHIFT_MASK);  // on emule un shift+clic gauche .
 					cairo_dock_notify_on_object (CAIRO_CONTAINER (pDock), NOTIFICATION_CLICK_ICON, icon, pDock, GDK_SHIFT_MASK);  // on emule un shift+clic gauche .
 				}
 			break;
@@ -334,9 +347,7 @@ gboolean cairo_dock_notification_drop_data (gpointer pUserData, const gchar *cRe
 	if (g_str_has_suffix (cReceivedData, ".desktop"))  // .desktop -> add a new launcher if dropped on or amongst launchers. 
 	{
 		cd_debug (" dropped a .desktop");
-		if ((myIconsParam.iSeparateIcons == 1 || myIconsParam.iSeparateIcons == 3) && CAIRO_DOCK_ICON_TYPE_IS_APPLI (icon))
-			return CAIRO_DOCK_LET_PASS_NOTIFICATION;
-		if ((myIconsParam.iSeparateIcons == 2 || myIconsParam.iSeparateIcons == 3) && CAIRO_DOCK_ICON_TYPE_IS_APPLET (icon))
+		if (! myTaskbarParam.bMixLauncherAppli && CAIRO_DOCK_ICON_TYPE_IS_APPLI (icon))
 			return CAIRO_DOCK_LET_PASS_NOTIFICATION;
 		cd_debug (" add it");
 		if (fOrder == CAIRO_DOCK_LAST_ORDER && CAIRO_DOCK_ICON_TYPE_IS_CONTAINER (icon) && icon->pSubDock != NULL)  // drop onto a container icon.
@@ -522,6 +533,8 @@ gboolean cairo_dock_notification_module_activated (gpointer pUserData, const gch
 	//g_print ("module %s (de)activated (%d)\n", cModuleName, bActivated);
 	cairo_dock_gui_trigger_update_module_state (cModuleName);
 	
+	cairo_dock_gui_trigger_reload_items ();  // for plug-ins that don't have an applet, like Cairo-Pinguin.
+	
 	return CAIRO_DOCK_LET_PASS_NOTIFICATION;
 }
 
@@ -542,3 +555,10 @@ gboolean cairo_dock_notification_module_detached (gpointer pUserData, CairoDockM
 	
 	return CAIRO_DOCK_LET_PASS_NOTIFICATION;
 }
+
+gboolean cairo_dock_notification_shortkey_added_or_removed (gpointer pUserData, CairoKeyBinding *pKeyBinding)
+{
+	cairo_dock_gui_trigger_reload_shortkeys ();
+	
+	return CAIRO_DOCK_LET_PASS_NOTIFICATION;
+}
