@@ -37,7 +37,7 @@
 **    Karl Lattimer <karl@qdh.org.uk>
 ** Originally conceived as a stress-test for cairo, librsvg, and glitz.
 **
-*********************** VERSION 0.1 and above (2007-2011) *********************
+*********************** VERSION 0.1 and above (2007-2012) *********************
 **
 ** author(s) :
 **     Fabrice Rey <fabounet@glx-dock.org>
@@ -63,7 +63,6 @@
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 
-#include <gtk/gtkgl.h>
 
 #include "config.h"
 #include "cairo-dock-icon-facility.h"  // cairo_dock_get_first_icon
@@ -77,6 +76,7 @@
 #include "cairo-dock-config.h"
 #include "cairo-dock-file-manager.h"
 #include "cairo-dock-log.h"
+#include "cairo-dock-keybinder.h"
 #include "cairo-dock-draw-opengl.h"
 #include "cairo-dock-launcher-manager.h"  // cairo_dock_launch_command
 
@@ -97,7 +97,7 @@
 // Nom du repertoire des themes de dock.
 #define CAIRO_DOCK_THEMES_DIR "themes"
 // Nom du repertoire des themes de dock sur le serveur
-#define CAIRO_DOCK_DISTANT_THEMES_DIR "themes2.4"
+#define CAIRO_DOCK_DISTANT_THEMES_DIR "themes3.0"
 
 extern gchar *g_cCairoDockDataDir;
 extern gchar *g_cCurrentThemePath;
@@ -108,7 +108,6 @@ extern int g_iMajorVersion, g_iMinorVersion, g_iMicroVersion;
 extern CairoDock *g_pMainDock;
 extern CairoDockGLConfig g_openglConfig;
 
-extern gboolean g_bUseGlitz;
 extern gboolean g_bUseOpenGL;
 extern gboolean g_bEasterEggs;
 
@@ -200,8 +199,8 @@ static void _cairo_dock_set_signal_interception (void)
 	signal (SIGSEGV, _cairo_dock_intercept_signal);  // Segmentation violation
 	signal (SIGFPE, _cairo_dock_intercept_signal);  // Floating-point exception
 	signal (SIGILL, _cairo_dock_intercept_signal);  // Illegal instruction
-	signal (SIGABRT, _cairo_dock_intercept_signal);  // Abort
-	signal (SIGTERM, _cairo_dock_quit);  // Abort
+	signal (SIGABRT, _cairo_dock_intercept_signal);  // Abort // kill -6
+	signal (SIGTERM, _cairo_dock_quit);  // Term // kill -15 (system)
 }
 
 static gboolean on_delete_maintenance_gui (GtkWidget *pWidget, GdkEvent *event, GMainLoop *pBlockingLoop)
@@ -262,6 +261,10 @@ static void _cairo_dock_get_global_config (const gchar *cCairoDockDataDir)
 }
 
 
+static void _on_shortkey (const gchar *cShortkey, gpointer data)
+{
+	g_print ("pouet\n");
+}
 int main (int argc, char** argv)
 {
 	//\___________________ build the command line used to respawn, and check if we have been launched from another life.
@@ -302,30 +305,33 @@ int main (int argc, char** argv)
 		g_set_print_handler(PrintMuteFunc);
 	
 	gtk_init (&argc, &argv);
-	gtk_gl_init (&argc, &argv);
 	
 	GError *erreur = NULL;
 	
+	//\___________________ internationalize the app.
+	bindtextdomain (CAIRO_DOCK_GETTEXT_PACKAGE, CAIRO_DOCK_LOCALE_DIR);
+	bind_textdomain_codeset (CAIRO_DOCK_GETTEXT_PACKAGE, "UTF-8");
+	textdomain (CAIRO_DOCK_GETTEXT_PACKAGE);
+	
 	//\___________________ get app's options.
-	gboolean bSafeMode = FALSE, bMaintenance = FALSE, bNoSticky = FALSE, bNormalHint = FALSE, bCappuccino = FALSE, bPrintVersion = FALSE, bTesting = FALSE, bForceIndirectRendering = FALSE, bForceOpenGL = FALSE, bToggleIndirectRendering = FALSE, bKeepAbove = FALSE, bForceColors = FALSE;
+	gboolean bSafeMode = FALSE, bMaintenance = FALSE, bNoSticky = FALSE, bNormalHint = FALSE, bCappuccino = FALSE, bPrintVersion = FALSE, bTesting = FALSE, bForceIndirectRendering = FALSE, bForceOpenGL = FALSE, bToggleIndirectRendering = FALSE, bKeepAbove = FALSE, bForceColors = FALSE, bAskBackend = FALSE;
 	gchar *cEnvironment = NULL, *cUserDefinedDataDir = NULL, *cVerbosity = 0, *cUserDefinedModuleDir = NULL, *cExcludeModule = NULL, *cThemeServerAdress = NULL;
 	int iDelay = 0;
 	GOptionEntry pOptionsTable[] =
 	{
+		// GLDI options: cairo, opengl, indirect-opengl, env, keep-above, no-sticky
 		{"cairo", 'c', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE,
 			&g_bForceCairo,
 			_("Use Cairo backend."), NULL},
 		{"opengl", 'o', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE,
 			&bForceOpenGL,
 			_("Use OpenGL backend."), NULL},
-#ifdef HAVE_GLITZ
-		{"glitz", 'g', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE,
-			&g_bUseGlitz,
-			_("Force Glitz backend (hardware acceleration for cairo, needs a glitz-enabled libcairo)."), NULL},
-#endif
 		{"indirect-opengl", 'O', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE,
 			&bToggleIndirectRendering,
 			_("Use OpenGL backend with indirect rendering. There are very few case where this option should be used."), NULL},
+		{"ask-backend", 'A', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE,
+			&bAskBackend,
+			_("Ask again on startup which backend to use."), NULL},
 		{"env", 'e', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_STRING,
 			&cEnvironment,
 			_("Force the dock to consider this environnement - use it with care."), NULL},
@@ -399,7 +405,7 @@ int main (int argc, char** argv)
 	}
 	
 	if (g_bLocked)
-		g_print ("Cairo-Dock will be locked.\n");
+		cd_warning ("Cairo-Dock will be locked.\n");
 	
 	if (cVerbosity != NULL)
 	{
@@ -422,16 +428,14 @@ int main (int argc, char** argv)
 		else if (strcmp (cEnvironment, "none") == 0)
 			iDesktopEnv = CAIRO_DOCK_UNKNOWN_ENV;
 		else
-			cd_warning ("unknown environnment '%s'", cEnvironment);
+			cd_warning ("Unknown environment '%s'", cEnvironment);
 		g_free (cEnvironment);
 	}
-#ifdef HAVE_GLITZ
-	cd_warning ("Compiled with Glitz (hardware acceleration support for cairo)");
-#endif
 	
 	if (bCappuccino)
 	{
-		g_print ("Cairo-Dock does anything, including coffee !.\n");
+		const gchar *cCappuccino = _("Cairo-Dock makes anything, including coffee !");
+		g_print ("%s\n", cCappuccino);
 		return 0;
 	}
 	
@@ -456,11 +460,11 @@ int main (int argc, char** argv)
 	}
 	bFirstLaunch = ! g_file_test (cRootDataDirPath, G_FILE_TEST_IS_DIR);
 	_cairo_dock_get_global_config (cRootDataDirPath);
-	
-	//\___________________ internationalize the app.
-	bindtextdomain (CAIRO_DOCK_GETTEXT_PACKAGE, CAIRO_DOCK_LOCALE_DIR);
-	bind_textdomain_codeset (CAIRO_DOCK_GETTEXT_PACKAGE, "UTF-8");
-	textdomain (CAIRO_DOCK_GETTEXT_PACKAGE);
+	if (bAskBackend)
+	{
+		g_free (s_cDefaulBackend);
+		s_cDefaulBackend = NULL;
+	}
 	
 	//\___________________ delay the startup if specified.
 	if (iDelay > 0)
@@ -505,20 +509,24 @@ int main (int argc, char** argv)
 				GTK_RESPONSE_NO,
 				NULL);
 			GtkWidget *label = gtk_label_new (_("OpenGL allows you to use the hardware acceleration, reducing the CPU load to the minimum.\nIt also allows some pretty visual effects similar to Compiz.\nHowever, some cards and/or their drivers don't fully support it, which may prevent the dock from running correctly.\nDo you want to activate OpenGL ?\n (To not show this dialog, launch the dock from the Application menu,\n  or with the -o option to force OpenGL and -c to force cairo.)"));
-			gtk_box_pack_start (GTK_BOX (GTK_DIALOG(dialog)->vbox), label, FALSE, FALSE, 0);
-			
-			GtkWidget *pAskBox = gtk_hbox_new (FALSE, 3);
-			gtk_box_pack_start (GTK_BOX (GTK_DIALOG(dialog)->vbox), pAskBox, FALSE, FALSE, 0);
+			#if (GTK_MAJOR_VERSION > 2 || GTK_MINOR_VERSION >= 14)
+				GtkWidget *pContentBox = gtk_dialog_get_content_area (GTK_DIALOG(dialog));
+			#else
+				GtkWidget *pContentBox =  GTK_DIALOG(dialog)->vbox;
+			#endif
+
+			gtk_box_pack_start (GTK_BOX (pContentBox), label, FALSE, FALSE, 0);
+
+			GtkWidget *pAskBox = _gtk_hbox_new (3);
+			gtk_box_pack_start (GTK_BOX (pContentBox), pAskBox, FALSE, FALSE, 0);
 			label = gtk_label_new (_("Remember this choice"));
 			GtkWidget *pCheckBox = gtk_check_button_new ();
 			gtk_box_pack_end (GTK_BOX (pAskBox), pCheckBox, FALSE, FALSE, 0);
 			gtk_box_pack_end (GTK_BOX (pAskBox), label, FALSE, FALSE, 0);
-			///g_signal_connect (G_OBJECT (pCheckBox), "toggled", G_CALLBACK(_toggle_remember_choice), dialog);
 			
 			gtk_widget_show_all (dialog);
 			
 			gint iAnswer = gtk_dialog_run (GTK_DIALOG (dialog));  // lance sa propre main loop, c'est pourquoi on peut le faire avant le gtk_main().
-			///gboolean bRememberChoice = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (dialog), "remember"));
 			gboolean bRememberChoice = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (pCheckBox));
 			gtk_widget_destroy (dialog);
 			if (iAnswer == GTK_RESPONSE_NO)
@@ -541,9 +549,16 @@ int main (int argc, char** argv)
 			cairo_dock_deactivate_opengl ();
 		}
 	}
-	g_print ("\n ============================================================================ \n\tCairo-Dock version: %s\n\tCompiled date:  %s %s\n\tRunning with OpenGL: %d\n ============================================================================\n\n",
+	g_print ("\n"
+	" ============================================================================\n"
+	"\tCairo-Dock version : %s\n"
+	"\tCompiled date      : %s %s\n"
+	"\tBuilt with GTK     : %d.%d\n"
+	"\tRunning with OpenGL: %d\n"
+	" ============================================================================\n\n",
 		CAIRO_DOCK_VERSION,
 		__DATE__, __TIME__,
+		GTK_MAJOR_VERSION, GTK_MINOR_VERSION,
 		g_bUseOpenGL);
 	
 	//\___________________ load plug-ins (must be done after everything is initialized).
@@ -612,7 +627,7 @@ int main (int argc, char** argv)
 		(CairoDockNotificationFunc) cairo_dock_notification_icon_moved,
 		CAIRO_DOCK_RUN_AFTER, NULL);
 	cairo_dock_register_notification_on_object (&myDocksMgr,
-		NOTIFICATION_STOP_DOCK,
+		NOTIFICATION_DESTROY,
 		(CairoDockNotificationFunc) cairo_dock_notification_dock_destroyed,
 		CAIRO_DOCK_RUN_AFTER, NULL);
 	cairo_dock_register_notification_on_object (&myModulesMgr,
@@ -636,13 +651,25 @@ int main (int argc, char** argv)
 		(CairoDockNotificationFunc) cairo_dock_notification_icon_removed,
 		CAIRO_DOCK_RUN_AFTER, NULL);
 	cairo_dock_register_notification_on_object (&myDeskletsMgr,
-		NOTIFICATION_STOP_DESKLET,
+		NOTIFICATION_DESTROY,
 		(CairoDockNotificationFunc) cairo_dock_notification_desklet_destroyed,
 		CAIRO_DOCK_RUN_AFTER, NULL);
 	cairo_dock_register_notification_on_object (&myDeskletsMgr,
 		NOTIFICATION_NEW_DESKLET,
 		(CairoDockNotificationFunc) cairo_dock_notification_desklet_destroyed,
 		CAIRO_DOCK_RUN_AFTER, NULL);
+	cairo_dock_register_notification_on_object (&myShortkeysMgr,
+		NOTIFICATION_SHORTKEY_ADDED,
+		(CairoDockNotificationFunc) cairo_dock_notification_shortkey_added_or_removed,
+		CAIRO_DOCK_RUN_AFTER, NULL);
+	cairo_dock_register_notification_on_object (&myShortkeysMgr,
+		NOTIFICATION_SHORTKEY_REMOVED,
+		(CairoDockNotificationFunc) cairo_dock_notification_shortkey_added_or_removed,
+		CAIRO_DOCK_RUN_AFTER, NULL);
+	cairo_dock_register_notification_on_object (&myShortkeysMgr,
+		NOTIFICATION_SHORTKEY_CHANGED,
+		(CairoDockNotificationFunc) cairo_dock_notification_shortkey_added_or_removed,
+		CAIRO_DOCK_RUN_AFTER, NULL);
 	
 	//\___________________ handle crashes.
 	if (! bTesting)
@@ -681,7 +708,14 @@ int main (int argc, char** argv)
 	cd_message ("loading theme ...");
 	if (! g_file_test (g_cConfFile, G_FILE_TEST_EXISTS))  // no theme yet, copy the default theme first.
 	{
-		gchar *cCommand = g_strdup_printf ("/bin/cp -r \"%s\"/* \"%s\"", CAIRO_DOCK_SHARE_DATA_DIR"/themes/_default_", g_cCurrentThemePath);
+		const gchar *cDesktopSessionEnv = g_getenv ("DESKTOP_SESSION");
+		const gchar *cThemeName;
+		if (g_strcmp0 (cDesktopSessionEnv, "cairo-dock") == 0
+			|| g_strcmp0 (cDesktopSessionEnv, "cairo-dock-fallback") == 0)
+			cThemeName = "Default-Panel";
+		else
+			cThemeName = "Default-Single";
+		gchar *cCommand = g_strdup_printf ("/bin/cp -r \"%s/%s\"/* \"%s\"", CAIRO_DOCK_SHARE_DATA_DIR"/themes", cThemeName, g_cCurrentThemePath);
 		cd_message (cCommand);
 		int r = system (cCommand);
 		g_free (cCommand);
@@ -759,14 +793,41 @@ int main (int argc, char** argv)
 	if (! bTesting)
 		g_timeout_add_seconds (5, _cairo_dock_successful_launch, GINT_TO_POINTER (bFirstLaunch));
 	
-	/*g_print ("\n\nTODO (2.5):\n"
+	/*g_print ("\n\nTODO (3.0):\n"
+	"** Must-Be-Done **\n"
+	"- crash: Unity-4 -> DustSand -> Clear -> Humanity -> Unity-4\n"
+	"- add a parameter for sub-dock icon size?\n"
+	"- fix quick-info size/drawing/blurry\n"
+	"- cairo-dock-gui-themes.c:111 -> cairo-dock-gui-manager.c:414\n"
+	"- cairo-dock-applications-manager.c:1663 -> cairo-dock-class-manager.c:66\n"
+	"- indicators on different icon sizes\n"
+	"- slide view view: separator & scrollbar in vertical mode, last line out of the dock in horizontal mode\n"
+	"- cairo_dock_get_pixbuf_from_pixmap: assertion `pIconPixbuf != NULL' failed\n"
+	"- drop/hover indicators on different icon sizes\n"
+	"- Sys-monitor: ... and 0.0 on quick-info\n"
+	"- MP: empty icon on startup (audacious)\n"
+	"- cairo_dock_get_max_scale & fMagnitudeMax must die\n"
+	"- check Notification Area, there might be a bug (bitecoin, litecoin, skype)\n"
+	"- make some tests with the Dbus API and sub-docks / tmp launcher\n"
+	"- PM: crash when no battery is plugged ?\n"
+	"** That-Would-Be-Nice **\n"
+	"- redirection texture: make it per-container, or use the container gl buffer instead\n"
+	"- Slide view: allow to use the same borders as default (radius, color, width)\n"
+	"- Dialogs: allow to use the same borders as default (radius, color)\n"
+	"- let the view place the X thumbnail\n"
+	"- get rid of CAIRO_DOCK_IS_DOCK\n"
+	"- Icon bg: add ratio\n"
+	"- Default theme more consistent\n"
+	"- Recent Events: handle recent apps\n"
+	"- Firefox launcher: handle recent URLs\n"
 	"- check for config panel (g_object_unref assertion)\n"
 	"- review Help hints\n"
-	"- find Kwin config tool for Composite-manager\n"
+	"- find Kwin/XFCE versions of gnome-control-center and KWin config tool for Composite-manager\n"
 	"- draw a preview of the dock in opengl\n"
 	"- display Help GUI in simple mode\n"
 	"- kde integration ++\n"
 	"- stack: enable iSubdockViewType\n"
+	"- link launchers with class+command\n"
 	"\n");*/
 	
 	gtk_main ();
