@@ -27,6 +27,7 @@
 #include <X11/Xutil.h>
 #include <gdk/gdkx.h>
 
+#include "cairo-dock-notifications.h"
 #include "cairo-dock-icon-facility.h"  // cairo_dock_set_icon_name
 #include "cairo-dock-dialog-manager.h"
 #include "cairo-dock-animations.h"
@@ -44,6 +45,7 @@
 extern CairoDock *g_pMainDock;
 extern CairoDockDesktopGeometry g_desktopGeometry;
 extern CairoDockHidingEffect *g_pHidingBackend;  // cairo_dock_is_hidden
+extern CairoContainer *g_pPrimaryContainer;
 
 static void _cairo_dock_appli_demands_attention (Icon *icon, CairoDock *pDock, gboolean bForceDemand, Icon *pHiddenIcon)
 {
@@ -72,7 +74,7 @@ static void _cairo_dock_appli_demands_attention (Icon *icon, CairoDock *pDock, g
 			if (pDock->iRefCount == 0 && pDock->iVisibility == CAIRO_DOCK_VISI_KEEP_BELOW && pDock->bIsBelow)
 				cairo_dock_pop_up (pDock);
 			gtk_window_set_keep_above (GTK_WINDOW (pDialog->container.pWidget), TRUE);
-			Window Xid = GDK_WINDOW_XID (pDialog->container.pWidget->window);
+			Window Xid = gldi_container_get_Xid (CAIRO_CONTAINER (pDialog));
 			cairo_dock_set_xwindow_type_hint (Xid, "_NET_WM_WINDOW_TYPE_DOCK");  // pour passer devant les fenetres plein ecran; depend du WM.
 		}
 	}
@@ -95,9 +97,12 @@ static void _cairo_dock_appli_demands_attention (Icon *icon, CairoDock *pDock, g
 			if (pParentDock)
 				cairo_dock_show_subdock (pPointedIcon, pParentDock);
 		}*/
-		cairo_dock_request_icon_animation (icon, pDock, myTaskbarParam.cAnimationOnDemandsAttention, 10000);  // animation de 2-3 heures.
+		cairo_dock_request_icon_animation (icon, CAIRO_CONTAINER (pDock), myTaskbarParam.cAnimationOnDemandsAttention, 10000);  // animation de 2-3 heures.
 		cairo_dock_launch_animation (CAIRO_CONTAINER (pDock));  // dans le au cas ou le dock ne serait pas encore visible, la fonction precedente n'a pas lance l'animation.
 	}
+	
+	// notify everybody
+	cairo_dock_notify_on_object (&myTaskbarMgr, NOTIFICATION_APPLI_ATTENTION_CHANGED, pHiddenIcon?pHiddenIcon:icon, TRUE);
 }
 void cairo_dock_appli_demands_attention (Icon *icon)
 {
@@ -107,9 +112,8 @@ void cairo_dock_appli_demands_attention (Icon *icon)
 		cd_message ("cette fenetre a deja le focus, elle ne peut demander l'attention en plus.");
 		return ;
 	}
-	if (icon->bIsDemandingAttention &&
-		/*cairo_dock_icon_has_dialog (icon) &&*/
-		icon->cLastAttentionDemand && icon->cName && strcmp (icon->cLastAttentionDemand, icon->cName) == 0)  // le message n'a pas change entre les 2 demandes.
+	if (icon->bIsDemandingAttention  // already demanding attention
+	&& icon->cLastAttentionDemand && icon->cName && strcmp (icon->cLastAttentionDemand, icon->cName) == 0)  // and message has not changed between the 2 demands.
 	{
 		return ;
 	}
@@ -153,6 +157,9 @@ static void _cairo_dock_appli_stops_demanding_attention (Icon *icon, CairoDock *
 	icon->bIsDemandingAttention = FALSE;
 	if (pDock->iRefCount == 0 && pDock->iVisibility == CAIRO_DOCK_VISI_KEEP_BELOW && ! pDock->bIsBelow && ! pDock->container.bInside)
 		cairo_dock_pop_down (pDock);
+	
+	// notify everybody
+	cairo_dock_notify_on_object (&myTaskbarMgr, NOTIFICATION_APPLI_ATTENTION_CHANGED, icon, FALSE);
 }
 void cairo_dock_appli_stops_demanding_attention (Icon *icon)
 {
@@ -181,7 +188,7 @@ void cairo_dock_animate_icon_on_active (Icon *icon, CairoDock *pParentDock)
 		if (myTaskbarParam.cAnimationOnActiveWindow)
 		{
 			if (cairo_dock_animation_will_be_visible (pParentDock) && icon->iAnimationState == CAIRO_DOCK_STATE_REST)
-				cairo_dock_request_icon_animation (icon, pParentDock, myTaskbarParam.cAnimationOnActiveWindow, 1);
+				cairo_dock_request_icon_animation (icon, CAIRO_CONTAINER (pParentDock), myTaskbarParam.cAnimationOnActiveWindow, 1);
 		}
 		else
 		{
@@ -296,7 +303,7 @@ static void _load_class_icon (Icon *icon)
 		}
 	}
 }
-static Icon *cairo_dock_create_icon_for_class_subdock (Icon *pSameClassIcon, CairoDock *pClassMateParentDock, CairoDock *pClassDock)
+static Icon *cairo_dock_create_icon_for_class_subdock (Icon *pSameClassIcon, CairoDock *pClassDock)
 {
 	Icon *pFakeClassIcon = cairo_dock_new_icon ();
 	pFakeClassIcon->iTrueType = CAIRO_DOCK_ICON_TYPE_CLASS_CONTAINER;
@@ -311,53 +318,50 @@ static Icon *cairo_dock_create_icon_for_class_subdock (Icon *pSameClassIcon, Cai
 	pFakeClassIcon->cParentDockName = g_strdup (pSameClassIcon->cParentDockName);
 	pFakeClassIcon->bHasIndicator = pSameClassIcon->bHasIndicator;
 	
-	/**pFakeClassIcon->fWidth = pSameClassIcon->fWidth / pClassMateParentDock->container.fRatio;
-	pFakeClassIcon->fHeight = pSameClassIcon->fHeight / pClassMateParentDock->container.fRatio;
-	pFakeClassIcon->fXMax = pSameClassIcon->fXMax;
-	pFakeClassIcon->fXMin = pSameClassIcon->fXMin;
-	pFakeClassIcon->fXAtRest = pSameClassIcon->fXAtRest;*/
-	//g_print ("%s() : TESTER SANS LES 5 LIGNES DU DESSUS...\n", __func__);
 	pFakeClassIcon->pSubDock = pClassDock;
+	
 	return pFakeClassIcon;
 }
 
-static CairoDock *_cairo_dock_set_parent_dock_name_for_appli (Icon *icon, CairoDock *pMainDock)
+// this function is used when we have an appli that is not inhibited. we can place it either in its subdock or in the dock next to an inhibitor or in the main dock amongst the other applis
+static CairoDock *_cairo_dock_set_parent_dock_name_for_appli (Icon *icon, CairoDock *pMainDock, const gchar *cMainDockName)
 {
 	cd_message ("%s (%s)", __func__, icon->cName);
 	CairoDock *pParentDock = pMainDock;
 	g_free (icon->cParentDockName);
-	if (CAIRO_DOCK_IS_APPLI (icon) && myTaskbarParam.bGroupAppliByClass && icon->cClass != NULL && ! cairo_dock_class_is_expanded (icon->cClass))
+	if (CAIRO_DOCK_IS_APPLI (icon) && myTaskbarParam.bGroupAppliByClass && icon->cClass != NULL && ! cairo_dock_class_is_expanded (icon->cClass))  // if this is a valid appli and we want to group the classes.
 	{
-		Icon *pSameClassIcon = cairo_dock_get_classmate (icon);  // un inhibiteur dans un dock OU une appli de meme classe dans le main dock.
-		if (pSameClassIcon == NULL)  // aucun classmate => elle va dans le main dock.
+		Icon *pSameClassIcon = cairo_dock_get_classmate (icon);  // un inhibiteur dans un dock avec Xid ou subdock OU une appli de meme classe dans un dock != class-sub-dock.
+		if (pSameClassIcon == NULL)  // aucun classmate => elle va dans son class sub-dock ou dans le main dock.
 		{
-			cd_message ("  classe %s encore vide", icon->cClass);
-			pParentDock = cairo_dock_search_dock_from_name (icon->cClass);
-			if (pParentDock == NULL)
+			cd_message ("  no classmate for %s", icon->cClass);
+			pParentDock = cairo_dock_get_class_subdock (icon->cClass);
+			if (pParentDock == NULL)  // no class sub-dock => go to main dock
 			{
-				pParentDock = pMainDock;
-				icon->cParentDockName = g_strdup (CAIRO_DOCK_MAIN_DOCK_NAME);
+				pParentDock = cairo_dock_search_dock_from_name (cMainDockName);
+				icon->cParentDockName = g_strdup (cMainDockName);
 			}
-			else
+			else  // go to class sub-dock
 			{
-				icon->cParentDockName = g_strdup (icon->cClass);
+				icon->cParentDockName = g_strdup (cairo_dock_get_class_subdock_name (icon->cClass));
 			}
 		}
 		else  // on la met dans le sous-dock de sa classe.
 		{
-			icon->cParentDockName = g_strdup (icon->cClass);
-
-			//\____________ On cree ce sous-dock si necessaire.
-			pParentDock = cairo_dock_search_dock_from_name (icon->cClass);
+			//\____________ create the class sub-dock if necessary
+			pParentDock = cairo_dock_get_class_subdock (icon->cClass);
 			if (pParentDock == NULL)  // alors il faut creer le sous-dock, qu'on associera soit a pSameClassIcon soit a un fake.
 			{
 				cd_message ("  creation du dock pour la classe %s", icon->cClass);
 				pMainDock = cairo_dock_search_dock_from_name (pSameClassIcon->cParentDockName);  // can be NULL (even if in practice will never be).
-				pParentDock = cairo_dock_create_subdock_from_scratch (NULL, icon->cClass, pMainDock);
+				pParentDock = cairo_dock_create_class_subdock (icon->cClass, pMainDock);
 			}
 			else
 				cd_message ("  sous-dock de la classe %s existant", icon->cClass);
 			
+			icon->cParentDockName = g_strdup (cairo_dock_get_class_subdock_name (icon->cClass));
+			
+			//\____________ link this sub-dock to the inhibitor, or to a fake appli icon.
 			if (CAIRO_DOCK_ICON_TYPE_IS_LAUNCHER (pSameClassIcon) || CAIRO_DOCK_ICON_TYPE_IS_APPLET (pSameClassIcon))  // c'est un inhibiteur.
 			{
 				if (pSameClassIcon->Xid != 0)  // actuellement l'inhibiteur inhibe 1 seule appli.
@@ -386,8 +390,8 @@ static CairoDock *_cairo_dock_set_parent_dock_name_for_appli (Icon *icon, CairoD
 					{
 						cd_debug (" on insere %s dans le dock de la classe", pInhibitedIcon->cName);
 						g_free (pInhibitedIcon->cParentDockName);
-						pInhibitedIcon->cParentDockName = g_strdup (icon->cClass);
-						cairo_dock_insert_icon_in_dock_full (pInhibitedIcon, pParentDock, CAIRO_DOCK_UPDATE_DOCK_SIZE, ! CAIRO_DOCK_ANIMATE_ICON, ! CAIRO_DOCK_INSERT_SEPARATOR, NULL);
+						pInhibitedIcon->cParentDockName = g_strdup (icon->cParentDockName);
+						cairo_dock_insert_icon_in_dock_full (pInhibitedIcon, pParentDock, ! CAIRO_DOCK_ANIMATE_ICON, ! CAIRO_DOCK_INSERT_SEPARATOR, NULL);
 					}
 				}
 				else if (pSameClassIcon->pSubDock != pParentDock)
@@ -398,32 +402,32 @@ static CairoDock *_cairo_dock_set_parent_dock_name_for_appli (Icon *icon, CairoD
 				//\______________ On cree une icone de paille.
 				cd_debug (" on cree un fake...");
 				CairoDock *pClassMateParentDock = cairo_dock_search_dock_from_name (pSameClassIcon->cParentDockName);  // c'est en fait le main dock.
-				Icon *pFakeClassIcon = cairo_dock_create_icon_for_class_subdock (pSameClassIcon, pClassMateParentDock, pParentDock);
-				
-				//\______________ On la charge.
-				cairo_dock_trigger_load_icon_buffers (pFakeClassIcon, CAIRO_CONTAINER (pClassMateParentDock));
+				Icon *pFakeClassIcon = cairo_dock_create_icon_for_class_subdock (pSameClassIcon, pParentDock);
 				
 				//\______________ On detache le classmate, on le place dans le sous-dock, et on lui substitue le faux.
 				cd_debug (" on detache %s pour la passer dans le sous-dock de sa classe", pSameClassIcon->cName);
-				cairo_dock_detach_icon_from_dock (pSameClassIcon, pClassMateParentDock, FALSE);
+				cairo_dock_detach_icon_from_dock_full (pSameClassIcon, pClassMateParentDock, FALSE);
 				g_free (pSameClassIcon->cParentDockName);
-				pSameClassIcon->cParentDockName = g_strdup (pSameClassIcon->cClass);
-				cairo_dock_insert_icon_in_dock_full (pSameClassIcon, pParentDock, ! CAIRO_DOCK_UPDATE_DOCK_SIZE, ! CAIRO_DOCK_ANIMATE_ICON, ! CAIRO_DOCK_INSERT_SEPARATOR, NULL);
+				pSameClassIcon->cParentDockName = g_strdup (icon->cParentDockName);
+				cairo_dock_insert_icon_in_dock_full (pSameClassIcon, pParentDock, ! CAIRO_DOCK_ANIMATE_ICON, ! CAIRO_DOCK_INSERT_SEPARATOR, NULL);
 				
 				cd_debug (" on lui substitue le fake");
-				cairo_dock_insert_icon_in_dock_full (pFakeClassIcon, pClassMateParentDock, CAIRO_DOCK_UPDATE_DOCK_SIZE, ! CAIRO_DOCK_ANIMATE_ICON, ! CAIRO_DOCK_INSERT_SEPARATOR, NULL);
-				cairo_dock_calculate_dock_icons (pClassMateParentDock);
-				cairo_dock_redraw_icon (pFakeClassIcon, CAIRO_CONTAINER (pClassMateParentDock));
+				cairo_dock_insert_icon_in_dock_full (pFakeClassIcon, pClassMateParentDock, ! CAIRO_DOCK_ANIMATE_ICON, ! CAIRO_DOCK_INSERT_SEPARATOR, NULL);
+				cairo_dock_redraw_container (CAIRO_CONTAINER (pClassMateParentDock));
+				
+				if (pFakeClassIcon->iSubdockViewType != 0)
+					cairo_dock_trigger_redraw_subdock_content_on_icon (pFakeClassIcon);
 			}
 		}
 	}
-	else
-		icon->cParentDockName = g_strdup (CAIRO_DOCK_MAIN_DOCK_NAME);
-
+	else  /// TODO: look for an inhibitor or a classmate to go in its dock (it's not necessarily the main dock) ...
+	{
+		icon->cParentDockName = g_strdup (cMainDockName);
+	}
 	return pParentDock;
 }
 
-CairoDock *cairo_dock_insert_appli_in_dock (Icon *icon, CairoDock *pMainDock, gboolean bUpdateSize, gboolean bAnimate)
+CairoDock *cairo_dock_insert_appli_in_dock (Icon *icon, CairoDock *pMainDock, gboolean bAnimate)
 {
 	if (! myTaskbarParam.bShowAppli)
 		return NULL;
@@ -431,7 +435,7 @@ CairoDock *cairo_dock_insert_appli_in_dock (Icon *icon, CairoDock *pMainDock, gb
 	
 	if (myTaskbarParam.bAppliOnCurrentDesktopOnly && ! cairo_dock_appli_is_on_current_desktop (icon))
 		return NULL;
-		
+	
 	//\_________________ On gere ses eventuels inhibiteurs.
 	if (myTaskbarParam.bMixLauncherAppli && cairo_dock_prevent_inhibited_class (icon))
 	{
@@ -446,18 +450,20 @@ CairoDock *cairo_dock_insert_appli_in_dock (Icon *icon, CairoDock *pMainDock, gb
 		return NULL;
 	}
 	
-	//\_________________ On determine dans quel dock l'inserer.
-	CairoDock *pParentDock = _cairo_dock_set_parent_dock_name_for_appli (icon, pMainDock);  // renseigne cParentDockName.
+	//\_________________ On determine dans quel dock l'inserer (cree au besoin).
+	CairoDock *pParentDock = _cairo_dock_set_parent_dock_name_for_appli (icon, pMainDock, CAIRO_DOCK_MAIN_DOCK_NAME);  // renseigne cParentDockName.   /// TODO: we should actually use the name of 'pMainDock' if one day we want to place the taskbar inside another dock...
 	g_return_val_if_fail (pParentDock != NULL, NULL);
 
 	//\_________________ On l'insere dans son dock parent en animant ce dernier eventuellement.
-	if ((myIconsParam.iSeparateIcons == 0 || myIconsParam.iSeparateIcons == 2) && pParentDock->iRefCount == 0)
+	if (myTaskbarParam.bMixLauncherAppli && pParentDock != cairo_dock_get_class_subdock (icon->cClass))  // this appli is amongst the launchers in the main dock
 	{
-		cairo_dock_set_class_order (icon);
+		cairo_dock_set_class_order_in_dock (icon, pParentDock);
 	}
-	else
-		icon->fOrder == CAIRO_DOCK_LAST_ORDER;  // en dernier.
-	cairo_dock_insert_icon_in_dock (icon, pParentDock, bUpdateSize, bAnimate);
+	else  // this appli is either in a different group or in the class sub-dock
+	{
+		cairo_dock_set_class_order_amongst_applis (icon, pParentDock);
+	}
+	cairo_dock_insert_icon_in_dock (icon, pParentDock, bAnimate);
 	cd_message (" insertion de %s complete (%.2f %.2fx%.2f) dans %s", icon->cName, icon->fInsertRemoveFactor, icon->fWidth, icon->fHeight, icon->cParentDockName);
 
 	if (bAnimate && cairo_dock_animation_will_be_visible (pParentDock))
@@ -480,20 +486,21 @@ CairoDock * cairo_dock_detach_appli (Icon *pIcon)
 	if (pParentDock == NULL)
 		return NULL;
 	
-	cairo_dock_detach_icon_from_dock (pIcon, pParentDock, TRUE);
+	cairo_dock_detach_icon_from_dock (pIcon, pParentDock);
 	
-	if (pIcon->cClass != NULL && pParentDock == cairo_dock_search_dock_from_name (pIcon->cClass))
+	if (pIcon->cClass != NULL && pParentDock == cairo_dock_get_class_subdock (pIcon->cClass))  // is in the sub-dock class -> check if we must destroy it.
 	{
 		gboolean bEmptyClassSubDock = cairo_dock_check_class_subdock_is_empty (pParentDock, pIcon->cClass);
-		if (bEmptyClassSubDock)
+		if (bEmptyClassSubDock)  // has been destroyed.
 			return NULL;
 	}
-	cairo_dock_update_dock_size (pParentDock);
+	///cairo_dock_update_dock_size (pParentDock);
 	return pParentDock;
 }
 
 #define x_icon_geometry(icon, pDock) (pDock->container.iWindowPositionX + icon->fXAtRest + (pDock->container.iWidth - pDock->fFlatDockWidth) / 2 + (pDock->iOffsetForExtend * (pDock->fAlign - .5) * 2))
-#define y_icon_geometry(icon, pDock) (pDock->container.iWindowPositionY + icon->fDrawY - icon->fHeight * myIconsParam.fAmplitude * pDock->fMagnitudeMax)
+///#define y_icon_geometry(icon, pDock) (pDock->container.iWindowPositionY + icon->fDrawY - icon->fHeight * myIconsParam.fAmplitude * pDock->fMagnitudeMax)
+#define y_icon_geometry(icon, pDock) (pDock->container.iWindowPositionY + icon->fDrawY)
 void  cairo_dock_set_one_icon_geometry_for_window_manager (Icon *icon, CairoDock *pDock)
 {
 	//g_print ("%s (%s)\n", __func__, icon->cName);
@@ -514,8 +521,10 @@ void cairo_dock_reserve_one_icon_geometry_for_window_manager (Window *Xid, Icon
 {
 	if (CAIRO_DOCK_IS_APPLI (icon) && icon->cParentDockName == NULL)
 	{
+		/// TODO: use the same algorithm as the class-manager to find the future position of the icon ...
+		
 		Icon *pInhibitor = cairo_dock_get_inhibitor (icon, FALSE);  // FALSE <=> meme en-dehors d'un dock
-		if (pInhibitor == NULL)  // cette icone n'est pas inhinbee, donc se minimisera dans le dock en une nouvelle icone.
+		if (pInhibitor == NULL)  // cette icone n'est pas inhibee, donc se minimisera dans le dock en une nouvelle icone.
 		{
 			int x, y;
 			Icon *pClassmate = cairo_dock_get_classmate (icon);
@@ -532,7 +541,7 @@ void cairo_dock_reserve_one_icon_geometry_for_window_manager (Window *Xid, Icon
 					y = y_icon_geometry (pClassmate, pClassmateDock);
 				}
 			}
-			else if (!myIconsParam.iSeparateIcons && pClassmate != NULL && pClassmateDock != NULL)  // on va se placer a cote.
+			else if (myTaskbarParam.bMixLauncherAppli && pClassmate != NULL && pClassmateDock != NULL)  // on va se placer a cote.
 			{
 				x = x_icon_geometry (pClassmate, pClassmateDock) + pClassmate->fWidth/2;
 				if (cairo_dock_is_hidden (pClassmateDock))
@@ -546,17 +555,31 @@ void cairo_dock_reserve_one_icon_geometry_for_window_manager (Window *Xid, Icon
 			}
 			else  // on va se placer a la fin de la barre des taches.
 			{
-				Icon *pLastAppli = cairo_dock_get_last_icon_until_order (pMainDock->icons, CAIRO_DOCK_APPLI);
-				if (pLastAppli != NULL)  // on se placera juste apres.
+				Icon *pIcon, *pLastLauncher = NULL;
+				GList *ic, *last_launcher_ic = NULL;
+				for (ic = pMainDock->icons; ic != NULL; ic = ic->next)
 				{
-					x = x_icon_geometry (pLastAppli, pMainDock) + pLastAppli->fWidth/2;
+					pIcon = ic->data;
+					if (CAIRO_DOCK_ICON_TYPE_IS_LAUNCHER (pIcon)  // launcher, even without class
+					|| CAIRO_DOCK_ICON_TYPE_IS_CONTAINER (pIcon)  // container icon (likely to contain some launchers)
+					|| (CAIRO_DOCK_ICON_TYPE_IS_APPLET (pIcon) && pIcon->cClass != NULL)  // applet acting like a launcher
+					|| (CAIRO_DOCK_ICON_TYPE_IS_SEPARATOR (pIcon)))  // separator (user or auto).
+					{
+						pLastLauncher = pIcon;
+						last_launcher_ic = ic;
+					}
+				}
+						
+				if (pLastLauncher != NULL)  // on se placera juste apres.
+				{
+					x = x_icon_geometry (pLastLauncher, pMainDock) + pLastLauncher->fWidth/2;
 					if (cairo_dock_is_hidden (pMainDock))
 					{
 						y = (pMainDock->container.bDirectionUp ? 0 : g_desktopGeometry.iXScreenHeight[CAIRO_DOCK_HORIZONTAL]);
 					}
 					else
 					{
-						y = y_icon_geometry (pLastAppli, pMainDock);
+						y = y_icon_geometry (pLastLauncher, pMainDock);
 					}
 				}
 				else  // aucune icone avant notre groupe, on sera insere en 1er.
@@ -640,3 +663,53 @@ void cairo_dock_move_window_to_current_desktop (Icon *pIcon)
 		0,
 		0);  // on ne veut pas decaler son viewport par rapport a nous.
 }
+
+
+const CairoDockImageBuffer *cairo_dock_appli_get_image_buffer (Icon *pIcon)
+{
+	static CairoDockImageBuffer image;
+	
+	if (pIcon->pIconBuffer == NULL)
+	{
+		const CairoDockImageBuffer *pImageBuffer = cairo_dock_get_class_image_buffer (pIcon->cClass);
+		if (pImageBuffer && pImageBuffer->pSurface)
+		{
+			memcpy (&image, pImageBuffer, sizeof (CairoDockImageBuffer));
+			return &image;
+		}
+	}
+	if (pIcon->pIconBuffer == NULL && g_pMainDock)
+	{
+		// set a size (we could set any size, but let's set something useful: if the icon is inserted in a dock and is already loaded at the correct size, it won't be loaded again).
+		gboolean bNoContainer = FALSE;
+		if (pIcon->pContainer == NULL)  // not in a container (=> no size) -> set a size before loading it.
+		{
+			bNoContainer = TRUE;
+			cairo_dock_set_icon_container (pIcon, g_pPrimaryContainer);
+			pIcon->fWidth = pIcon->fHeight = 0;  // no request
+			pIcon->iImageWidth = pIcon->iImageHeight = 0;
+			cairo_dock_set_icon_size_in_dock (g_pMainDock, pIcon);
+		}
+		
+		// load the icon
+		cairo_dock_load_icon_image (pIcon, g_pPrimaryContainer);
+		if (bNoContainer)
+		{
+			cairo_dock_set_icon_container (pIcon, NULL);
+		}
+	}
+	if (pIcon->pIconBuffer != NULL || pIcon->iIconTexture != 0)
+	{
+		int iWidth, iHeight;
+		cairo_dock_get_icon_extent (pIcon, &iWidth, &iHeight);
+		image.pSurface = pIcon->pIconBuffer;  // since we got the texture with the load(), we don't use the image buffer constructor.
+		image.iWidth = iWidth;
+		image.iHeight = iHeight;
+		image.iTexture = pIcon->iIconTexture;
+		return &image;
+	}
+	else
+	{
+		return NULL;
+	}
+}
