@@ -231,9 +231,9 @@ static void _cairo_dock_render_to_texture (CairoDataRenderer *pRenderer, Icon *p
 		pRenderer->bisRotate = TRUE;
 	}
 	
-	glPushMatrix ();
+	//glPushMatrix ();
 	pRenderer->interface.render_opengl (pRenderer);
-	glPopMatrix ();
+	//glPopMatrix ();
 	
 	//\________________ On dessine les overlays.
 	/**int iNbValues = cairo_data_renderer_get_nb_values (pRenderer);
@@ -301,9 +301,9 @@ void cairo_dock_render_overlays_to_context (CairoDataRenderer *pRenderer, int iN
 			PangoFontDescription *fd = pango_font_description_from_string ("Monospace 12");
 			pango_layout_set_font_description (pLayout, fd);
 			
-			PangoRectangle ink, log;
+			PangoRectangle log;
 			pango_layout_set_text (pLayout, pRenderer->cFormatBuffer, -1);
-			pango_layout_get_pixel_extents (pLayout, &ink, &log);
+			pango_layout_get_pixel_extents (pLayout, NULL, &log);
 			double fZoom = MIN (pText->fWidth * pRenderer->iWidth / (log.width), pText->fHeight * pRenderer->iHeight / log.height);
 			
 			cairo_move_to (pCairoContext,
@@ -337,9 +337,9 @@ static void _cairo_dock_render_to_context (CairoDataRenderer *pRenderer, Icon *p
 		//cairo_translate (pCairoContext, -pRenderer->iHeight/2, -pRenderer->iWidth/2);
 	}
 	
-	cairo_save (pCairoContext);
+	//cairo_save (pCairoContext);
 	pRenderer->interface.render (pRenderer, pCairoContext);
-	cairo_restore (pCairoContext);
+	//cairo_restore (pCairoContext);
 	
 	//\________________ On dessine les overlays.
 	/**int iNbValues = cairo_data_renderer_get_nb_values (pRenderer);
@@ -350,10 +350,10 @@ static void _cairo_dock_render_to_context (CairoDataRenderer *pRenderer, Icon *p
 	}*/
 	cairo_restore (pCairoContext);
 	
-	if (pContainer->bUseReflect)
+	/**if (pContainer->bUseReflect)
 	{
 		cairo_dock_add_reflection_to_icon (pIcon, pContainer);
-	}
+	}*/
 	
 	if (CAIRO_DOCK_CONTAINER_IS_OPENGL (pContainer))
 		cairo_dock_update_icon_texture (pIcon);
@@ -499,6 +499,9 @@ void cairo_dock_add_new_data_renderer_on_icon (Icon *pIcon, CairoContainer *pCon
 			CAIRO_DOCK_RUN_AFTER, NULL);  // pour l'affichage fluide.
 	}
 	
+	if (pRenderer->bCanRenderValueAsText && pRenderer->bWriteValues)
+		cairo_dock_set_quick_info (pIcon, pContainer, NULL);
+	
 	//\___________________ On le charge.
 	pRenderer->interface.load (pRenderer, pContainer, pAttribute);
 	
@@ -507,7 +510,22 @@ void cairo_dock_add_new_data_renderer_on_icon (Icon *pIcon, CairoContainer *pCon
 }
 
 
-
+static gboolean _render_delayed (Icon *pIcon)
+{
+	CairoDataRenderer *pRenderer = cairo_dock_get_icon_data_renderer (pIcon);
+	g_return_val_if_fail (pRenderer != NULL, FALSE);
+	
+	CairoContainer *pContainer = pIcon->pContainer;
+	g_return_val_if_fail (pContainer != NULL, FALSE);
+	
+	if (pContainer->iWidth == 1 && pContainer->iHeight == 1)  // container not yet resized, retry later
+		return TRUE;
+	
+	_cairo_dock_render_to_texture (pRenderer, pIcon, pContainer);
+	
+	pRenderer->iSidRenderIdle = 0;
+	return FALSE;
+}
 void cairo_dock_render_new_data_on_icon (Icon *pIcon, CairoContainer *pContainer, cairo_t *pCairoContext, double *pNewValues)
 {
 	CairoDataRenderer *pRenderer = cairo_dock_get_icon_data_renderer (pIcon);
@@ -546,7 +564,15 @@ void cairo_dock_render_new_data_on_icon (Icon *pIcon, CairoContainer *pContainer
 		else
 		{
 			pRenderer->fLatency = 0;
-			_cairo_dock_render_to_texture (pRenderer, pIcon, pContainer);
+			if (pContainer->iWidth == 1 && pContainer->iHeight == 1)  // container not yet resized, delay the rendering (OpenGL only).
+			{
+				if (pRenderer->iSidRenderIdle == 0)
+					pRenderer->iSidRenderIdle = g_idle_add ((GSourceFunc)_render_delayed, pIcon);  // if pIcon is freed, the data-renderer will be freed too, so this signal will vanish.
+			}
+			else
+			{
+				_cairo_dock_render_to_texture (pRenderer, pIcon, pContainer);
+			}
 		}
 	}
 	else
@@ -575,8 +601,6 @@ void cairo_dock_render_new_data_on_icon (Icon *pIcon, CairoContainer *pContainer
 		cairo_dock_set_quick_info (pIcon, pContainer, cBuffer);
 		g_free (cBuffer);
 	}
-	else
-		cairo_dock_set_quick_info (pIcon, pContainer, NULL);
 	
 	cairo_dock_redraw_icon (pIcon, pContainer);
 }
@@ -588,6 +612,9 @@ void cairo_dock_free_data_renderer (CairoDataRenderer *pRenderer)
 	if (pRenderer == NULL)
 		return ;
 	
+	if (pRenderer->iSidRenderIdle != 0)
+		g_source_remove (pRenderer->iSidRenderIdle);
+	
 	if (pRenderer->interface.unload)
 		pRenderer->interface.unload (pRenderer);
 	
@@ -634,11 +661,16 @@ void cairo_dock_free_data_renderer (CairoDataRenderer *pRenderer)
 void cairo_dock_remove_data_renderer_on_icon (Icon *pIcon)
 {
 	CairoDataRenderer *pRenderer = cairo_dock_get_icon_data_renderer (pIcon);
-	
-	cairo_dock_remove_notification_func_on_object (pIcon, NOTIFICATION_UPDATE_ICON_SLOW, (CairoDockNotificationFunc) cairo_dock_update_icon_data_renderer_notification, NULL);
-	
-	cairo_dock_free_data_renderer (pRenderer);
-	cairo_dock_set_data_renderer_on_icon (pIcon, NULL);
+	if (pRenderer != NULL)
+	{
+		cairo_dock_remove_notification_func_on_object (pIcon, NOTIFICATION_UPDATE_ICON_SLOW, (CairoDockNotificationFunc) cairo_dock_update_icon_data_renderer_notification, NULL);
+		
+		if (! pRenderer->bCanRenderValueAsText && pRenderer->bWriteValues)
+			cairo_dock_set_quick_info (pIcon, NULL, NULL);
+		
+		cairo_dock_free_data_renderer (pRenderer);
+		cairo_dock_set_data_renderer_on_icon (pIcon, NULL);
+	}
 }
 
 
