@@ -1,47 +1,46 @@
-/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 2; tab-width: 2 -*- */
-/*
-** Login : <ctaf42@gmail.com>
-** Started on  Sun Jan 27 18:35:38 2008 Cedric GESTES
-** $Id$
-**
-** Author(s)
-**  - Cedric GESTES <ctaf42@gmail.com>
-**  - Fabrice REY
-**
-** Copyright (C) 2008 Cedric GESTES
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 3 of the License, or
-** (at your option) any later version.
-**
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-** GNU General Public License for more details.
-**
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+/**
+* This file is a part of the Cairo-Dock project
+* Login : <ctaf42@gmail.com>
+* Started on  Sun Jan 27 18:35:38 2008 Cedric GESTES
+* $Id$
+*
+* Author(s)
+*  - Cedric GESTES <ctaf42@gmail.com>
+*  - Fabrice REY
+*
+* Copyright : (C) 2008 Cedric GESTES
+* E-mail    : see the 'copyright' file.
+*
+* This program is free software; you can redistribute it and/or
+* modify it under the terms of the GNU General Public License
+* as published by the Free Software Foundation; either version 3
+* of the License, or (at your option) any later version.
+*
+* This program is distributed in the hope that it will be useful,
+* but WITHOUT ANY WARRANTY; without even the implied warranty of
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+* GNU General Public License for more details.
+* You should have received a copy of the GNU General Public License
+* along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <stdlib.h>
 #include <string.h>
 #include <math.h>
 
-#include <gtk/gtkgl.h>
 #include <GL/gl.h> 
 #include <GL/glu.h> 
 #include <GL/glx.h> 
-#include <gdk/x11/gdkglx.h>
 
 #include <gdk/gdkx.h>
 #include "cairo-dock-draw.h"
 #include "cairo-dock-module-factory.h"
+#include "cairo-dock-icon-facility.h"  // cairo_dock_set_icon_container
 #include "cairo-dock-module-manager.h"  // cairo_dock_detach_module_instance
 #include "cairo-dock-dialog-manager.h"
 #include "cairo-dock-icon-factory.h"
 #include "cairo-dock-icon-facility.h"
-#include "cairo-dock-config.h"
+#include "cairo-dock-keyfile-utilities.h"
 #include "cairo-dock-X-manager.h"
 #include "cairo-dock-notifications.h"
 #include "cairo-dock-log.h"
@@ -70,29 +69,22 @@ static void on_drag_data_received_desklet (GtkWidget *pWidget, GdkDragContext *d
 ///////////////
 
 static gboolean on_expose_desklet(GtkWidget *pWidget,
+#if (GTK_MAJOR_VERSION < 3)
 	GdkEventExpose *pExpose,
+#else
+	cairo_t *ctx,
+#endif
 	CairoDesklet *pDesklet)
 {
-	if (pDesklet->iDesiredWidth != 0 && pDesklet->iDesiredHeight != 0 && (pDesklet->iKnownWidth != pDesklet->iDesiredWidth || pDesklet->iKnownHeight != pDesklet->iDesiredHeight))  // skip the drawing until the desklet has reached its size.
+	if (pDesklet->iDesiredWidth != 0 && pDesklet->iDesiredHeight != 0 && (pDesklet->iKnownWidth != pDesklet->iDesiredWidth || pDesklet->iKnownHeight != pDesklet->iDesiredHeight))  // skip the drawing until the desklet has reached its size, only make it transparent.
 	{
 		//g_print ("on saute le dessin\n");
 		if (g_bUseOpenGL)
 		{
-			GdkGLContext *pGlContext = gtk_widget_get_gl_context (pDesklet->container.pWidget);
-			GdkGLDrawable *pGlDrawable = gtk_widget_get_gl_drawable (pDesklet->container.pWidget);
-			if (!gdk_gl_drawable_gl_begin (pGlDrawable, pGlContext))
+			if (! gldi_glx_begin_draw_container (CAIRO_CONTAINER (pDesklet)))
 				return FALSE;
 			
-			glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
-			glLoadIdentity ();
-			
-			cairo_dock_apply_desktop_background_opengl (CAIRO_CONTAINER (pDesklet));
-			
-			if (gdk_gl_drawable_is_double_buffered (pGlDrawable))
-				gdk_gl_drawable_swap_buffers (pGlDrawable);
-			else
-				glFlush ();
-			gdk_gl_drawable_gl_end (pGlDrawable);
+			gldi_glx_end_draw_container (CAIRO_CONTAINER (pDesklet));
 		}
 		else
 		{
@@ -104,31 +96,18 @@ static gboolean on_expose_desklet(GtkWidget *pWidget,
 	
 	if (g_bUseOpenGL && pDesklet->pRenderer && pDesklet->pRenderer->render_opengl)
 	{
-		GdkGLContext *pGlContext = gtk_widget_get_gl_context (pDesklet->container.pWidget);
-		GdkGLDrawable *pGlDrawable = gtk_widget_get_gl_drawable (pDesklet->container.pWidget);
-		if (!gdk_gl_drawable_gl_begin (pGlDrawable, pGlContext))
+		if (! gldi_glx_begin_draw_container (CAIRO_CONTAINER (pDesklet)))
 			return FALSE;
 		
-		glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
-		glLoadIdentity ();
+		cairo_dock_notify_on_object (pDesklet, NOTIFICATION_RENDER, pDesklet, NULL);
 		
-		cairo_dock_apply_desktop_background_opengl (CAIRO_CONTAINER (pDesklet));
-		
-		cairo_dock_notify_on_object (&myDeskletsMgr, NOTIFICATION_RENDER_DESKLET, pDesklet, NULL);
-		cairo_dock_notify_on_object (pDesklet, NOTIFICATION_RENDER_DESKLET, pDesklet, NULL);
-		
-		if (gdk_gl_drawable_is_double_buffered (pGlDrawable))
-			gdk_gl_drawable_swap_buffers (pGlDrawable);
-		else
-			glFlush ();
-		gdk_gl_drawable_gl_end (pGlDrawable);
+		gldi_glx_end_draw_container (CAIRO_CONTAINER (pDesklet));
 	}
 	else
 	{
 		cairo_t *pCairoContext = cairo_dock_create_drawing_context_on_container (CAIRO_CONTAINER (pDesklet));
 		
-		cairo_dock_notify_on_object (&myDeskletsMgr, NOTIFICATION_RENDER_DESKLET, pDesklet, pCairoContext);
-		cairo_dock_notify_on_object (pDesklet, NOTIFICATION_RENDER_DESKLET, pDesklet, pCairoContext);
+		cairo_dock_notify_on_object (pDesklet, NOTIFICATION_RENDER, pDesklet, pCairoContext);
 		
 		cairo_destroy (pCairoContext);
 	}
@@ -138,35 +117,19 @@ static gboolean on_expose_desklet(GtkWidget *pWidget,
 
 static void _cairo_dock_set_desklet_input_shape (CairoDesklet *pDesklet)
 {
-	gtk_widget_input_shape_combine_mask (pDesklet->container.pWidget,
-		NULL,
-		0,
-		0);
+	gldi_container_set_input_shape (CAIRO_CONTAINER (pDesklet), NULL);
+	
 	if (pDesklet->bNoInput)
 	{
-		GdkBitmap *pShapeBitmap = (GdkBitmap*) gdk_pixmap_new (NULL,
-			pDesklet->container.iWidth,
-			pDesklet->container.iHeight,
-			1);
-		
-		cairo_t *pCairoContext = gdk_cairo_create (pShapeBitmap);
-		cairo_set_source_rgba (pCairoContext, 0.0f, 0.0f, 0.0f, 0.0f);
-		cairo_set_operator (pCairoContext, CAIRO_OPERATOR_SOURCE);
-		cairo_paint (pCairoContext);
-		cairo_set_source_rgba (pCairoContext, 1., 1., 1., 1.);
-		cairo_rectangle (pCairoContext,
+		GldiShape *pShapeBitmap = gldi_container_create_input_shape (CAIRO_CONTAINER (pDesklet),
 			pDesklet->container.iWidth - myDeskletsParam.iDeskletButtonSize,
 			pDesklet->container.iHeight - myDeskletsParam.iDeskletButtonSize,
 			myDeskletsParam.iDeskletButtonSize,
 			myDeskletsParam.iDeskletButtonSize);
-		cairo_fill (pCairoContext);
-		cairo_destroy (pCairoContext);
-		gtk_widget_input_shape_combine_mask (pDesklet->container.pWidget,
-			pShapeBitmap,
-			0,
-			0);
-		g_object_unref (pShapeBitmap);
-		//g_print ("input shape : %dx%d\n", pDesklet->container.iWidth, pDesklet->container.iHeight);
+		
+		gldi_container_set_input_shape (CAIRO_CONTAINER (pDesklet), pShapeBitmap);
+		
+		gldi_shape_destroy (pShapeBitmap);
 	}
 }
 
@@ -179,7 +142,7 @@ static gboolean _cairo_dock_write_desklet_size (CairoDesklet *pDesklet)
 			G_TYPE_STRING, "Desklet", "size", cSize,
 			G_TYPE_INVALID);
 		g_free (cSize);
-		cairo_dock_notify_on_object (&myDeskletsMgr, NOTIFICATION_CONFIGURE_DESKLET, pDesklet);
+		cairo_dock_notify_on_object (pDesklet, NOTIFICATION_CONFIGURE_DESKLET, pDesklet);
 	}
 	pDesklet->iSidWriteSize = 0;
 	pDesklet->iKnownWidth = pDesklet->container.iWidth;
@@ -189,9 +152,7 @@ static gboolean _cairo_dock_write_desklet_size (CairoDesklet *pDesklet)
 		pDesklet->iDesiredWidth = 0;
 		pDesklet->iDesiredHeight = 0;
 		
-		cairo_t *pCairoContext = cairo_dock_create_drawing_context_generic (CAIRO_CONTAINER (pDesklet));
 		cairo_dock_load_desklet_decorations (pDesklet);
-		cairo_destroy (pCairoContext);
 		
 		if (pDesklet->pIcon != NULL && pDesklet->pIcon->pModuleInstance != NULL)
 		{
@@ -222,7 +183,7 @@ static gboolean _cairo_dock_write_desklet_size (CairoDesklet *pDesklet)
 					{
 						pIcon->iImageWidth = pIcon->fWidth;
 						pIcon->iImageHeight = pIcon->fHeight;
-						cairo_dock_trigger_load_icon_buffers (pIcon, CAIRO_CONTAINER (pDesklet));
+						cairo_dock_trigger_load_icon_buffers (pIcon);
 					}
 				}
 				
@@ -237,7 +198,6 @@ static gboolean _cairo_dock_write_desklet_size (CairoDesklet *pDesklet)
 			gtk_widget_queue_draw (pDesklet->container.pWidget);  // sinon on ne redessine que l'interieur.
 		}
 		
-		Window Xid = GDK_WINDOW_XID (pDesklet->container.pWidget->window);
 		if (pDesklet->bSpaceReserved)  // l'espace est reserve, on reserve la nouvelle taille.
 		{
 			_cairo_dock_reserve_space_for_desklet (pDesklet, TRUE);
@@ -257,8 +217,7 @@ static gboolean _cairo_dock_write_desklet_position (CairoDesklet *pDesklet)
 		int iNumDesktop = -1;
 		if (! cairo_dock_desklet_is_sticky (pDesklet))
 		{
-			GdkWindow *window = pDesklet->container.pWidget->window;
-			Window Xid = GDK_WINDOW_XID (window);
+			Window Xid = gldi_container_get_Xid (CAIRO_CONTAINER (pDesklet));
 			cd_debug ("This window (%d) is not sticky", (int) Xid);
 			int iDesktop = cairo_dock_get_xwindow_desktop (Xid);
 			int iGlobalPositionX, iGlobalPositionY, iWidthExtent, iHeightExtent;
@@ -291,7 +250,7 @@ static gboolean _cairo_dock_write_desklet_position (CairoDesklet *pDesklet)
 			G_TYPE_INT, "Desklet", "y position", iRelativePositionY,
 			G_TYPE_INT, "Desklet", "num desktop", iNumDesktop,
 			G_TYPE_INVALID);
-		cairo_dock_notify_on_object (&myDeskletsMgr, NOTIFICATION_CONFIGURE_DESKLET, pDesklet);
+		cairo_dock_notify_on_object (pDesklet, NOTIFICATION_CONFIGURE_DESKLET, pDesklet);
 	}
 	
 	if (pDesklet->bSpaceReserved)  // l'espace est reserve, on reserve a la nouvelle position.
@@ -314,7 +273,7 @@ static gboolean on_configure_desklet (GtkWidget* pWidget,
 	{
 		if ((pEvent->width < pDesklet->container.iWidth || pEvent->height < pDesklet->container.iHeight) && (pDesklet->iDesiredWidth != 0 && pDesklet->iDesiredHeight != 0))
 		{
-			gdk_window_resize (pDesklet->container.pWidget->window,
+			gdk_window_resize (gldi_container_get_gdk_window (CAIRO_CONTAINER (pDesklet)),
 				pDesklet->iDesiredWidth,
 				pDesklet->iDesiredHeight);
 		}
@@ -324,18 +283,15 @@ static gboolean on_configure_desklet (GtkWidget* pWidget,
 		
 		if (g_bUseOpenGL)
 		{
-			GdkGLContext* pGlContext = gtk_widget_get_gl_context (pWidget);
-			GdkGLDrawable* pGlDrawable = gtk_widget_get_gl_drawable (pWidget);
 			GLsizei w = pEvent->width;
 			GLsizei h = pEvent->height;
-			if (!gdk_gl_drawable_gl_begin (pGlDrawable, pGlContext))
+			
+			if (! gldi_glx_make_current (CAIRO_CONTAINER (pDesklet)))
 				return FALSE;
 			
 			glViewport(0, 0, w, h);
 			
 			cairo_dock_set_perspective_view (CAIRO_CONTAINER (pDesklet));
-			
-			gdk_gl_drawable_gl_end (pGlDrawable);
 		}
 		
 		if (pDesklet->bNoInput)
@@ -386,8 +342,7 @@ static gboolean on_scroll_desklet (GtkWidget* pWidget,
 	if (! (pScroll->state & (GDK_SHIFT_MASK | GDK_CONTROL_MASK)))
 	{
 		Icon *icon = cairo_dock_find_clicked_icon_in_desklet (pDesklet);  // can be NULL
-		cairo_dock_notify_on_object (&myContainersMgr, NOTIFICATION_SCROLL_ICON, icon, pDesklet, pScroll->direction);
-		cairo_dock_notify_on_object (CAIRO_CONTAINER (pDesklet), NOTIFICATION_SCROLL_ICON, icon, pDesklet, pScroll->direction);
+		cairo_dock_notify_on_object (pDesklet, NOTIFICATION_SCROLL_ICON, icon, pDesklet, pScroll->direction);
 	}
 	return FALSE;
 }
@@ -397,7 +352,7 @@ static gboolean on_unmap_desklet (GtkWidget* pWidget,
 	CairoDesklet *pDesklet)
 {
 	cd_debug ("unmap desklet (bAllowMinimize:%d)\n", pDesklet->bAllowMinimize);
-	Window Xid = GDK_WINDOW_XID (pWidget->window);
+	Window Xid = gldi_container_get_Xid (CAIRO_CONTAINER (pDesklet));
 	if (pDesklet->iVisibility == CAIRO_DESKLET_ON_WIDGET_LAYER)  // on the widget layer, let pass the unmap event..
 	//if (cairo_dock_window_is_utility (Xid))  // sur la couche des widgets, on ne fait rien.
 		return FALSE;
@@ -473,7 +428,7 @@ static gboolean on_button_press_desklet(GtkWidget *pWidget,
 					G_TYPE_INT, "Desklet", "rotation", (int) (pDesklet->fRotation / G_PI * 180.),
 					G_TYPE_INVALID);
 				gtk_widget_queue_draw (pDesklet->container.pWidget);
-				cairo_dock_notify_on_object (&myDeskletsMgr, NOTIFICATION_CONFIGURE_DESKLET, pDesklet);
+				cairo_dock_notify_on_object (pDesklet, NOTIFICATION_CONFIGURE_DESKLET, pDesklet);
 			}
 			else if (pDesklet->retaching)
 			{
@@ -501,10 +456,10 @@ static gboolean on_button_press_desklet(GtkWidget *pWidget,
 						G_TYPE_INVALID);
 					_cairo_dock_set_desklet_input_shape (pDesklet);
 					gtk_widget_queue_draw (pDesklet->container.pWidget);
-					if (pDesklet->bNoInput)
-						cairo_dock_allow_widget_to_receive_data (pDesklet->container.pWidget, G_CALLBACK (on_drag_data_received_desklet), pDesklet);
+					/**if (pDesklet->bNoInput)
+						gldi_container_enable_drop (CAIRO_CONTAINER (pDesklet), G_CALLBACK (on_drag_data_received_desklet), pDesklet);
 					else
-						cairo_dock_disallow_widget_to_receive_data (pDesklet->container.pWidget);
+						gldi_container_disable_drop (CAIRO_CONTAINER (pDesklet));*/
 				}
 			}
 			else if (pDesklet->rotatingY)
@@ -526,8 +481,7 @@ static gboolean on_button_press_desklet(GtkWidget *pWidget,
 			else
 			{
 				Icon *pClickedIcon = cairo_dock_find_clicked_icon_in_desklet (pDesklet);
-				cairo_dock_notify_on_object (&myContainersMgr, NOTIFICATION_CLICK_ICON, pClickedIcon, pDesklet, pButton->state);
-				cairo_dock_notify_on_object (CAIRO_CONTAINER (pDesklet), NOTIFICATION_CLICK_ICON, pClickedIcon, pDesklet, pButton->state);
+				cairo_dock_notify_on_object (pDesklet, NOTIFICATION_CLICK_ICON, pClickedIcon, pDesklet, pButton->state);
 			}
 			// prudence.
 			pDesklet->rotating = FALSE;
@@ -541,8 +495,7 @@ static gboolean on_button_press_desklet(GtkWidget *pWidget,
 			if (! (pButton->x < myDeskletsParam.iDeskletButtonSize && pButton->y < myDeskletsParam.iDeskletButtonSize) && ! (pButton->x > (pDesklet->container.iWidth - myDeskletsParam.iDeskletButtonSize)/2 && pButton->x < (pDesklet->container.iWidth + myDeskletsParam.iDeskletButtonSize)/2 && pButton->y < myDeskletsParam.iDeskletButtonSize))
 			{
 				Icon *pClickedIcon = cairo_dock_find_clicked_icon_in_desklet (pDesklet);  // can be NULL
-				cairo_dock_notify_on_object (&myContainersMgr, NOTIFICATION_DOUBLE_CLICK_ICON, pClickedIcon, pDesklet);
-				cairo_dock_notify_on_object (CAIRO_CONTAINER (pDesklet), NOTIFICATION_DOUBLE_CLICK_ICON, pClickedIcon, pDesklet);
+				cairo_dock_notify_on_object (pDesklet, NOTIFICATION_DOUBLE_CLICK_ICON, pClickedIcon, pDesklet);
 			}
 		}
 	}
@@ -563,7 +516,7 @@ static gboolean on_button_press_desklet(GtkWidget *pWidget,
 			cairo_dock_update_conf_file (pDesklet->pIcon->pModuleInstance->cConfFilePath,
 				G_TYPE_INT, "Desklet", "rotation", 0,
 				G_TYPE_INVALID);
-			cairo_dock_notify_on_object (&myDeskletsMgr, NOTIFICATION_CONFIGURE_DESKLET, pDesklet);
+			cairo_dock_notify_on_object (pDesklet, NOTIFICATION_CONFIGURE_DESKLET, pDesklet);
 		}
 		else if (pButton->x > (pDesklet->container.iWidth - myDeskletsParam.iDeskletButtonSize)/2 && pButton->x < (pDesklet->container.iWidth + myDeskletsParam.iDeskletButtonSize)/2 && pButton->y < myDeskletsParam.iDeskletButtonSize)
 		{
@@ -583,8 +536,7 @@ static gboolean on_button_press_desklet(GtkWidget *pWidget,
 		}
 		else
 		{
-			cairo_dock_notify_on_object (&myContainersMgr, NOTIFICATION_MIDDLE_CLICK_ICON, pDesklet->pIcon, pDesklet);
-			cairo_dock_notify_on_object (CAIRO_CONTAINER (pDesklet), NOTIFICATION_MIDDLE_CLICK_ICON, pDesklet->pIcon, pDesklet);
+			cairo_dock_notify_on_object (pDesklet, NOTIFICATION_MIDDLE_CLICK_ICON, pDesklet->pIcon, pDesklet);
 		}
 	}
 	return FALSE;
@@ -595,7 +547,7 @@ static void on_drag_data_received_desklet (GtkWidget *pWidget, GdkDragContext *d
 	//g_print ("%s (%dx%d)\n", __func__, x, y);
 	
 	//\_________________ On recupere l'URI.
-	gchar *cReceivedData = (gchar *) selection_data->data;  // gtk_selection_data_get_text
+	gchar *cReceivedData = (gchar *) gtk_selection_data_get_text (selection_data);
 	g_return_if_fail (cReceivedData != NULL);
 	int length = strlen (cReceivedData);
 	if (cReceivedData[length-1] == '\n')
@@ -622,8 +574,7 @@ static gboolean on_motion_notify_desklet (GtkWidget *pWidget,
 		pDesklet->container.iMouseX = pMotion->x;
 		pDesklet->container.iMouseY = pMotion->y;
 		gboolean bStartAnimation = FALSE;
-		cairo_dock_notify_on_object (&myContainersMgr, NOTIFICATION_MOUSE_MOVED, pDesklet, &bStartAnimation);
-		cairo_dock_notify_on_object (CAIRO_CONTAINER (pDesklet), NOTIFICATION_MOUSE_MOVED, pDesklet, &bStartAnimation);
+		cairo_dock_notify_on_object (pDesklet, NOTIFICATION_MOUSE_MOVED, pDesklet, &bStartAnimation);
 		if (bStartAnimation)
 			cairo_dock_launch_animation (CAIRO_CONTAINER (pDesklet));
 	}
@@ -671,8 +622,7 @@ static gboolean on_motion_notify_desklet (GtkWidget *pWidget,
 				pIcon->bPointed = TRUE;
 				
 				//g_print ("on survole %s\n", pIcon->cName);
-				cairo_dock_notify_on_object (&myContainersMgr, NOTIFICATION_ENTER_ICON, pIcon, pDesklet, &bStartAnimation);
-				cairo_dock_notify_on_object (CAIRO_CONTAINER (pDesklet), NOTIFICATION_ENTER_ICON, pIcon, pDesklet, &bStartAnimation);
+				cairo_dock_notify_on_object (pDesklet, NOTIFICATION_ENTER_ICON, pIcon, pDesklet, &bStartAnimation);
 			}
 		}
 		else
@@ -683,8 +633,7 @@ static gboolean on_motion_notify_desklet (GtkWidget *pWidget,
 				pPointedIcon->bPointed = FALSE;
 				
 				//g_print ("kedal\n");
-				cairo_dock_notify_on_object (&myContainersMgr, NOTIFICATION_ENTER_ICON, pPointedIcon, pDesklet, &bStartAnimation);
-				cairo_dock_notify_on_object (CAIRO_CONTAINER (pDesklet), NOTIFICATION_ENTER_ICON, pPointedIcon, pDesklet, &bStartAnimation);
+				cairo_dock_notify_on_object (pDesklet, NOTIFICATION_ENTER_ICON, pPointedIcon, pDesklet, &bStartAnimation);
 			}
 		}
 		if (bStartAnimation)
@@ -716,14 +665,10 @@ static gboolean on_enter_desklet (GtkWidget* pWidget,
 		pDesklet->container.bInside = TRUE;
 		gtk_widget_queue_draw (pWidget);  // redessin des boutons.
 		
-		///if (g_bUseOpenGL)
-		{
-			gboolean bStartAnimation = FALSE;
-			cairo_dock_notify_on_object (&myDeskletsMgr, NOTIFICATION_ENTER_DESKLET, pDesklet, &bStartAnimation);
-			cairo_dock_notify_on_object (pDesklet, NOTIFICATION_ENTER_DESKLET, pDesklet, &bStartAnimation);
-			if (bStartAnimation)
-				cairo_dock_launch_animation (CAIRO_CONTAINER (pDesklet));
-		}
+		gboolean bStartAnimation = FALSE;
+		cairo_dock_notify_on_object (pDesklet, NOTIFICATION_ENTER_DESKLET, pDesklet, &bStartAnimation);
+		if (bStartAnimation)
+			cairo_dock_launch_animation (CAIRO_CONTAINER (pDesklet));
 	}
 	return FALSE;
 }
@@ -733,7 +678,17 @@ static gboolean on_leave_desklet (GtkWidget* pWidget,
 {
 	//g_print ("%s (%d)\n", __func__, pDesklet->container.bInside);
 	int iMouseX, iMouseY;
-	gdk_window_get_pointer (pWidget->window, &iMouseX, &iMouseY, NULL);
+	if (pEvent != NULL)
+	{
+		iMouseX = pEvent->x_root;
+		iMouseY = pEvent->y_root;
+	}
+	else
+	{
+		gldi_container_update_mouse_position (CAIRO_CONTAINER (pDesklet));
+		iMouseX = pDesklet->container.iMouseX;
+		iMouseY = pDesklet->container.iMouseY;
+	}
 	if (gtk_bin_get_child (GTK_BIN (pDesklet->container.pWidget)) != NULL && iMouseX > 0 && iMouseX < pDesklet->container.iWidth && iMouseY > 0 && iMouseY < pDesklet->container.iHeight)  // en fait on est dans un widget fils, donc on ne fait rien.
 	{
 		return FALSE;
@@ -743,7 +698,6 @@ static gboolean on_leave_desklet (GtkWidget* pWidget,
 	gtk_widget_queue_draw (pWidget);  // redessin des boutons.
 	
 	gboolean bStartAnimation = FALSE;
-	cairo_dock_notify_on_object (&myDeskletsMgr, NOTIFICATION_LEAVE_DESKLET, pDesklet, &bStartAnimation);
 	cairo_dock_notify_on_object (pDesklet, NOTIFICATION_LEAVE_DESKLET, pDesklet, &bStartAnimation);
 	if (bStartAnimation)
 		cairo_dock_launch_animation (CAIRO_CONTAINER (pDesklet));
@@ -756,11 +710,51 @@ static gboolean on_leave_desklet (GtkWidget* pWidget,
  /// FACTORY ///
 ///////////////
 
-static void _cairo_dock_set_icon_size (CairoContainer *pContainer, Icon *icon)
+
+static gboolean _cairo_desklet_animation_loop (CairoContainer *pContainer)
 {
 	CairoDesklet *pDesklet = CAIRO_DESKLET (pContainer);
-	/**if (pDesklet->pRenderer && pDesklet->pRenderer->set_icon_size)
-		pDesklet->pRenderer->set_icon_size (pDesklet, icon);*/
+	gboolean bContinue = FALSE;
+	gboolean bUpdateSlowAnimation = FALSE;
+	pContainer->iAnimationStep ++;
+	if (pContainer->iAnimationStep * pContainer->iAnimationDeltaT >= CAIRO_DOCK_MIN_SLOW_DELTA_T)
+	{
+		bUpdateSlowAnimation = TRUE;
+		pContainer->iAnimationStep = 0;
+		pContainer->bKeepSlowAnimation = FALSE;
+	}
+	
+	if (pDesklet->pIcon != NULL)
+	{
+		gboolean bIconIsAnimating = FALSE;
+		
+		if (bUpdateSlowAnimation)
+		{
+			cairo_dock_notify_on_object (pDesklet->pIcon, NOTIFICATION_UPDATE_ICON_SLOW, pDesklet->pIcon, pDesklet, &bIconIsAnimating);
+			pDesklet->container.bKeepSlowAnimation |= bIconIsAnimating;
+		}
+		
+		cairo_dock_notify_on_object (pDesklet->pIcon, NOTIFICATION_UPDATE_ICON, pDesklet->pIcon, pDesklet, &bIconIsAnimating);
+		if (! bIconIsAnimating)
+			pDesklet->pIcon->iAnimationState = CAIRO_DOCK_STATE_REST;
+		else
+			bContinue = TRUE;
+	}
+	
+	if (bUpdateSlowAnimation)
+	{
+		cairo_dock_notify_on_object (pDesklet, NOTIFICATION_UPDATE_SLOW, pDesklet, &pContainer->bKeepSlowAnimation);
+	}
+	
+	cairo_dock_notify_on_object (pDesklet, NOTIFICATION_UPDATE, pDesklet, &bContinue);
+	
+	if (! bContinue && ! pContainer->bKeepSlowAnimation)
+	{
+		pContainer->iSidGLAnimation = 0;
+		return FALSE;
+	}
+	else
+		return TRUE;
 }
 
 CairoDesklet *cairo_dock_new_desklet (void)
@@ -773,18 +767,24 @@ CairoDesklet *cairo_dock_new_desklet (void)
 	pDesklet->container.fRatio = 1;
 	pDesklet->container.fRatio = 1;
 	
-	pDesklet->container.iface.set_icon_size = _cairo_dock_set_icon_size;
+	pDesklet->container.iface.animation_loop = _cairo_desklet_animation_loop;
 	
 	GtkWidget* pWindow = cairo_dock_init_container (CAIRO_CONTAINER (pDesklet));
-	cairo_dock_install_notifications_on_object (pDesklet, NB_NOTIFICATIONS_DESKLET);
+	gldi_object_set_manager (GLDI_OBJECT (pDesklet), GLDI_MANAGER (&myDeskletsMgr));
 	
 	gtk_window_set_title (GTK_WINDOW(pWindow), "cairo-dock-desklet");
-	gtk_widget_add_events( pWindow, GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK | GDK_FOCUS_CHANGE_MASK);
-	gtk_container_set_border_width(GTK_CONTAINER(pWindow), 2);  // comme ca.
-	gtk_window_set_default_size(GTK_WINDOW(pWindow), 10, 10);  // idem.
-
+	gtk_widget_add_events( pWindow,
+		GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_SCROLL_MASK |
+		GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK | GDK_FOCUS_CHANGE_MASK |
+		GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK);
+	gtk_container_set_border_width(GTK_CONTAINER(pWindow), 1);  // comme ca.
+	
 	g_signal_connect (G_OBJECT (pWindow),
+		#if (GTK_MAJOR_VERSION < 3)
 		"expose-event",
+		#else
+		"draw",
+		#endif
 		G_CALLBACK (on_expose_desklet),
 		pDesklet);
 	g_signal_connect (G_OBJECT (pWindow),
@@ -827,7 +827,7 @@ CairoDesklet *cairo_dock_new_desklet (void)
 		"scroll-event",
 		G_CALLBACK (on_scroll_desklet),
 		pDesklet);
-	cairo_dock_allow_widget_to_receive_data (pWindow, G_CALLBACK (on_drag_data_received_desklet), pDesklet);
+	gldi_container_enable_drop (CAIRO_CONTAINER (pDesklet), G_CALLBACK (on_drag_data_received_desklet), pDesklet);
 	
 	gtk_widget_show_all (pWindow);
 	
@@ -843,12 +843,10 @@ void cairo_dock_free_desklet (CairoDesklet *pDesklet)
 		g_source_remove (pDesklet->iSidWriteSize);
 	if (pDesklet->iSidWritePosition != 0)
 		g_source_remove (pDesklet->iSidWritePosition);
-	cairo_dock_notify_on_object (&myDeskletsMgr, NOTIFICATION_STOP_DESKLET, pDesklet);
-	cairo_dock_notify_on_object (pDesklet, NOTIFICATION_STOP_DESKLET, pDesklet);
 	
 	cairo_dock_steal_interactive_widget_from_desklet (pDesklet);
 
-	cairo_dock_finish_container (CAIRO_CONTAINER (pDesklet));
+	cairo_dock_finish_container (CAIRO_CONTAINER (pDesklet));  // -> NOTIFICATION_DESTROY
 	
 	if (pDesklet->pRenderer != NULL)
 	{
@@ -882,7 +880,7 @@ void cairo_dock_configure_desklet (CairoDesklet *pDesklet, CairoDeskletAttribute
 	{
 		pDesklet->iDesiredWidth = pAttribute->iDeskletWidth;
 		pDesklet->iDesiredHeight = pAttribute->iDeskletHeight;
-		gdk_window_resize (pDesklet->container.pWidget->window,
+		gdk_window_resize (gldi_container_get_gdk_window (CAIRO_CONTAINER (pDesklet)),
 			pAttribute->iDeskletWidth,
 			pAttribute->iDeskletHeight);
 	}
@@ -898,7 +896,7 @@ void cairo_dock_configure_desklet (CairoDesklet *pDesklet, CairoDeskletAttribute
 	iAbsolutePositionY = MAX (0, MIN (g_desktopGeometry.iXScreenHeight[CAIRO_DOCK_HORIZONTAL] - pAttribute->iDeskletHeight, iAbsolutePositionY));
 	
 	if (pAttribute->bOnAllDesktops)
-		gdk_window_move(pDesklet->container.pWidget->window,
+		gdk_window_move (gldi_container_get_gdk_window (CAIRO_CONTAINER (pDesklet)),
 			iAbsolutePositionX,
 			iAbsolutePositionY);
 	//g_print (" let's place the deklet at (%d;%d)", iAbsolutePositionX, iAbsolutePositionY);
@@ -912,7 +910,7 @@ void cairo_dock_configure_desklet (CairoDesklet *pDesklet, CairoDeskletAttribute
 	else
 	{
 		gtk_window_unstick (GTK_WINDOW (pDesklet->container.pWidget));
-		Window Xid = GDK_WINDOW_XID (pDesklet->container.pWidget->window);
+		Window Xid = gldi_container_get_Xid (CAIRO_CONTAINER (pDesklet));
 		if (g_desktopGeometry.iNbViewportX > 0 && g_desktopGeometry.iNbViewportY > 0)
 		{
 			int iNumDesktop, iNumViewportX, iNumViewportY;
@@ -934,8 +932,8 @@ void cairo_dock_configure_desklet (CairoDesklet *pDesklet, CairoDeskletAttribute
 	}
 	pDesklet->bPositionLocked = pAttribute->bPositionLocked;
 	pDesklet->bNoInput = pAttribute->bNoInput;
-	if (pDesklet->bNoInput)
-		cairo_dock_disallow_widget_to_receive_data (pDesklet->container.pWidget);
+	/**if (pDesklet->bNoInput)
+		gldi_container_disable_drop (CAIRO_CONTAINER (pDesklet));*/
 	pDesklet->fRotation = pAttribute->iRotation / 180. * G_PI ;
 	pDesklet->fDepthRotationY = pAttribute->iDepthRotationY / 180. * G_PI ;
 	pDesklet->fDepthRotationX = pAttribute->iDepthRotationX / 180. * G_PI ;
@@ -950,9 +948,7 @@ void cairo_dock_configure_desklet (CairoDesklet *pDesklet, CairoDeskletAttribute
 	//cd_debug ("%s (%dx%d ; %d)", __func__, pDesklet->iDesiredWidth, pDesklet->iDesiredHeight, pDesklet->iSidWriteSize);
 	if (pDesklet->iDesiredWidth == 0 && pDesklet->iDesiredHeight == 0 && pDesklet->iSidWriteSize == 0)
 	{
-		cairo_t *pCairoContext = cairo_dock_create_drawing_context_generic (CAIRO_CONTAINER (pDesklet));
 		cairo_dock_load_desklet_decorations (pDesklet);
-		cairo_destroy (pCairoContext);
 	}
 }
 
@@ -1028,7 +1024,7 @@ void cairo_dock_add_interactive_widget_to_desklet_full (GtkWidget *pInteractiveW
 	}
 	
 	//gtk_container_add (GTK_CONTAINER (pDesklet->container.pWidget), pInteractiveWidget);
-	GtkWidget *pHBox = gtk_hbox_new (FALSE, 0);
+	GtkWidget *pHBox = _gtk_hbox_new (0);
 	gtk_container_add (GTK_CONTAINER (pDesklet->container.pWidget), pHBox);
 	
 	gtk_box_pack_start (GTK_BOX (pHBox), pInteractiveWidget, TRUE, TRUE, 0);
@@ -1036,8 +1032,8 @@ void cairo_dock_add_interactive_widget_to_desklet_full (GtkWidget *pInteractiveW
 	
 	if (iRightMargin != 0)
 	{
-		GtkWidget *pMarginBox = gtk_vbox_new (FALSE, 0);
-		gtk_widget_set (pMarginBox, "width-request", iRightMargin, NULL);
+		GtkWidget *pMarginBox = _gtk_vbox_new (0);
+		g_object_set (pMarginBox, "width-request", iRightMargin, NULL);
 		gtk_box_pack_start (GTK_BOX (pHBox), pMarginBox, FALSE, FALSE, 0);  // a tester ...
 	}
 	
@@ -1057,12 +1053,12 @@ void cairo_dock_set_desklet_margin (CairoDesklet *pDesklet, int iRightMargin)
 			if (pChildList->next != NULL)
 			{
 				GtkWidget *pMarginBox = GTK_WIDGET (pChildList->next->data);
-				gtk_widget_set (pMarginBox, "width-request", iRightMargin, NULL);
+				g_object_set (pMarginBox, "width-request", iRightMargin, NULL);
 			}
 			else  // on rajoute le widget de la marge.
 			{
-				GtkWidget *pMarginBox = gtk_vbox_new (FALSE, 0);
-				gtk_widget_set (pMarginBox, "width-request", iRightMargin, NULL);
+				GtkWidget *pMarginBox = _gtk_vbox_new (0);
+				g_object_set (pMarginBox, "width-request", iRightMargin, NULL);
 				gtk_box_pack_start (GTK_BOX (pHBox), pMarginBox, FALSE, FALSE, 0);
 			}
 			g_list_free (pChildList);
@@ -1118,7 +1114,7 @@ void cairo_dock_zoom_out_desklet (CairoDesklet *pDesklet)
 static void _cairo_dock_reserve_space_for_desklet (CairoDesklet *pDesklet, gboolean bReserve)
 {
 	cd_debug ("%s (%d)\n", __func__, bReserve);
-	Window Xid = GDK_WINDOW_XID (pDesklet->container.pWidget->window);
+	Window Xid = gldi_container_get_Xid (CAIRO_CONTAINER (pDesklet));
 	int left=0, right=0, top=0, bottom=0;
 	int left_start_y=0, left_end_y=0, right_start_y=0, right_end_y=0, top_start_x=0, top_end_x=0, bottom_start_x=0, bottom_end_x=0;
 	int iHeight = pDesklet->container.iHeight, iWidth = pDesklet->container.iWidth;
@@ -1170,7 +1166,7 @@ void cairo_dock_set_desklet_accessibility (CairoDesklet *pDesklet, CairoDeskletV
 	
 	gtk_window_set_keep_above (GTK_WINDOW (pDesklet->container.pWidget), iVisibility == CAIRO_DESKLET_KEEP_ABOVE);
 
-	Window Xid = GDK_WINDOW_XID (pDesklet->container.pWidget->window);
+	Window Xid = gldi_container_get_Xid (CAIRO_CONTAINER (pDesklet));
 	cairo_dock_wm_set_on_widget_layer (Xid, iVisibility == CAIRO_DESKLET_ON_WIDGET_LAYER);
 	
 	if (iVisibility == CAIRO_DESKLET_RESERVE_SPACE)
@@ -1222,9 +1218,9 @@ void cairo_dock_set_desklet_sticky (CairoDesklet *pDesklet, gboolean bSticky)
 
 gboolean cairo_dock_desklet_is_sticky (CairoDesklet *pDesklet)
 {
-	GdkWindow *window = pDesklet->container.pWidget->window;
-	#if (GDK_MAJOR_VERSION >= 3)
-	return (window->state & GDK_WINDOW_STATE_STICKY);  // API change: http://mail.gnome.org/archives/commits-list/2010-July/msg00002.html
+	GdkWindow *window = gldi_container_get_gdk_window (CAIRO_CONTAINER (pDesklet));
+	#if (GTK_MAJOR_VERSION >= 3)
+	return ((gdk_window_get_state (window)) & GDK_WINDOW_STATE_STICKY);
 	#else
 	return (((GdkWindowObject*) window)->state & GDK_WINDOW_STATE_STICKY);
 	#endif
@@ -1263,7 +1259,7 @@ void cairo_dock_update_desklet_icons (CairoDesklet *pDesklet)
 		{
 			pIcon->iImageWidth = pIcon->fWidth;
 			pIcon->iImageHeight = pIcon->fHeight;
-			cairo_dock_trigger_load_icon_buffers (pIcon, CAIRO_CONTAINER (pDesklet));
+			cairo_dock_trigger_load_icon_buffers (pIcon);
 		}
 	}
 	
@@ -1281,7 +1277,7 @@ void cairo_dock_insert_icon_in_desklet (Icon *icon, CairoDesklet *pDesklet)
 	pDesklet->icons = g_list_insert_sorted (pDesklet->icons,
 		icon,
 		(GCompareFunc)cairo_dock_compare_icons_order);
-	icon->pContainerForLoad = CAIRO_CONTAINER (pDesklet);
+	cairo_dock_set_icon_container (icon, pDesklet);
 	
 	// calculate icons
 	cairo_dock_update_desklet_icons (pDesklet);
@@ -1299,7 +1295,7 @@ gboolean cairo_dock_detach_icon_from_desklet (Icon *icon, CairoDesklet *pDesklet
 	// remove icon
 	pDesklet->icons = g_list_delete_link (pDesklet->icons, ic);
 	ic =  NULL;
-	icon->pContainerForLoad = NULL;
+	cairo_dock_set_icon_container (icon, NULL);
 	
 	// calculate icons
 	cairo_dock_update_desklet_icons (pDesklet);
