@@ -1,27 +1,27 @@
-/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 2; tab-width: 2 -*- */
-/*
-** Login : <ctaf42@gmail.com>
-** Started on  Sun Jan 27 18:35:38 2008 Cedric GESTES
-** $Id$
-**
-** Author(s)
-**  - Cedric GESTES <ctaf42@gmail.com>
-**  - Fabrice REY
-**
-** Copyright (C) 2008 Cedric GESTES
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 3 of the License, or
-** (at your option) any later version.
-**
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-** GNU General Public License for more details.
-**
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+/**
+* This file is a part of the Cairo-Dock project
+* Login : <ctaf42@gmail.com>
+* Started on  Sun Jan 27 18:35:38 2008 Cedric GESTES
+* $Id$
+*
+* Author(s)
+*  - Cedric GESTES <ctaf42@gmail.com>
+*  - Fabrice REY
+*
+* Copyright (C) 2008 Cedric GESTES
+* E-mail    : see the 'copyright' file.
+* 
+* This program is free software; you can redistribute it and/or
+* modify it under the terms of the GNU General Public License
+* as published by the Free Software Foundation; either version 3
+* of the License, or (at your option) any later version.
+*
+* This program is distributed in the hope that it will be useful,
+* but WITHOUT ANY WARRANTY; without even the implied warranty of
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+* GNU General Public License for more details.
+* You should have received a copy of the GNU General Public License
+* along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 
@@ -30,16 +30,15 @@
 #include <math.h>
 #include <time.h>
 
-#include <gtk/gtkgl.h>
 #include <GL/gl.h> 
 #include <GL/glu.h> 
 #include <GL/glx.h> 
-#include <gdk/x11/gdkglx.h>
 #include <gdk/gdkx.h>
 
 #include "gldi-config.h"
 #include "cairo-dock-module-factory.h"  // cairo_dock_pre_read_module_instance_config
 #include "cairo-dock-config.h"
+#include "cairo-dock-icon-facility.h"  // cairo_dock_set_icon_container
 #include "cairo-dock-X-manager.h"
 #include "cairo-dock-notifications.h"
 #include "cairo-dock-log.h"
@@ -471,6 +470,9 @@ CairoDesklet *cairo_dock_create_desklet (Icon *pIcon, CairoDeskletAttribute *pAt
 {
 	CairoDesklet *pDesklet = cairo_dock_new_desklet ();
 	pDesklet->pIcon = pIcon;
+	cairo_dock_set_icon_container (pIcon, pDesklet);
+	if (CAIRO_DOCK_IS_APPLET (pIcon))
+		gtk_window_set_title (GTK_WINDOW (pDesklet->container.pWidget), pIcon->pModuleInstance->pModule->pVisitCard->cModuleName);
 	
 	if (pAttributes != NULL)
 		cairo_dock_configure_desklet (pDesklet, pAttributes);
@@ -482,8 +484,7 @@ CairoDesklet *cairo_dock_create_desklet (Icon *pIcon, CairoDeskletAttribute *pAt
 	
 	s_pDeskletList = g_list_prepend (s_pDeskletList, pDesklet);
 	
-	cairo_dock_notify_on_object (&myDeskletsMgr, NOTIFICATION_NEW_DESKLET, pDesklet);
-	cairo_dock_notify_on_object (pDesklet, NOTIFICATION_NEW_DESKLET, pDesklet);
+	cairo_dock_notify_on_object (pDesklet, NOTIFICATION_NEW_DESKLET, pDesklet);  // in practice, only the notification on the manager can be caught, since 'pDesklet' is brand new.
 	
 	return pDesklet;
 }
@@ -567,7 +568,7 @@ void cairo_dock_reload_desklets_decorations (gboolean bDefaultThemeOnly)  // tou
 static void _cairo_dock_set_one_desklet_visible (CairoDesklet *pDesklet, gpointer data)
 {
 	gboolean bOnWidgetLayerToo = GPOINTER_TO_INT (data);
-	Window Xid = GDK_WINDOW_XID (pDesklet->container.pWidget->window);
+	Window Xid = gldi_container_get_Xid (CAIRO_CONTAINER (pDesklet));
 	//gboolean bIsOnWidgetLayer = cairo_dock_window_is_utility (Xid);
 	gboolean bIsOnWidgetLayer = (pDesklet->iVisibility == CAIRO_DESKLET_ON_WIDGET_LAYER);
 	if (bOnWidgetLayerToo || ! bIsOnWidgetLayer)
@@ -624,7 +625,7 @@ CairoDesklet *cairo_dock_get_desklet_by_Xid (Window Xid)
 	for (dl = s_pDeskletList; dl != NULL; dl = dl->next)
 	{
 		pDesklet = dl->data;
-		if (GDK_WINDOW_XID (pDesklet->container.pWidget->window) == Xid)
+		if (gldi_container_get_Xid (CAIRO_CONTAINER (pDesklet)) == Xid)
 			return pDesklet;
 	}
 	return NULL;
@@ -637,9 +638,7 @@ static Icon *_cairo_dock_pick_icon_on_opengl_desklet (CairoDesklet *pDesklet)
 	GLint hits=0;
 	GLint viewport[4];
 	
-	GdkGLContext* pGlContext = gtk_widget_get_gl_context (pDesklet->container.pWidget);
-	GdkGLDrawable* pGlDrawable = gtk_widget_get_gl_drawable (pDesklet->container.pWidget);
-	if (!gdk_gl_drawable_gl_begin (pGlDrawable, pGlContext))
+	if (! gldi_glx_make_current (CAIRO_CONTAINER (pDesklet)))
 		return NULL;
 	
 	glGetIntegerv (GL_VIEWPORT, viewport);
@@ -777,7 +776,6 @@ static Icon *_cairo_dock_pick_icon_on_opengl_desklet (CairoDesklet *pDesklet)
 		}
 	}
 	
-	gdk_gl_drawable_gl_end (pGlDrawable);
 	return pFoundIcon;
 }
 Icon *cairo_dock_find_clicked_icon_in_desklet (CairoDesklet *pDesklet)
@@ -934,7 +932,7 @@ static void unload (void)
 static void init (void)
 {
 	cairo_dock_register_notification_on_object (&myDeskletsMgr,
-		NOTIFICATION_UPDATE_DESKLET,
+		NOTIFICATION_UPDATE,
 		(CairoDockNotificationFunc) _cairo_dock_update_desklet_notification,
 		CAIRO_DOCK_RUN_FIRST, NULL);
 	cairo_dock_register_notification_on_object (&myDeskletsMgr,
@@ -946,7 +944,7 @@ static void init (void)
 		(CairoDockNotificationFunc) _cairo_dock_enter_leave_desklet_notification,
 		CAIRO_DOCK_RUN_FIRST, NULL);
 	cairo_dock_register_notification_on_object (&myDeskletsMgr,
-		NOTIFICATION_RENDER_DESKLET,
+		NOTIFICATION_RENDER,
 		(CairoDockNotificationFunc) _cairo_dock_render_desklet_notification,
 		CAIRO_DOCK_RUN_FIRST, NULL);
 	s_iStartupTime = time (NULL);  // on startup, the WM can take a long time before it has positionned all the desklets. To avoid irrelevant configure events, we set a delay.
@@ -980,6 +978,7 @@ void gldi_register_desklets_manager (void)
 	myDeskletsMgr.mgr.pData = (GldiManagerDataPtr)NULL;
 	// signals
 	cairo_dock_install_notifications_on_object (&myDeskletsMgr, NB_NOTIFICATIONS_DESKLET);
+	gldi_object_set_manager (GLDI_OBJECT (&myDeskletsMgr), GLDI_MANAGER (&myContainersMgr));
 	// register
 	gldi_register_manager (GLDI_MANAGER(&myDeskletsMgr));
 }
