@@ -21,12 +21,11 @@
 #include <string.h>
 #include <math.h>
 #include <gdk/gdkkeysyms.h>
+#include <gdk/gdkx.h> // GDK_WINDOW_XID
 
-#include <gtk/gtkgl.h>
 #include <GL/gl.h>
 #include <GL/glu.h>
 #include <GL/glx.h>
-#include <gdk/x11/gdkglx.h>
 
 #include "cairo-dock-icon-factory.h"
 #include "cairo-dock-icon-facility.h"
@@ -83,23 +82,21 @@ static void _cairo_dock_compute_dialog_sizes (CairoDialog *pDialog)
 }
 
 static gboolean on_expose_dialog (GtkWidget *pWidget,
+#if (GTK_MAJOR_VERSION < 3)
 	GdkEventExpose *pExpose,
+#else
+	cairo_t *ctx,
+#endif
 	CairoDialog *pDialog)
 {
 	//g_print ("%s (%dx%d ; %d;%d)\n", __func__, pDialog->container.iWidth, pDialog->container.iHeight, pExpose->area.x, pExpose->area.y);
 	int x, y;
-	if (0 && g_bUseOpenGL && (pDialog->pDecorator == NULL || pDialog->pDecorator->render_opengl != NULL) && (pDialog->pRenderer == NULL || pDialog->pRenderer->render_opengl != NULL))
+	// OpenGL renderers are not ready for dialogs.
+	/*if (g_bUseOpenGL && (pDialog->pDecorator == NULL || pDialog->pDecorator->render_opengl != NULL) && (pDialog->pRenderer == NULL || pDialog->pRenderer->render_opengl != NULL))
 	{
-		GdkGLContext *pGlContext = gtk_widget_get_gl_context (pDialog->container.pWidget);
-		GdkGLDrawable *pGlDrawable = gtk_widget_get_gl_drawable (pDialog->container.pWidget);
-		if (!gdk_gl_drawable_gl_begin (pGlDrawable, pGlContext))
+		if (! gldi_glx_begin_draw_container (CAIRO_CONTAINER (pDialog)))
 			return FALSE;
 		
-		glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
-		glLoadIdentity ();
-		
-		cairo_dock_apply_desktop_background_opengl (CAIRO_CONTAINER (pDialog));
-		
 		if (pDialog->pDecorator != NULL && pDialog->pDecorator->render_opengl != NULL)
 		{
 			glPushMatrix ();
@@ -107,22 +104,29 @@ static gboolean on_expose_dialog (GtkWidget *pWidget,
 			glPopMatrix ();
 		}
 		
-		cairo_dock_notify_on_object (&myDialogsMgr, NOTIFICATION_RENDER_DIALOG, pDialog, NULL);
-		cairo_dock_notify_on_object (pDialog, NOTIFICATION_RENDER_DIALOG, pDialog, NULL);
+		cairo_dock_notify_on_object (pDialog, NOTIFICATION_RENDER, pDialog, NULL);
 		
-		if (gdk_gl_drawable_is_double_buffered (pGlDrawable))
-			gdk_gl_drawable_swap_buffers (pGlDrawable);
-		else
-			glFlush ();
-		gdk_gl_drawable_gl_end (pGlDrawable);
+		gldi_glx_end_draw_container (CAIRO_CONTAINER (pDialog));
 	}
 	else
-	{
+	{*/
 		cairo_t *pCairoContext;
 		
-		if ((pExpose->area.x != 0 || pExpose->area.y != 0))
+		GdkRectangle area;
+		#if (GTK_MAJOR_VERSION < 3)
+		memcpy (&area, &pExpose->area, sizeof (GdkRectangle));
+		#else
+		double x1, x2, y1, y2;
+		cairo_clip_extents (ctx, &x1, &y1, &x2, &y2);
+		area.x = x1;
+		area.y = y1;
+		area.width = x2 - x1;
+		area.height = y2 - y1;  /// or the opposite ?...
+		#endif
+		
+		if (area.x != 0 || area.y != 0)
 		{
-			pCairoContext = cairo_dock_create_drawing_context_on_area (CAIRO_CONTAINER (pDialog), &pExpose->area, myDialogsParam.fDialogColor);
+			pCairoContext = cairo_dock_create_drawing_context_on_area (CAIRO_CONTAINER (pDialog), &area, myDialogsParam.fDialogColor);
 		}
 		else
 		{
@@ -136,8 +140,7 @@ static gboolean on_expose_dialog (GtkWidget *pWidget,
 			cairo_restore (pCairoContext);
 		}
 		
-		cairo_dock_notify_on_object (&myDialogsMgr, NOTIFICATION_RENDER_DIALOG, pDialog, pCairoContext);
-		cairo_dock_notify_on_object (pDialog, NOTIFICATION_RENDER_DIALOG, pDialog, pCairoContext);
+		cairo_dock_notify_on_object (pDialog, NOTIFICATION_RENDER, pDialog, pCairoContext);
 		
 		if (pDialog->fAppearanceCounter < 1.)
 		{
@@ -154,32 +157,22 @@ static gboolean on_expose_dialog (GtkWidget *pWidget,
 		}
 		
 		cairo_destroy (pCairoContext);
-	}
+	//}
 	return FALSE;
 }
 
 static void _cairo_dock_set_dialog_input_shape (CairoDialog *pDialog)
 {
-	pDialog->pShapeBitmap = (GdkBitmap*) gdk_pixmap_new (NULL,
-		pDialog->container.iWidth,
-		pDialog->container.iHeight,
-		1);
-	cairo_t *pCairoContext = gdk_cairo_create (pDialog->pShapeBitmap);
-	cairo_set_source_rgba (pCairoContext, 0.0f, 0.0f, 0.0f, 0.0f);
-	cairo_set_operator (pCairoContext, CAIRO_OPERATOR_SOURCE);
-	cairo_paint (pCairoContext);
-	cairo_set_source_rgba (pCairoContext, 1.0f, 1.0f, 1.0f, 1.0f);
-	cairo_rectangle (pCairoContext,
+	if (pDialog->pShapeBitmap != NULL)
+		gldi_shape_destroy (pDialog->pShapeBitmap);
+	
+	pDialog->pShapeBitmap = gldi_container_create_input_shape (CAIRO_CONTAINER (pDialog),
 		0,
 		0,
 		1,
-		1);
-	cairo_fill (pCairoContext);  // workaround sur un bug de X...
-	cairo_destroy (pCairoContext);
-	gtk_widget_input_shape_combine_mask (pDialog->container.pWidget,
-		pDialog->pShapeBitmap,
-		0,
-		0);
+		1);  // workaround a bug in X with fully transparent window => let 1 pixel ON.
+
+	gldi_container_set_input_shape (CAIRO_CONTAINER (pDialog), pDialog->pShapeBitmap);
 }
 
 static gboolean on_configure_dialog (GtkWidget* pWidget,
@@ -205,7 +198,11 @@ static gboolean on_configure_dialog (GtkWidget* pWidget,
 	{
 		int w = pDialog->iInteractiveWidth, h = pDialog->iInteractiveHeight;
 		GtkRequisition requisition;
+		#if (GTK_MAJOR_VERSION < 3)
 		gtk_widget_size_request (pDialog->pInteractiveWidget, &requisition);
+		#else
+		gtk_widget_get_preferred_size (pDialog->pInteractiveWidget, &requisition, NULL);
+		#endif
 		pDialog->iInteractiveWidth = requisition.width;
 		pDialog->iInteractiveHeight = requisition.height;
 		//g_print ("  pInteractiveWidget : %dx%d\n", pDialog->iInteractiveWidth, pDialog->iInteractiveHeight);
@@ -276,13 +273,13 @@ static gboolean on_unmap_dialog (GtkWidget* pWidget,
 
 static GtkWidget *_cairo_dock_add_dialog_internal_box (CairoDialog *pDialog, int iWidth, int iHeight, gboolean bCanResize)
 {
-	GtkWidget *pBox = gtk_hbox_new (0, FALSE);
+	GtkWidget *pBox = _gtk_hbox_new (0);
 	if (iWidth != 0 && iHeight != 0)
-		gtk_widget_set (pBox, "height-request", iHeight, "width-request", iWidth, NULL);
+		g_object_set (pBox, "height-request", iHeight, "width-request", iWidth, NULL);
 	else if (iWidth != 0)
-			gtk_widget_set (pBox, "width-request", iWidth, NULL);
+			g_object_set (pBox, "width-request", iWidth, NULL);
 	else if (iHeight != 0)
-			gtk_widget_set (pBox, "height-request", iHeight, NULL);
+			g_object_set (pBox, "height-request", iHeight, NULL);
 	gtk_box_pack_start (GTK_BOX (pDialog->pWidgetLayout),
 		pBox,
 		bCanResize,
@@ -291,6 +288,49 @@ static GtkWidget *_cairo_dock_add_dialog_internal_box (CairoDialog *pDialog, int
 	return pBox;
 }
 
+static gboolean _cairo_dialog_animation_loop (CairoContainer *pContainer)
+{
+	CairoDialog *pDialog = CAIRO_DIALOG (pContainer);
+	gboolean bContinue = FALSE;
+	gboolean bUpdateSlowAnimation = FALSE;
+	pContainer->iAnimationStep ++;
+	if (pContainer->iAnimationStep * pContainer->iAnimationDeltaT >= CAIRO_DOCK_MIN_SLOW_DELTA_T)
+	{
+		bUpdateSlowAnimation = TRUE;
+		pContainer->iAnimationStep = 0;
+		pContainer->bKeepSlowAnimation = FALSE;
+	}
+	
+	if (pDialog->fAppearanceCounter < 1)
+	{
+		pDialog->fAppearanceCounter += .08;
+		if (pDialog->fAppearanceCounter > .99)
+		{
+			pDialog->fAppearanceCounter = 1.;
+		}
+		else
+		{
+			bContinue = TRUE;
+		}
+	}
+	
+	if (bUpdateSlowAnimation)
+	{
+		cairo_dock_notify_on_object (pDialog, NOTIFICATION_UPDATE_SLOW, pDialog, &pContainer->bKeepSlowAnimation);
+	}
+	
+	cairo_dock_notify_on_object (pDialog, NOTIFICATION_UPDATE, pDialog, &bContinue);
+	
+	cairo_dock_redraw_container (CAIRO_CONTAINER (pDialog));
+	if (! bContinue && ! pContainer->bKeepSlowAnimation)
+	{
+		pContainer->iSidGLAnimation = 0;
+		return FALSE;
+	}
+	else
+		return TRUE;
+}
+
 static CairoDialog *_cairo_dock_create_empty_dialog (gboolean bInteractive)
 {
 	//\________________ On cree un dialogue qu'on insere immediatement dans la liste.
@@ -302,7 +342,10 @@ static CairoDialog *_cairo_dock_create_empty_dialog (gboolean bInteractive)
 	//\________________ On construit la fenetre du dialogue.
 	//GtkWidget* pWindow = gtk_window_new (bInteractiveWindow ? GTK_WINDOW_TOPLEVEL : GTK_WINDOW_POPUP);  // les popups ne prennent pas le focus. En fait, ils ne sont meme pas controles par le WM.
 	GtkWidget* pWindow = cairo_dock_init_container_no_opengl (CAIRO_CONTAINER (pDialog));
-	cairo_dock_install_notifications_on_object (pDialog, NB_NOTIFICATIONS_DIALOG);
+	///cairo_dock_install_notifications_on_object (pDialog, NB_NOTIFICATIONS_DIALOG);
+	gldi_object_set_manager (GLDI_OBJECT (pDialog), GLDI_MANAGER (&myDialogsMgr));
+	
+	pDialog->container.iface.animation_loop = _cairo_dialog_animation_loop;
 	
 	gtk_window_set_title (GTK_WINDOW (pWindow), "cairo-dock-dialog");
 	if (! bInteractive)
@@ -321,14 +364,10 @@ static cairo_surface_t *_cairo_dock_create_dialog_text_surface (const gchar *cTe
 	if (cText == NULL)
 		return NULL;
 	
-	return cairo_dock_create_surface_from_text_full (cText,
+	return cairo_dock_create_surface_from_text (cText,
 		(pTextDescription ? pTextDescription : &myDialogsParam.dialogTextDescription),
-		1.,
-		0,
 		iTextWidth,
-		iTextHeight,
-		NULL,
-		NULL);
+		iTextHeight);
 }
 
 static cairo_surface_t *_cairo_dock_create_dialog_icon_surface (const gchar *cImageFilePath, int iNbFrames, Icon *pIcon, CairoContainer *pContainer, int iDesiredSize, int *iIconSize)
@@ -405,7 +444,7 @@ CairoDialog *cairo_dock_new_dialog (CairoDialogAttribute *pAttribute, Icon *pIco
 	if (pAttribute->bForceAbove)
 	{
 		gtk_window_set_keep_above (GTK_WINDOW (pDialog->container.pWidget), TRUE);
-		Window Xid = GDK_WINDOW_XID (pDialog->container.pWidget->window);
+		Window Xid = gldi_container_get_Xid (CAIRO_CONTAINER (pDialog));
 		cairo_dock_set_xwindow_type_hint (Xid, "_NET_WM_WINDOW_TYPE_DOCK");  // pour passer devant les fenetres plein ecran; depend du WM.
 	}
 	
@@ -441,7 +480,11 @@ CairoDialog *cairo_dock_new_dialog (CairoDialogAttribute *pAttribute, Icon *pIco
 		pDialog->pInteractiveWidget = pAttribute->pInteractiveWidget;
 		
 		GtkRequisition requisition;
+		#if (GTK_MAJOR_VERSION < 3)
 		gtk_widget_size_request (pAttribute->pInteractiveWidget, &requisition);
+		#else
+		gtk_widget_get_preferred_size (pAttribute->pInteractiveWidget, &requisition, NULL);
+		#endif
 		pDialog->iInteractiveWidth = requisition.width;
 		pDialog->iInteractiveHeight = requisition.height;
 	}
@@ -473,7 +516,8 @@ CairoDialog *cairo_dock_new_dialog (CairoDialogAttribute *pAttribute, Icon *pIco
 			{
 				gchar *cButtonPath;
 				if (*cButtonImage != '/')
-					cButtonPath = cairo_dock_search_icon_s_path (cButtonImage);
+					cButtonPath = cairo_dock_search_icon_s_path (cButtonImage,
+						MAX (myDialogsParam.iDialogButtonWidth, myDialogsParam.iDialogButtonHeight));
 				else
 					cButtonPath = (gchar*)cButtonImage;
 				pDialog->pButtons[i].pSurface = cairo_dock_create_surface_from_image_simple (cButtonPath,
@@ -515,25 +559,25 @@ CairoDialog *cairo_dock_new_dialog (CairoDialogAttribute *pAttribute, Icon *pIco
 	cairo_dock_set_dialog_orientation (pDialog, pContainer);  // renseigne aussi bDirectionUp, bIsHorizontal, et iHeight.
 	
 	//\________________ On reserve l'espace pour les decorations.
-	GtkWidget *pMainHBox = gtk_hbox_new (0, FALSE);
+	GtkWidget *pMainHBox = _gtk_hbox_new (0);
 	gtk_container_add (GTK_CONTAINER (pDialog->container.pWidget), pMainHBox);
-	pDialog->pLeftPaddingBox = gtk_vbox_new (0, FALSE);
-	gtk_widget_set (pDialog->pLeftPaddingBox, "width-request", pDialog->iLeftMargin, NULL);
+	pDialog->pLeftPaddingBox = _gtk_vbox_new (0);
+	g_object_set (pDialog->pLeftPaddingBox, "width-request", pDialog->iLeftMargin, NULL);
 	gtk_box_pack_start (GTK_BOX (pMainHBox),
 		pDialog->pLeftPaddingBox,
 		FALSE,
 		FALSE,
 		0);
-	
-	pDialog->pWidgetLayout = gtk_vbox_new (0, FALSE);
+
+	pDialog->pWidgetLayout = _gtk_vbox_new (0);
 	gtk_box_pack_start (GTK_BOX (pMainHBox),
 		pDialog->pWidgetLayout,
 		FALSE,
 		FALSE,
 		0);
-	
-	pDialog->pRightPaddingBox = gtk_vbox_new (0, FALSE);
-	gtk_widget_set (pDialog->pRightPaddingBox, "width-request", pDialog->iRightMargin, NULL);
+
+	pDialog->pRightPaddingBox = _gtk_vbox_new (0);
+	g_object_set (pDialog->pRightPaddingBox, "width-request", pDialog->iRightMargin, NULL);
 	gtk_box_pack_start (GTK_BOX (pMainHBox),
 		pDialog->pRightPaddingBox,
 		FALSE,
@@ -551,13 +595,11 @@ CairoDialog *cairo_dock_new_dialog (CairoDialogAttribute *pAttribute, Icon *pIco
 	}
 	if (pDialog->pInteractiveWidget != NULL)
 	{
-		cd_debug (" ref = %d", pAttribute->pInteractiveWidget->object.parent_instance.ref_count);
 		gtk_box_pack_start (GTK_BOX (pDialog->pWidgetLayout),
 			pDialog->pInteractiveWidget,
 			FALSE,
 			FALSE,
 			0);
-		cd_debug (" pack -> ref = %d", pAttribute->pInteractiveWidget->object.parent_instance.ref_count);
 		cd_debug ("grab focus");
 		gtk_window_present (GTK_WINDOW (pDialog->container.pWidget));
 		gtk_widget_grab_focus (pDialog->pInteractiveWidget);
@@ -579,7 +621,11 @@ CairoDialog *cairo_dock_new_dialog (CairoDialogAttribute *pAttribute, Icon *pIco
 	}
 	//\________________ On connecte les signaux utiles.
 	g_signal_connect (G_OBJECT (pDialog->container.pWidget),
+		#if (GTK_MAJOR_VERSION < 3)
 		"expose-event",
+		#else
+		"draw",
+		#endif
 		G_CALLBACK (on_expose_dialog),
 		pDialog);
 	g_signal_connect (G_OBJECT (pDialog->container.pWidget),
@@ -653,7 +699,7 @@ void cairo_dock_free_dialog (CairoDialog *pDialog)
 		g_timer_destroy (pDialog->pUnmapTimer);
 	
 	if (pDialog->pShapeBitmap != NULL)
-		g_object_unref ((gpointer) pDialog->pShapeBitmap);
+		gldi_shape_destroy (pDialog->pShapeBitmap);
 	
 	if (pDialog->pUserData != NULL && pDialog->pFreeUserDataFunc != NULL)
 		pDialog->pFreeUserDataFunc (pDialog->pUserData);
@@ -669,7 +715,7 @@ static void _cairo_dock_dialog_calculate_aimed_point (Icon *pIcon, CairoContaine
 	if (CAIRO_DOCK_IS_DOCK (pContainer))
 	{
 		CairoDock *pDock = CAIRO_DOCK (pContainer);
-		if (pDock->iRefCount > 0 && ! GTK_WIDGET_VISIBLE (pContainer->pWidget))  // sous-dock invisible.
+		if (pDock->iRefCount > 0 && ! gldi_container_is_visible (pContainer))  // sous-dock invisible.
 		{
 			CairoDock *pParentDock = NULL;
 			Icon *pPointingIcon = cairo_dock_search_icon_pointing_on_dock (pDock, &pParentDock);
@@ -678,6 +724,12 @@ static void _cairo_dock_dialog_calculate_aimed_point (Icon *pIcon, CairoContaine
 		else  // dock principal ou sous-dock visible.
 		{
 			*bIsHorizontal = (pContainer->bIsHorizontal == CAIRO_DOCK_HORIZONTAL);
+			if (! *bIsHorizontal)
+			{
+				int *tmp = iX;
+				iX = iY;
+				iY = tmp;
+			}
 			int dy;
 			if (pDock->iInputState == CAIRO_DOCK_INPUT_ACTIVE)
 				dy = pContainer->iHeight - pDock->iActiveHeight;
@@ -744,6 +796,7 @@ void cairo_dock_set_dialog_orientation (CairoDialog *pDialog, CairoContainer *pC
 	if (pContainer != NULL/* && pDialog->pIcon != NULL*/)
 	{
 		_cairo_dock_dialog_calculate_aimed_point (pDialog->pIcon, pContainer, &pDialog->iAimedX, &pDialog->iAimedY, &pDialog->bRight, &pDialog->bTopBottomDialog, &pDialog->container.bDirectionUp, pDialog->fAlign);
+		//g_print ("%s (%d,%d %d %d %d)\n", __func__, pDialog->iAimedX, pDialog->iAimedY, pDialog->bRight, pDialog->bTopBottomDialog, pDialog->container.bDirectionUp);
 	}
 	else
 	{
@@ -758,10 +811,8 @@ GtkWidget *cairo_dock_steal_widget_from_its_container (GtkWidget *pWidget)
 	GtkWidget *pContainer = gtk_widget_get_parent (pWidget);
 	if (pContainer != NULL)
 	{
-		cd_debug (" ref : %d", pWidget->object.parent_instance.ref_count);
 		g_object_ref (G_OBJECT (pWidget));
 		gtk_container_remove (GTK_CONTAINER (pContainer), pWidget);
-		cd_debug (" -> %d", pWidget->object.parent_instance.ref_count);
 		
 		// if we were monitoring the click events on the widget, stop it.
 		g_signal_handlers_disconnect_matched (pWidget,
@@ -791,20 +842,38 @@ GtkWidget *cairo_dock_steal_interactive_widget_from_dialog (CairoDialog *pDialog
 
 void cairo_dock_set_dialog_widget_text_color (GtkWidget *pWidget)
 {
+	#if (GTK_MAJOR_VERSION < 3)
 	static GdkColor color;
 	color.red = myDialogsParam.dialogTextDescription.fColorStart[0] * 65535;
 	color.green = myDialogsParam.dialogTextDescription.fColorStart[1] * 65535;
 	color.blue = myDialogsParam.dialogTextDescription.fColorStart[2] * 65535;
 	gtk_widget_modify_fg (pWidget, GTK_STATE_NORMAL, &color);
+	#else
+	static GdkRGBA color;
+	color.red = myDialogsParam.dialogTextDescription.fColorStart[0];
+	color.green = myDialogsParam.dialogTextDescription.fColorStart[1];
+	color.blue = myDialogsParam.dialogTextDescription.fColorStart[2];
+	color.alpha = myDialogsParam.dialogTextDescription.fColorStart[3];
+	gtk_widget_override_color (pWidget, GTK_STATE_NORMAL, &color);
+	#endif
 }
 
 void cairo_dock_set_dialog_widget_bg_color (GtkWidget *pWidget)
 {
+	#if (GTK_MAJOR_VERSION < 3)
 	static GdkColor color;
 	color.red = myDialogsParam.fDialogColor[0] * 65535;
 	color.green = myDialogsParam.fDialogColor[1] * 65535;
 	color.blue = myDialogsParam.fDialogColor[2] * 65535;
 	gtk_widget_modify_bg (pWidget, GTK_STATE_NORMAL, &color);
+	#else
+	static GdkRGBA color;
+	color.red = myDialogsParam.fDialogColor[0];
+	color.green = myDialogsParam.fDialogColor[1];
+	color.blue = myDialogsParam.fDialogColor[2];
+	color.alpha = myDialogsParam.fDialogColor[3];
+	gtk_widget_override_color (pWidget, GTK_STATE_NORMAL, &color);
+	#endif
 }
 
 void cairo_dock_set_new_dialog_text_surface (CairoDialog *pDialog, cairo_surface_t *pNewTextSurface, int iNewTextWidth, int iNewTextHeight)
@@ -824,7 +893,7 @@ void cairo_dock_set_new_dialog_text_surface (CairoDialog *pDialog, cairo_surface
 
 	if (pDialog->iMessageWidth != iPrevMessageWidth || pDialog->iMessageHeight != iPrevMessageHeight)
 	{
-		gtk_widget_set (pDialog->pMessageWidget, "width-request", pDialog->iMessageWidth, "height-request", pDialog->iMessageHeight, NULL);  // inutile de replacer le dialogue puisque sa gravite fera le boulot.
+		g_object_set (pDialog->pMessageWidget, "width-request", pDialog->iMessageWidth, "height-request", pDialog->iMessageHeight, NULL);  // inutile de replacer le dialogue puisque sa gravite fera le boulot.
 		
 		gtk_widget_queue_draw (pDialog->container.pWidget);
 		
@@ -861,7 +930,7 @@ void cairo_dock_set_new_dialog_icon_surface (CairoDialog *pDialog, cairo_surface
 
 	if (pDialog->iMessageWidth != iPrevMessageWidth || pDialog->iMessageHeight != iPrevMessageHeight)
 	{
-		gtk_widget_set (pDialog->pMessageWidget, "width-request", pDialog->iMessageWidth, "height-request", pDialog->iMessageHeight, NULL);  // inutile de replacer le dialogue puisque sa gravite fera le boulot.
+		g_object_set (pDialog->pMessageWidget, "width-request", pDialog->iMessageWidth, "height-request", pDialog->iMessageHeight, NULL);  // inutile de replacer le dialogue puisque sa gravite fera le boulot.
 		
 		gtk_widget_queue_draw (pDialog->container.pWidget);
 	}
