@@ -22,11 +22,9 @@
 #include <math.h>
 #include <gdk/gdkkeysyms.h>
 
-#include <gtk/gtkgl.h>
 #include <GL/gl.h>
 #include <GL/glu.h>
 #include <GL/glx.h>
-#include <gdk/x11/gdkglx.h>
 
 #include "gldi-config.h"
 #include "cairo-dock-icon-factory.h"
@@ -157,8 +155,17 @@ static gboolean on_leave_dialog (GtkWidget* pWidget,
 {
 	Icon *pIcon = pDialog->pIcon;
 	int iMouseX, iMouseY;
-	gdk_window_get_pointer (pDialog->container.pWidget->window, &iMouseX, &iMouseY, NULL);
-	
+	if (pEvent != NULL)
+	{
+		iMouseX = pEvent->x_root;
+		iMouseY = pEvent->y_root;
+	}
+	else
+	{
+		gldi_container_update_mouse_position (CAIRO_CONTAINER (pDialog));
+		iMouseX = pDialog->container.iMouseX;
+		iMouseY = pDialog->container.iMouseY;
+	}
 	if (iMouseX > 0 && iMouseX < pDialog->container.iWidth && iMouseY > 0 && iMouseY < pDialog->container.iHeight && pDialog->pInteractiveWidget != NULL)  // en fait on est dedans (peut arriver si le dialogue a un widget a l'interieur).
 	{
 		if (pIcon != NULL)
@@ -170,7 +177,7 @@ static gboolean on_leave_dialog (GtkWidget* pWidget,
 				return FALSE;
 			}
 			//else
-			//	g_print ("leave dialog\n");
+			//	//g_print ("leave dialog\n");
 		}
 	}
 	
@@ -263,6 +270,8 @@ static gboolean on_button_press_dialog (GtkWidget* pWidget,
 	return FALSE;
 }
 
+#define KEY(x) GDK_KEY_##x
+
 static gboolean on_key_press_dialog (GtkWidget *pWidget,
 	GdkEventKey *pKey,
 	CairoDialog *pDialog)
@@ -273,11 +282,12 @@ static gboolean on_key_press_dialog (GtkWidget *pWidget,
 	{
 		switch (pKey->keyval)
 		{
-			case GDK_Return :
+			case GLDI_KEY(Return) :
+			case GLDI_KEY(KP_Enter) :
 				pDialog->action_on_answer (-1, pDialog->pInteractiveWidget, pDialog->pUserData, pDialog);
 				cairo_dock_dialog_unreference (pDialog);
 			break ;
-			case GDK_Escape :
+			case GLDI_KEY(Escape) :
 				pDialog->action_on_answer (-2, pDialog->pInteractiveWidget, pDialog->pUserData, pDialog);
 				cairo_dock_dialog_unreference (pDialog);
 			break ;
@@ -598,7 +608,7 @@ CairoDialog *cairo_dock_build_dialog (CairoDialogAttribute *pAttribute, Icon *pI
 	CairoDialog *pDialog = cairo_dock_new_dialog (pAttribute, pIcon, pContainer);
 	if (pIcon && pIcon->pSubDock)  // un sous-dock par-dessus le dialogue est tres genant.
 		cairo_dock_emit_leave_signal (CAIRO_CONTAINER (pIcon->pSubDock));
-	if (pIcon && CAIRO_DOCK_IS_DOCK (pContainer) && CAIRO_DOCK (pContainer)->fMagnitudeMax == 0)  // view without zoom, the dialog is stuck to the icon, and therefore is under the label, so hide this one.
+	if (pIcon && CAIRO_DOCK_IS_DOCK (pContainer) && cairo_dock_get_icon_max_scale (pIcon) < 1.01)  // view without zoom, the dialog is stuck to the icon, and therefore is under the label, so hide this one.
 	{
 		if (pIcon->iHideLabel == 0 && pContainer)
 			gtk_widget_queue_draw (pContainer->pWidget);
@@ -668,7 +678,7 @@ static void _cairo_dock_dialog_find_optimal_placement (CairoDialog *pDialog)
 			continue ;
 		if (pDialogOnOurWay != pDialog)  // check if this dialog can overlap us.
 		{
-			if (pDialogOnOurWay->container.pWidget && GTK_WIDGET_VISIBLE (pDialogOnOurWay->container.pWidget) && pDialogOnOurWay->pIcon != NULL)
+			if (pDialogOnOurWay->container.pWidget && gldi_container_is_visible (CAIRO_CONTAINER (pDialogOnOurWay)) && pDialogOnOurWay->pIcon != NULL)
 			{
 				iTopY = pDialogOnOurWay->container.iWindowPositionY;
 				iBottomY = pDialogOnOurWay->container.iWindowPositionY + pDialogOnOurWay->container.iHeight;
@@ -718,7 +728,6 @@ static void cairo_dock_place_dialog (CairoDialog *pDialog, CairoContainer *pCont
 	if (pDialog->container.bInside && ! (pDialog->pInteractiveWidget || pDialog->action_on_answer))  // in the case of a modal dialog, the dialog takes the dock's events, including the "enter-event" one. So we are inside the dialog as soon as we enter the dock, and consequently, the dialog is not replaced when the dock unhides itself.
 		return;
 	
-	int w, h;
 	GdkGravity iGravity;
 	if (pContainer != NULL/* && pDialog->pIcon != NULL*/)
 	{
@@ -731,9 +740,9 @@ static void cairo_dock_place_dialog (CairoDialog *pDialog, CairoContainer *pCont
 		}
 		else  // dialogue lie a un dock vertical, on ne cherche pas a optimiser le placement.
 		{
-			int tmp = pDialog->iAimedX;
+			/**int tmp = pDialog->iAimedX;
 			pDialog->iAimedX = pDialog->iAimedY;
-			pDialog->iAimedY = tmp;
+			pDialog->iAimedY = tmp;*/
 			
 			pDialog->iComputedPositionX = (pDialog->bRight ? MAX (0, pDialog->iAimedX - pDialog->container.iWidth) : pDialog->iAimedX);
 			pDialog->iComputedPositionY = (pDialog->container.bDirectionUp ? MAX (0, pDialog->iAimedY - pDialog->iComputedHeight) : pDialog->iAimedY + pDialog->iMinBottomGap);  // on place la bulle (et non pas la fenetre) sans faire d'optimisation.
@@ -787,7 +796,7 @@ void cairo_dock_refresh_all_dialogs (gboolean bReplace)
 		pDialog = ic->data;
 		
 		pIcon = pDialog->pIcon;
-		if (pIcon != NULL && GTK_WIDGET_VISIBLE (pDialog->container.pWidget))  // on ne replace pas les dialogues en cours de destruction ou caches.
+		if (pIcon != NULL && gldi_container_is_visible (CAIRO_CONTAINER (pDialog)))  // on ne replace pas les dialogues en cours de destruction ou caches.
 		{
 			pContainer = cairo_dock_search_container_from_icon (pIcon);
 			//if (CAIRO_DOCK_IS_DOCK (pContainer))
@@ -900,18 +909,22 @@ static inline GtkWidget *_cairo_dock_make_entry_for_dialog (const gchar *cTextFo
 {
 	GtkWidget *pWidget = gtk_entry_new ();
 	gtk_entry_set_has_frame (GTK_ENTRY (pWidget), FALSE);
-	gtk_widget_set (pWidget, "width-request", CAIRO_DIALOG_MIN_ENTRY_WIDTH, NULL);
+	g_object_set (pWidget, "width-request", CAIRO_DIALOG_MIN_ENTRY_WIDTH, NULL);
 	if (cTextForEntry != NULL)
 		gtk_entry_set_text (GTK_ENTRY (pWidget), cTextForEntry);
 	return pWidget;
 }
 static inline GtkWidget *_cairo_dock_make_hscale_for_dialog (double fValueForHScale, double fMaxValueForHScale)
 {
+	#if (GTK_MAJOR_VERSION < 3)
 	GtkWidget *pWidget = gtk_hscale_new_with_range (0, fMaxValueForHScale, fMaxValueForHScale / 100.);
+	#else
+	GtkWidget *pWidget = gtk_scale_new_with_range (GTK_ORIENTATION_HORIZONTAL, 0, fMaxValueForHScale, fMaxValueForHScale / 100.);
+	#endif
 	gtk_scale_set_digits (GTK_SCALE (pWidget), 2);
 	gtk_range_set_value (GTK_RANGE (pWidget), fValueForHScale);
 
-	gtk_widget_set (pWidget, "width-request", CAIRO_DIALOG_MIN_SCALE_WIDTH, NULL);
+	g_object_set (pWidget, "width-request", CAIRO_DIALOG_MIN_SCALE_WIDTH, NULL);
 	cairo_dock_set_dialog_widget_text_color (pWidget);
 	return pWidget;
 }
@@ -1083,10 +1096,6 @@ Icon *cairo_dock_get_dialogless_icon_full (CairoDock *pDock)
 	if (pIcon != NULL && ! cairo_dock_icon_has_dialog (pIcon) && pIcon->cParentDockName != NULL && ! cairo_dock_icon_is_being_removed (pIcon))
 		return pIcon;
 	
-	pIcon = cairo_dock_get_first_icon_of_group (pDock->icons, CAIRO_DOCK_SEPARATOR23);
-	if (pIcon != NULL && ! cairo_dock_icon_has_dialog (pIcon) && pIcon->cParentDockName != NULL && ! cairo_dock_icon_is_being_removed (pIcon))
-		return pIcon;
-	
 	pIcon = cairo_dock_get_first_icon_of_true_type (pDock->icons, CAIRO_DOCK_ICON_TYPE_SEPARATOR);
 	if (pIcon != NULL && ! cairo_dock_icon_has_dialog (pIcon) && pIcon->cParentDockName != NULL && ! cairo_dock_icon_is_being_removed (pIcon))
 		return pIcon;
@@ -1124,7 +1133,7 @@ int cairo_dock_ask_general_question_and_wait (const gchar *cQuestion)
 void cairo_dock_hide_dialog (CairoDialog *pDialog)
 {
 	cd_debug ("%s ()", __func__);
-	if (GTK_WIDGET_VISIBLE (pDialog->container.pWidget))
+	if (gldi_container_is_visible (CAIRO_CONTAINER (pDialog)))
 	{
 		pDialog->bAllowMinimize = TRUE;
 		gtk_widget_hide (pDialog->container.pWidget);
@@ -1153,7 +1162,7 @@ void cairo_dock_hide_dialog (CairoDialog *pDialog)
 void cairo_dock_unhide_dialog (CairoDialog *pDialog)
 {
 	cd_debug ("%s ()", __func__);
-	if (! GTK_WIDGET_VISIBLE (pDialog->container.pWidget))
+	if (! gldi_container_is_visible (CAIRO_CONTAINER (pDialog)))
 	{
 		if (pDialog->pInteractiveWidget != NULL)
 			gtk_widget_grab_focus (pDialog->pInteractiveWidget);
@@ -1163,7 +1172,7 @@ void cairo_dock_unhide_dialog (CairoDialog *pDialog)
 			CairoContainer *pContainer = cairo_dock_search_container_from_icon (pIcon);
 			cairo_dock_place_dialog (pDialog, pContainer);
 			
-			if (CAIRO_DOCK_IS_DOCK (pContainer) && CAIRO_DOCK (pContainer)->fMagnitudeMax == 0)
+			if (CAIRO_DOCK_IS_DOCK (pContainer) && cairo_dock_get_icon_max_scale (pIcon) < 1.01)  // same remark
 			{
 				if (pIcon->iHideLabel == 0 && pContainer)
 					gtk_widget_queue_draw (pContainer->pWidget);
@@ -1177,7 +1186,7 @@ void cairo_dock_unhide_dialog (CairoDialog *pDialog)
 
 void cairo_dock_toggle_dialog_visibility (CairoDialog *pDialog)
 {
-	if (GTK_WIDGET_VISIBLE (pDialog->container.pWidget))
+	if (gldi_container_is_visible (CAIRO_CONTAINER (pDialog)))
 		cairo_dock_hide_dialog (pDialog);
 	else
 		cairo_dock_unhide_dialog (pDialog);
@@ -1300,7 +1309,7 @@ static void unload (void)
 static void init (void)
 {
 	cairo_dock_register_notification_on_object (&myDialogsMgr,
-		NOTIFICATION_RENDER_DIALOG,
+		NOTIFICATION_RENDER,
 		(CairoDockNotificationFunc) _cairo_dock_render_dialog_notification,
 		CAIRO_DOCK_RUN_AFTER, NULL);
 }
@@ -1330,6 +1339,7 @@ void gldi_register_dialogs_manager (void)
 	myDialogsMgr.mgr.pData = (GldiManagerDataPtr)NULL;
 	// signals
 	cairo_dock_install_notifications_on_object (&myDialogsMgr, NB_NOTIFICATIONS_DIALOG);
+	gldi_object_set_manager (GLDI_OBJECT (&myDialogsMgr), GLDI_MANAGER (&myContainersMgr));
 	// register
 	gldi_register_manager (GLDI_MANAGER(&myDialogsMgr));
 }
