@@ -27,14 +27,6 @@
 #include <gdk/gdkx.h>
 
 #include <cairo.h>
-#include <pango/pango.h>
-#include <librsvg/rsvg.h>
-#include <librsvg/rsvg-cairo.h>
-
-#ifdef HAVE_GLITZ
-#include <glitz-glx.h>
-#include <cairo-glitz.h>
-#endif
 
 #include "gldi-config.h"
 #include "cairo-dock-applications-manager.h"
@@ -59,7 +51,6 @@
 #include "cairo-dock-draw.h"
 #include "cairo-dock-animations.h"
 #include "cairo-dock-container.h"
-#include "cairo-dock-emblem.h"
 #include "cairo-dock-keybinder.h"
 #include "cairo-dock-gui-manager.h"
 #include "cairo-dock-indicator-manager.h"  // myIndicatorsParam.bUseClassIndic
@@ -87,35 +78,25 @@ static guint s_iSidPollScreenEdge = 0;
 static int s_iNbPolls = 0;
 static gboolean s_bQuickHide = FALSE;
 static gboolean s_bKeepAbove = FALSE;
+static CairoKeyBinding *s_pPopupBinding = NULL;  // option 'pop up on shortkey'
+
+#define MOUSE_POLLING_DT 150  // mouse polling delay in ms
 
 static gboolean cairo_dock_read_root_dock_config (const gchar *cDockName, CairoDock *pDock);
 
+typedef struct {
+	gboolean bUpToDate;
+	gint x;
+	gint y;
+	gboolean bNoMove;
+	gdouble dx;
+	gdouble dy;
+} CDMousePolling;
+
   /////////////
  // MANAGER //
 /////////////
 
-void cairo_dock_init_dock_manager (void)
-{
-	cd_message ("");
-	if (s_hDocksTable == NULL)
-	{
-		s_hDocksTable = g_hash_table_new_full (g_str_hash,
-			g_str_equal,
-			g_free,
-			NULL);  // donc on peut utiliser g_hash_table_remove plutot que g_hash_table_steal.
-		
-		cairo_dock_register_notification_on_object (&myDocksMgr,
-			NOTIFICATION_RENDER_DOCK,
-			(CairoDockNotificationFunc) cairo_dock_render_dock_notification,
-			CAIRO_DOCK_RUN_FIRST, NULL);
-		cairo_dock_register_notification_on_object (&myDocksMgr,
-			NOTIFICATION_LEAVE_DOCK,
-			(CairoDockNotificationFunc) cairo_dock_on_leave_dock_notification,
-			CAIRO_DOCK_RUN_FIRST, NULL);
-	}
-	memset (&g_pVisibleZoneBuffer, 0, sizeof (CairoDockImageBuffer));
-}
-
 
 void cairo_dock_force_docks_above (void)
 {
@@ -144,25 +125,11 @@ void cairo_dock_reset_docks_table (void)
 }
 
 
-CairoDock *cairo_dock_create_dock (const gchar *cDockName, const gchar *cRendererName)
+static inline CairoDock *_create_dock (const gchar *cDockName)
 {
-	cd_message ("%s (%s)", __func__, cDockName);
-	g_return_val_if_fail (cDockName != NULL, NULL);
-	
-	//\__________________ On verifie qu'il n'existe pas deja.
-	CairoDock *pExistingDock = g_hash_table_lookup (s_hDocksTable, cDockName);
-	if (pExistingDock != NULL)
-	{
-		return pExistingDock;
-	}
-	
 	//\__________________ On cree un nouveau dock.
-	CairoDock *pDock = cairo_dock_new_dock (cRendererName);
-	
-	if (s_bKeepAbove)
-		gtk_window_set_keep_above (GTK_WINDOW (pDock->container.pWidget), s_bKeepAbove);
-	if (myContainersParam.bUseFakeTransparency)
-		gtk_window_set_keep_below (GTK_WINDOW (pDock->container.pWidget), TRUE);
+	CairoDock *pDock = cairo_dock_new_dock ();
+	pDock->iIconSize = myIconsParam.iIconWidth;  // by default
 	
 	//\__________________ On l'enregistre.
 	if (g_hash_table_size (s_hDocksTable) == 0)  // c'est le 1er.
@@ -170,60 +137,116 @@ CairoDock *cairo_dock_create_dock (const gchar *cDockName, const gchar *cRendere
 		pDock->bIsMainDock = TRUE;
 		g_pMainDock = pDock;
 		pDock->bGlobalBg = TRUE;
+		pDock->bGlobalIconSize = TRUE;
 	}
 	g_hash_table_insert (s_hDocksTable, g_strdup (cDockName), pDock);
-	s_pRootDockList = g_list_prepend (s_pRootDockList, pDock);
 	
-	//\__________________ On le positionne si ce n'est pas le main dock.
-	if (! pDock->bIsMainDock)
-	{
-		if (cairo_dock_read_root_dock_config (cDockName, pDock))  // si pas de config (sub-dock ou config inexistante), se contente de lui mettre la meme position et renvoit FALSE.
-			cairo_dock_move_resize_dock (pDock);
-	}
+	if (s_bKeepAbove)
+		gtk_window_set_keep_above (GTK_WINDOW (pDock->container.pWidget), s_bKeepAbove);
+	if (myContainersParam.bUseFakeTransparency)
+		gtk_window_set_keep_below (GTK_WINDOW (pDock->container.pWidget), TRUE);
+	
 	return pDock;
 }
 
-void cairo_dock_reference_dock (CairoDock *pDock, CairoDock *pParentDock)
+CairoDock *cairo_dock_create_dock (const gchar *cDockName)
 {
-	pDock->iRefCount ++;
+	cd_message ("%s (%s)", __func__, cDockName);
+	g_return_val_if_fail (cDockName != NULL, NULL);
 	
-	if (pDock->iRefCount == 1)  // il devient un sous-dock.
+	//\__________________ if it already exists, return it.
+	CairoDock *pExistingDock = g_hash_table_lookup (s_hDocksTable, cDockName);
+	if (pExistingDock != NULL)
 	{
+		return pExistingDock;
+	}
+	
+	//\__________________ create a new dock.
+	CairoDock *pDock = _create_dock (cDockName);
+	
+	//\__________________ register it as a main dock
+	s_pRootDockList = g_list_prepend (s_pRootDockList, pDock);
+	
+	//\__________________ get its config params
+	if (! pDock->bIsMainDock)
+	{
+		if (cairo_dock_read_root_dock_config (cDockName, pDock))  // si pas de config (sub-dock ou config inexistante), se contente de lui mettre la meme position et renvoit FALSE.
+			//cairo_dock_move_resize_dock (pDock);  /// TODO: try without...
+			cd_debug ("NO MOVE");
+	}
+	
+	//\__________________ set a renderer (got from the conf, or the default one).
+	cairo_dock_set_default_renderer (pDock);
+	
+	return pDock;
+}
+
+CairoDock *cairo_dock_create_subdock (const gchar *cDockName, const gchar *cRendererName, CairoDock *pParentDock, GList *pIconList)
+{
+	//\__________________ create it if needed.
+	CairoDock *pSubDock = g_hash_table_lookup (s_hDocksTable, cDockName);
+	if (pSubDock == NULL)
+	{
+		pSubDock = _create_dock (cDockName);
+	}
+	
+	//\__________________ set the icons.
+	pSubDock->icons = pIconList;  // set icons now, before we set the ratio and the renderer. 
+	Icon *icon;
+	GList *ic;
+	for (ic = pIconList; ic != NULL; ic = ic->next)
+	{
+		icon = ic->data;
+		if (icon->cParentDockName == NULL)
+			icon->cParentDockName = g_strdup (cDockName);
+		cairo_dock_set_icon_container (icon, pSubDock);
+	}
+	cairo_dock_insert_automatic_separators_in_dock (pSubDock);  // it will update fFlatDockWidth, but we don't care as we'll recalculate it.
+	
+	//\__________________ make it a sub-dock.
+	if (pParentDock == NULL)
+		pParentDock = g_pMainDock;
+	cairo_dock_make_sub_dock (pSubDock, pParentDock, cRendererName);
+	
+	//\__________________ load the icons.
+	if (pIconList != NULL)
+	{
+		cairo_dock_reload_buffers_in_dock (pSubDock, FALSE, FALSE);  // idle reload; FALSE = don't compute their size, since it has been done in cairo_dock_make_sub_dock().
+	}
+	
+	cairo_dock_update_dock_size (pSubDock);
+	return pSubDock;
+}
+
+void cairo_dock_main_dock_to_sub_dock (CairoDock *pDock, CairoDock *pParentDock, const gchar *cRendererName)
+{
+	//g_print ("%s (%s)\n", __func__, cRendererName);
+	if (pDock->iRefCount == 0)  // il devient un sous-dock.
+	{
+		//\__________________ make it a sub-dock.
+		CairoDockPositionType iScreenBorder = ((! pDock->container.bIsHorizontal) << 1) | (! pDock->container.bDirectionUp);
+		
 		if (pParentDock == NULL)
 			pParentDock = g_pMainDock;
-		cairo_dock_make_sub_dock (pDock, pParentDock);
+		cairo_dock_make_sub_dock (pDock, pParentDock, cRendererName);
+		cairo_dock_update_dock_size (pDock);
 		
+		/**if (iScreenBorder != (((! pDock->container.bIsHorizontal) << 1) | (! pDock->container.bDirectionUp)))
+		{
+			cd_debug ("changement de position -> %d/%d", pDock->container.bIsHorizontal, pDock->container.bDirectionUp);
+			cairo_dock_reload_reflects_in_dock (pDock);
+		}*/
+		
+		//\__________________ remove from main-dock land
 		const gchar *cDockName = cairo_dock_search_dock_name (pDock);
-		cairo_dock_remove_root_dock_config (cDockName);
-		
-		cairo_dock_set_dock_visibility (pDock, CAIRO_DOCK_VISI_KEEP_ABOVE);  // si la visibilite n'avait pas ete mise (sub-dock), ne fera rien (vu que la visibilite par defaut est KEEP_ABOVE).
+		cairo_dock_remove_root_dock_config (cDockName);  // just in case.
 		
 		s_pRootDockList = g_list_remove (s_pRootDockList, pDock);
+		
+		cairo_dock_set_dock_visibility (pDock, CAIRO_DOCK_VISI_KEEP_ABOVE);  // si la visibilite n'avait pas ete mise (sub-dock), ne fera rien (vu que la visibilite par defaut est KEEP_ABOVE).
 	}
 }
 
-CairoDock *cairo_dock_create_subdock_from_scratch (GList *pIconList, gchar *cDockName, CairoDock *pParentDock)
-{
-	CairoDock *pSubDock = cairo_dock_create_dock (cDockName, NULL);
-	g_return_val_if_fail (pSubDock != NULL, NULL);
-	
-	cairo_dock_reference_dock (pSubDock, pParentDock);  // on le fait tout de suite pour avoir la bonne reference avant le 'load'.
-	
-	pSubDock->icons = pIconList;
-	if (pIconList != NULL)
-	{
-		Icon *icon;
-		GList *ic;
-		for (ic = pIconList; ic != NULL; ic = ic->next)
-		{
-			icon = ic->data;
-			if (icon->cParentDockName == NULL)
-				icon->cParentDockName = g_strdup (cDockName);
-		}
-		cairo_dock_load_buffers_in_one_dock (pSubDock);  // reload en idle.
-	}
-	return pSubDock;
-}
 
 void cairo_dock_destroy_dock (CairoDock *pDock, const gchar *cDockName)
 {
@@ -265,8 +288,7 @@ void cairo_dock_destroy_dock (CairoDock *pDock, const gchar *cDockName)
 		cairo_dock_stop_polling_screen_edge ();
 	}
 	
-	cairo_dock_free_dock (pDock);  // -> NOTIFICATION_STOP_DOCK
-	///cairo_dock_trigger_refresh_launcher_gui ();
+	cairo_dock_free_dock (pDock);  // -> NOTIFICATION_DESTROY
 }
 
 
@@ -322,7 +344,6 @@ gchar *cairo_dock_get_readable_name_for_fock (CairoDock *pDock)
 	{
 		int i = 0;
 		gpointer data[3] = {pDock, &i, NULL};
-		///cairo_dock_foreach_root_docks ((GFunc)_find_similar_root_dock, data);
 		GList *d = g_list_last (s_pRootDockList);
 		for (;d != NULL; d = d->prev)  // parse the list from the end to get the main dock first (docks are prepended).
 			_find_similar_root_dock (d->data, data);
@@ -475,6 +496,8 @@ void cairo_dock_foreach_icons_in_docks (CairoDockForeachIconFunc pFunction, gpoi
 
 static gboolean _cairo_dock_hide_dock_if_parent (gchar *cDockName, CairoDock *pDock, CairoDock *pChildDock)
 {
+	if (pDock == pChildDock)
+		return FALSE;
 	if (pDock->container.bInside)
 		return FALSE;
 		
@@ -488,14 +511,6 @@ static gboolean _cairo_dock_hide_dock_if_parent (gchar *cDockName, CairoDock *pD
 		}
 		else
 		{
-			if (pDock->iScrollOffset != 0)  // on remet systematiquement a 0 l'offset pour les containers.
-			{
-				pDock->iScrollOffset = 0;
-				pDock->container.iMouseX = pDock->container.iWidth / 2;  // utile ?
-				pDock->container.iMouseY = 0;
-				cairo_dock_calculate_dock_icons (pDock);
-			}
-
 			//cd_message ("on cache %s par parente", cDockName);
 			gtk_widget_hide (pDock->container.pWidget);
 			cairo_dock_hide_parent_dock (pDock);
@@ -518,7 +533,7 @@ gboolean cairo_dock_hide_child_docks (CairoDock *pDock)
 		icon = ic->data;
 		if (icon->pSubDock == NULL)
 			continue;
-		if (GTK_WIDGET_VISIBLE (icon->pSubDock->container.pWidget))
+		if (gldi_container_is_visible (CAIRO_CONTAINER (icon->pSubDock)))
 		{
 			if (icon->pSubDock->container.bInside)
 			{
@@ -528,7 +543,6 @@ gboolean cairo_dock_hide_child_docks (CairoDock *pDock)
 			else if (icon->pSubDock->iSidLeaveDemand == 0)  // si on sort du dock sans passer par le sous-dock, par exemple en sortant par le bas.
 			{
 				//cd_debug ("on cache %s par filiation", icon->cName);
-				icon->pSubDock->iScrollOffset = 0;
 				icon->pSubDock->fFoldingFactor = (myDocksParam.bAnimateSubDock ? 1 : 0);  /// 0
 				gtk_widget_hide (icon->pSubDock->container.pWidget);
 			}
@@ -538,13 +552,14 @@ gboolean cairo_dock_hide_child_docks (CairoDock *pDock)
 }
 
 
-static void _reload_buffer_in_one_dock (const gchar *cDockName, CairoDock *pDock, gpointer data)
+static void _reload_buffer_in_one_dock (/**const gchar *cDockName, */CairoDock *pDock, gpointer data)
 {
-	cairo_dock_reload_buffers_in_dock (pDock, GPOINTER_TO_INT (data), FALSE);
+	cairo_dock_reload_buffers_in_dock (pDock, TRUE, GPOINTER_TO_INT (data));
 }
-void cairo_dock_reload_buffers_in_all_docks (gboolean bReloadAppletsToo)
+void cairo_dock_reload_buffers_in_all_docks (gboolean bUpdateIconSize)
 {
-	g_hash_table_foreach (s_hDocksTable, (GHFunc) _reload_buffer_in_one_dock, GINT_TO_POINTER (bReloadAppletsToo));
+	///g_hash_table_foreach (s_hDocksTable, (GHFunc) _reload_buffer_in_one_dock, GINT_TO_POINTER (bUpdateIconSize));
+	g_list_foreach (s_pRootDockList, (GFunc)_reload_buffer_in_one_dock, GINT_TO_POINTER (bUpdateIconSize));  // we load the root docks first, so that sub-docks can have the correct icon size.
 	
 	cairo_dock_draw_subdock_icons ();
 }
@@ -561,7 +576,7 @@ static void _cairo_dock_draw_one_subdock_icon (const gchar *cDockName, CairoDock
 		&& (CAIRO_DOCK_ICON_TYPE_IS_CONTAINER (icon) || CAIRO_DOCK_IS_MULTI_APPLI (icon) || CAIRO_DOCK_IS_APPLET (icon))
 		&& (icon->iSubdockViewType != 0
 			|| (CAIRO_DOCK_IS_MULTI_APPLI (icon) && !myIndicatorsParam.bUseClassIndic))
-		&& icon->iSidRedrawSubdockContent == 0)  // icone de sous-dock ou de classe ou d'applets.
+		/**&& icon->iSidRedrawSubdockContent == 0*/)  // icone de sous-dock ou de classe ou d'applets.
 		{
 			cairo_dock_trigger_redraw_subdock_content_on_icon (icon);
 		}
@@ -574,6 +589,15 @@ void cairo_dock_draw_subdock_icons (void)
 }
 
 
+/**static void _update_dock_size (const gchar *cDockName, CairoDock *pDock, G_GNUC_UNUSED gpointer data)
+{
+	cairo_dock_update_dock_size (pDock);
+}
+void cairo_dock_update_all_docks_size (void)
+{
+	g_hash_table_foreach (s_hDocksTable, (GHFunc) _update_dock_size, NULL);
+}
+
 static void _cairo_dock_reset_one_dock_view (gchar *cDockName, CairoDock *pDock, gpointer data)
 {
 	cairo_dock_set_renderer (pDock, NULL);  // on met NULL plutot que CAIRO_DOCK_DEFAULT_RENDERER_NAME pour ne pas ecraser le nom de la vue.
@@ -582,7 +606,7 @@ void cairo_dock_reset_all_views (void)
 {
 	//g_print ("%s ()\n", __func__);
 	g_hash_table_foreach (s_hDocksTable, (GHFunc) _cairo_dock_reset_one_dock_view, NULL);
-}
+}*/
 
 static void _cairo_dock_set_one_dock_view_to_default (gchar *cDockName, CairoDock *pDock, gpointer data)
 {
@@ -592,9 +616,6 @@ static void _cairo_dock_set_one_dock_view_to_default (gchar *cDockName, CairoDoc
 	{
 		cairo_dock_set_default_renderer (pDock);
 		cairo_dock_update_dock_size (pDock);
-		pDock->pRenderer->calculate_icons (pDock);
-		if (pDock->iRefCount == 0 && pDock->iVisibility == CAIRO_DOCK_VISI_RESERVE)
-			cairo_dock_reserve_space_for_dock (pDock, TRUE);
 	}
 }
 void cairo_dock_set_all_views_to_default (int iDockType)
@@ -625,9 +646,6 @@ void cairo_dock_write_root_dock_gaps (CairoDock *pDock)
 		if (! g_file_test (cConfFilePath, G_FILE_TEST_EXISTS))
 		{
 			cairo_dock_copy_file (GLDI_SHARE_DATA_DIR"/"CAIRO_DOCK_MAIN_DOCK_CONF_FILE, cConfFilePath);
-			/**gchar *cCommand = g_strdup_printf ("cp '%s/%s' '%s'", GLDI_SHARE_DATA_DIR, CAIRO_DOCK_MAIN_DOCK_CONF_FILE, cConfFilePath);
-			int r = system (cCommand);
-			g_free (cCommand);*/
 		}
 		
 		cairo_dock_update_conf_file (cConfFilePath,
@@ -638,6 +656,79 @@ void cairo_dock_write_root_dock_gaps (CairoDock *pDock)
 	}
 }
 
+int cairo_dock_convert_icon_size_to_pixels (GldiIconSizeEnum s, double *fMaxScale, double *fReflectSize, int *iIconGap)
+{
+	int iIconSize;
+	switch (s)
+	{
+		case ICON_DEFAULT:
+		default:
+			iIconSize = myIconsParam.iIconWidth;
+			*fMaxScale = 1 + myIconsParam.fAmplitude;
+			*iIconGap = myIconsParam.iIconGap;
+			*fReflectSize = myIconsParam.fReflectHeightRatio;
+		break;
+		case ICON_TINY:
+			iIconSize = ICON_SIZE_TINY;
+			*fMaxScale = 2;
+			*iIconGap = 4;
+			*fReflectSize = .4;
+		break;
+		case ICON_VERY_SMALL:
+			iIconSize = ICON_SIZE_VERY_SMALL;
+			*fMaxScale = 1.8;
+			*iIconGap = 4;
+			*fReflectSize = .4;
+		break;
+		case ICON_SMALL:
+			iIconSize = ICON_SIZE_SMALL;
+			*fMaxScale = 1.8;
+			*iIconGap = 4;
+			*fReflectSize = .4;
+		break;
+		case ICON_MEDIUM:
+			iIconSize = ICON_SIZE_MEDIUM;
+			*fMaxScale = 1.6;
+			*iIconGap = 3;
+			*fReflectSize = .5;
+		break;
+		case ICON_BIG:
+			iIconSize = ICON_SIZE_BIG;
+			*fMaxScale = 1.5;
+			*iIconGap = 2;
+			*fReflectSize = .6;
+		break;
+		case ICON_HUGE:
+			iIconSize = ICON_SIZE_HUGE;
+			*fMaxScale = 1.3;
+			*iIconGap = 2;
+			*fReflectSize = .6;
+		break;
+	}
+	return iIconSize;
+}
+
+GldiIconSizeEnum cairo_dock_convert_icon_size_to_enum (int iIconSize)
+{
+	GldiIconSizeEnum s = ICON_DEFAULT;
+	if (iIconSize <= ICON_SIZE_TINY+2)
+		s = ICON_TINY;
+	else if (iIconSize <= ICON_SIZE_VERY_SMALL+2)
+		s = ICON_VERY_SMALL;
+	else if (iIconSize >= ICON_SIZE_HUGE-2)
+		s = ICON_HUGE;
+	else if (iIconSize > ICON_SIZE_MEDIUM)
+		s = ICON_BIG;
+	else
+	{
+		if (myIconsParam.fAmplitude >= 2 || iIconSize <= ICON_SIZE_SMALL)
+			s = ICON_SMALL;
+		else
+			s = ICON_MEDIUM;  // moyennes.
+	}
+	return s;
+}
+
 static gboolean cairo_dock_read_root_dock_config (const gchar *cDockName, CairoDock *pDock)
 {
 	g_return_val_if_fail (cDockName != NULL && pDock != NULL, FALSE);
@@ -687,11 +778,18 @@ static gboolean cairo_dock_read_root_dock_config (const gchar *cDockName, CairoD
 	else
 		pDock->iNumScreen = pDock->iScreenOffsetX = pDock->iScreenOffsetY = 0;
 	
-	//\______________ Visibilite.
+	//\______________ Visibility.
 	CairoDockVisibility iVisibility = cairo_dock_get_integer_key_value (pKeyFile, "Behavior", "visibility", &bFlushConfFileNeeded, FALSE, "Position", NULL);
 	cairo_dock_set_dock_visibility (pDock, iVisibility);
 	
-	//\______________ Vue.
+	//\______________ Icons size.
+	int s = cairo_dock_get_integer_key_value (pKeyFile, "Appearance", "icon size", &bFlushConfFileNeeded, ICON_DEFAULT, NULL, NULL);  // ICON_DEFAULT <=> same as main dock
+	double fMaxScale, fReflectSize;
+	int iIconGap;
+	pDock->iIconSize = cairo_dock_convert_icon_size_to_pixels (s, &fMaxScale, &fReflectSize, &iIconGap);
+	pDock->bGlobalIconSize = (s == ICON_DEFAULT);
+	
+	//\______________ View.
 	g_free (pDock->cRendererName);
 	pDock->cRendererName = cairo_dock_get_string_key_value (pKeyFile, "Appearance", "main dock view", &bFlushConfFileNeeded, NULL, "Views", NULL);
 	
@@ -732,7 +830,6 @@ static gboolean cairo_dock_read_root_dock_config (const gchar *cDockName, CairoD
 	if (bFlushConfFileNeeded)
 	{
 		//g_print ("update %s conf file\n", cDockName);
-		///cairo_dock_flush_conf_file (pKeyFile, cConfFilePath, GLDI_SHARE_DATA_DIR, CAIRO_DOCK_MAIN_DOCK_CONF_FILE);
 		cairo_dock_upgrade_conf_file (cConfFilePath, pKeyFile, GLDI_SHARE_DATA_DIR"/"CAIRO_DOCK_MAIN_DOCK_CONF_FILE);
 	}
 	
@@ -759,9 +856,6 @@ void cairo_dock_add_root_dock_config_for_name (const gchar *cDockName)
 	cd_debug ("%s (%s)", __func__, cDockName);
 	gchar *cConfFilePath = g_strdup_printf ("%s/%s.conf", g_cCurrentThemePath, cDockName);
 	cairo_dock_copy_file (GLDI_SHARE_DATA_DIR"/"CAIRO_DOCK_MAIN_DOCK_CONF_FILE, cConfFilePath);
-	/**gchar *cCommand = g_strdup_printf ("cp '%s' '%s'", GLDI_SHARE_DATA_DIR"/"CAIRO_DOCK_MAIN_DOCK_CONF_FILE, cConfFilePath);
-	int r = system (cCommand);
-	g_free (cCommand);*/
 	
 	// on placera le nouveau dock a l'oppose du main dock, meme ecran et meme visibilite.
 	cairo_dock_update_conf_file (cConfFilePath,
@@ -792,17 +886,13 @@ void cairo_dock_reload_one_root_dock (const gchar *cDockName, CairoDock *pDock)
 {
 	cairo_dock_read_root_dock_config (cDockName, pDock);
 	
-	///cairo_dock_load_buffers_in_one_dock (pDock);  // recharge les icones et les applets.
-	cairo_dock_reload_buffers_in_dock (pDock, TRUE, TRUE);  // recharge les icones et les applets, recursivement.
+	cairo_dock_set_default_renderer (pDock);
 	
 	pDock->backgroundBuffer.iWidth ++;  // pour forcer le chargement du fond.
-	cairo_dock_set_default_renderer (pDock);
-	cairo_dock_update_dock_size (pDock);
-	cairo_dock_calculate_dock_icons (pDock);
+	cairo_dock_reload_buffers_in_dock (pDock, TRUE, TRUE);
+	
+	_cairo_dock_draw_one_subdock_icon (NULL, pDock, NULL);  // container-icons may be drawn differently according to the orientation (ex.: box). must be done after sub-docks are reloaded.
 	
-	cairo_dock_move_resize_dock (pDock);
-	if (pDock->iVisibility == CAIRO_DOCK_VISI_RESERVE)  // la position/taille a change, il faut refaire la reservation.
-		cairo_dock_reserve_space_for_dock (pDock, TRUE);
 	gtk_widget_queue_draw (pDock->container.pWidget);
 }
 
@@ -849,8 +939,8 @@ void cairo_dock_synchronize_one_sub_dock_orientation (CairoDock *pSubDock, Cairo
 	{
 		pSubDock->container.bDirectionUp = pDock->container.bDirectionUp;
 		pSubDock->container.bIsHorizontal = pDock->container.bIsHorizontal;
-		if (bReloadBuffersIfNecessary)
-			cairo_dock_reload_reflects_in_dock (pSubDock);
+		/**if (bReloadBuffersIfNecessary)
+			cairo_dock_reload_reflects_in_dock (pSubDock);*/
 		cairo_dock_update_dock_size (pSubDock);
 		
 		cairo_dock_synchronize_sub_docks_orientation (pSubDock, bReloadBuffersIfNecessary);
@@ -1017,39 +1107,43 @@ static gboolean _cairo_dock_unhide_dock_delayed (CairoDock *pDock)
 	pDock->iSidUnhideDelayed = 0;
 	return FALSE;
 }
-static void _cairo_dock_unhide_root_dock_on_mouse_hit (CairoDock *pDock, int *pMouse)
+static void _cairo_dock_unhide_root_dock_on_mouse_hit (CairoDock *pDock, CDMousePolling *pMouse)
 {
-	static int iPrevPointerX = -1, iPrevPointerY = -1;
 	if (! pDock->bAutoHide && pDock->iVisibility != CAIRO_DOCK_VISI_KEEP_BELOW)
 		return;
 	
 	//\________________ On recupere la position du pointeur.
 	gint x, y;
-	if (! pMouse[0])  // pas encore recupere le pointeur.
+	if (! pMouse->bUpToDate)  // pas encore recupere le pointeur.
 	{
-		pMouse[0] = TRUE;
-		gdk_display_get_pointer (gdk_display_get_default(), NULL, &x, &y, NULL);
-		if (x == pMouse[1] && y == pMouse[2])  // le pointeur n'a pas bouge, on quitte.
+		pMouse->bUpToDate = TRUE;
+		gldi_display_get_pointer (&x, &y);
+		if (x == pMouse->x && y == pMouse->y)  // le pointeur n'a pas bouge, on quitte.
 		{
-			pMouse[3] = TRUE;
+			pMouse->bNoMove = TRUE;
 			return ;
 		}
-		pMouse[3] = FALSE;
-		pMouse[1] = x;
-		pMouse[2] = y;
+		pMouse->bNoMove = FALSE;
+		pMouse->dx = (x - pMouse->x);
+		pMouse->dy = (y - pMouse->y);
+		double d = sqrt (pMouse->dx * pMouse->dx + pMouse->dy * pMouse->dy);
+		pMouse->dx /= d;
+		pMouse->dy /= d;
+		pMouse->x = x;
+		pMouse->y = y;
 	}
 	else  // le pointeur a ete recupere auparavant.
 	{
-		if (pMouse[3])  // position inchangee.
+		if (pMouse->bNoMove)  // position inchangee.
 			return;
-		x = pMouse[1];
-		y = pMouse[2];
+		x = pMouse->x;
+		y = pMouse->y;
 	}
 	
 	if (!pDock->container.bIsHorizontal)
 	{
-		x = pMouse[2];
-		y = pMouse[1];
+		x = pMouse->y;
+		y = pMouse->x;
 	}
 	if (pDock->container.bDirectionUp)
 	{
@@ -1057,17 +1151,18 @@ static void _cairo_dock_unhide_root_dock_on_mouse_hit (CairoDock *pDock, int *pM
 	}
 	
 	//\________________ On verifie les conditions.
-	int x1, x2;
+	int x1, x2;  // coordinates range on the screen edge.
 	gboolean bShow = FALSE;
 	switch (myDocksParam.iCallbackMethod)
 	{
-		case 0:
+		case CAIRO_HIT_SCREEN_BORDER:
 		default:
 			if (y != 0)
 				break;
 			bShow = TRUE;
 		break;
-		case 1:
+		
+		case CAIRO_HIT_DOCK_PLACE:
 			if (y != 0)
 				break;
 			x1 = pDock->container.iWindowPositionX + MIN (15, pDock->container.iWidth/3);  // on evite les coins, car c'est en fait le but de cette option (les coins peuvent etre utilises par le WM pour declencher des actions).
@@ -1076,14 +1171,14 @@ static void _cairo_dock_unhide_root_dock_on_mouse_hit (CairoDock *pDock, int *pM
 				break;
 			bShow = TRUE;
 		break;
-		case 2:
+		case CAIRO_HIT_SCREEN_CORNER:
 			if (y != 0)
 				break;
 			if (x > 0 && x < g_desktopGeometry.iScreenWidth[pDock->container.bIsHorizontal] - 1)
 				break ;
 			bShow = TRUE;
 		break;
-		case 3:
+		case CAIRO_HIT_ZONE:
 			if (y > myDocksParam.iZoneHeight)
 				break;
 			x1 = pDock->container.iWindowPositionX + (pDock->container.iWidth - myDocksParam.iZoneWidth)/2;  // on evite les coins, car c'est en fait le but de cette option (les coins peuvent etre utilises par le WM pour declencher des actions).
@@ -1105,11 +1200,28 @@ static void _cairo_dock_unhide_root_dock_on_mouse_hit (CairoDock *pDock, int *pM
 	}
 	
 	//\________________ On montre ou on programme le montrage du dock.
-	//g_print (" dock will be shown (%d)\n", pDock->bIsMainDock);
-	if (myDocksParam.iUnhideDockDelay != 0)  // on programme une apparition.
+	int nx, ny;  // normal vector to the screen edge.
+	double cost;  // cos (teta), where teta = angle between mouse vector and dock's normal
+	double f = 1.;  // delay factor
+	if (pDock->container.bIsHorizontal)
+	{
+		nx = 0;
+		ny = (pDock->container.bDirectionUp ? -1 : 1);
+	}
+	else
+	{
+		ny = 0;
+		nx = (pDock->container.bDirectionUp ? -1 : 1);
+	}
+	cost = nx * pMouse->dx + ny * pMouse->dy;
+	f = 2 + cost;  // so if cost = -1, we arrive straight onto the screen edge, and f = 1, => normal delay. if cost = 0, f = 2 and we have a bigger delay.
+	
+	int iDelay = f * myDocksParam.iUnhideDockDelay;
+	//g_print (" dock will be shown in %dms (%.2f, %d)\n", iDelay, f, pDock->bIsMainDock);
+	if (iDelay != 0)  // on programme une apparition.
 	{
 		if (pDock->iSidUnhideDelayed == 0)
-			pDock->iSidUnhideDelayed = g_timeout_add (myDocksParam.iUnhideDockDelay, (GSourceFunc) _cairo_dock_unhide_dock_delayed, (gpointer) pDock);
+			pDock->iSidUnhideDelayed = g_timeout_add (iDelay, (GSourceFunc) _cairo_dock_unhide_dock_delayed, (gpointer) pDock);
 	}
 	else  // on montre le dock tout de suite.
 	{
@@ -1119,10 +1231,10 @@ static void _cairo_dock_unhide_root_dock_on_mouse_hit (CairoDock *pDock, int *pM
 
 static gboolean _cairo_dock_poll_screen_edge (gpointer data)  // thanks to Smidgey for the pop-up patch !
 {
-	static int mouse[4];
-	mouse[0] = FALSE;
+	static CDMousePolling mouse;
 	
-	g_list_foreach (s_pRootDockList, (GFunc) _cairo_dock_unhide_root_dock_on_mouse_hit, mouse);
+	mouse.bUpToDate = FALSE;  // mouse position will be updated by the first hidden dock.
+	g_list_foreach (s_pRootDockList, (GFunc) _cairo_dock_unhide_root_dock_on_mouse_hit, &mouse);
 	
 	return TRUE;
 }
@@ -1131,7 +1243,7 @@ void cairo_dock_start_polling_screen_edge (void)
 	s_iNbPolls ++;
 	cd_debug ("%s (%d)", __func__, s_iNbPolls);
 	if (s_iSidPollScreenEdge == 0)
-		s_iSidPollScreenEdge = g_timeout_add (200, (GSourceFunc) _cairo_dock_poll_screen_edge, NULL);
+		s_iSidPollScreenEdge = g_timeout_add (MOUSE_POLLING_DT, (GSourceFunc) _cairo_dock_poll_screen_edge, NULL);
 }
 
 void cairo_dock_stop_polling_screen_edge (void)
@@ -1158,7 +1270,7 @@ void cairo_dock_set_dock_visibility (CairoDock *pDock, CairoDockVisibility iVisi
 	gboolean bKeepBelow0 = (pDock->iVisibility == CAIRO_DOCK_VISI_KEEP_BELOW);
 	gboolean bAutoHideOnOverlap0 = (pDock->iVisibility == CAIRO_DOCK_VISI_AUTO_HIDE_ON_OVERLAP);
 	gboolean bAutoHideOnAnyOverlap0 = (pDock->iVisibility == CAIRO_DOCK_VISI_AUTO_HIDE_ON_OVERLAP_ANY);
-		gboolean bAutoHide0 = (pDock->iVisibility == CAIRO_DOCK_VISI_AUTO_HIDE);
+	gboolean bAutoHide0 = (pDock->iVisibility == CAIRO_DOCK_VISI_AUTO_HIDE);
 	gboolean bShortKey0 = (pDock->iVisibility == CAIRO_DOCK_VISI_SHORTKEY);
 	
 	pDock->iVisibility = iVisibility;
@@ -1207,26 +1319,25 @@ void cairo_dock_set_dock_visibility (CairoDock *pDock, CairoDockVisibility iVisi
 		}
 	}
 	
-	//\_______________ changement dans le raccourci.
-	if (pDock->bIsMainDock && bShortKey != bShortKey0)
+	//\_______________ shortkey
+	if (pDock->bIsMainDock)
 	{
-		if (bShortKey0)  // option desormais non active => on remontre (le unbind s'est fait dans le Accessibility manager quand on avait encore le raccourci).
+		if (bShortKey)  // option is enabled.
 		{
-			cairo_dock_reposition_root_docks (FALSE);  // FALSE => tous.
-		}
-		else  // option nouvellement active => on bind et on cache.
-		{
-			if (cd_keybinder_bind (myDocksParam.cRaiseDockShortcut, (CDBindkeyHandler) cairo_dock_raise_from_shortcut, NULL))  // succes => on cache.
+			if (s_pPopupBinding && cd_keybinder_could_grab (s_pPopupBinding))  // a shortkey has been registered and grabbed to show/hide the dock => hide the dock.
 			{
 				gtk_widget_hide (pDock->container.pWidget);
 			}
-			else  // le bind n'a pas pu se faire.
+			else  // bind couldn't be done (no shortkey or couldn't grab it).
 			{
-				g_free (myDocksParam.cRaiseDockShortcut);
-				myDocksParam.cRaiseDockShortcut = NULL;
+				//g_print ("bind couldn't be done (no shortkey or couldn't grab it).\n");
 				pDock->iVisibility = CAIRO_DOCK_VISI_KEEP_ABOVE;
 			}
 		}
+		else if (bShortKey0)  // option is now disabled => show the dock.
+		{
+			cairo_dock_reposition_root_docks (FALSE);  // FALSE => tous.
+		}
 	}
 	
 	//\_______________ on arrete/demarre la scrutation des bords.
@@ -1312,6 +1423,10 @@ static gboolean get_config (GKeyFile *pKeyFile, CairoDocksParam *pDocksParam)
 	
 	pAccessibility->bExtendedMode = cairo_dock_get_boolean_key_value (pKeyFile, "Background", "extended", &bFlushConfFileNeeded, FALSE, "Accessibility", NULL);
 	
+	// hidden bg
+	double hcolor[4] = {.8, .8, .8, .5};
+	cairo_dock_get_double_list_key_value (pKeyFile, "Background", "hidden bg color", &bFlushConfFileNeeded, pBackground->fHiddenBg, 4, hcolor, NULL, NULL);
+	
 	// position
 	pPosition->iGapX = cairo_dock_get_integer_key_value (pKeyFile, "Position", "x gap", &bFlushConfFileNeeded, 0, NULL, NULL);
 	pPosition->iGapY = cairo_dock_get_integer_key_value (pKeyFile, "Position", "y gap", &bFlushConfFileNeeded, 0, NULL, NULL);
@@ -1395,16 +1510,8 @@ static gboolean get_config (GKeyFile *pKeyFile, CairoDocksParam *pDocksParam)
 		}
 		if (iVisibility == CAIRO_DOCK_VISI_SHORTKEY)
 		{
-			if (cShortkey == NULL)
-			{
-				cd_warning ("no shortcut defined to make the dock appear, we'll keep it above.");
-				iVisibility = CAIRO_DOCK_VISI_KEEP_ABOVE;
-			}
-			else
-			{
-				pAccessibility->cRaiseDockShortcut = cShortkey;
-				cShortkey = NULL;
-			}
+			pAccessibility->cRaiseDockShortcut = cShortkey;
+			cShortkey = NULL;
 		}
 	}
 	pAccessibility->iVisibility = iVisibility;
@@ -1415,9 +1522,9 @@ static gboolean get_config (GKeyFile *pKeyFile, CairoDocksParam *pDocksParam)
 		g_key_file_set_string (pKeyFile, "Accessibility", "hide effect", pAccessibility->cHideEffect);
 	}
 	
-	pAccessibility->iCallbackMethod = cairo_dock_get_integer_key_value (pKeyFile, "Accessibility", "callback", &bFlushConfFileNeeded, 0, NULL, NULL);
+	pAccessibility->iCallbackMethod = cairo_dock_get_integer_key_value (pKeyFile, "Accessibility", "callback", &bFlushConfFileNeeded, CAIRO_HIT_DOCK_PLACE, NULL, NULL);
 	
-	if (pAccessibility->iCallbackMethod == 3)
+	if (pAccessibility->iCallbackMethod == CAIRO_HIT_ZONE)
 	{
 		if (! g_key_file_has_key (pKeyFile, "Accessibility", "zone size", NULL))
 		{
@@ -1438,7 +1545,14 @@ static gboolean get_config (GKeyFile *pKeyFile, CairoDocksParam *pDocksParam)
 	//\____________________ Autres parametres.
 	pAccessibility->iMaxAuthorizedWidth = cairo_dock_get_integer_key_value (pKeyFile, "Accessibility", "max_authorized_width", &bFlushConfFileNeeded, 0, "Position", NULL);  // obsolete, cache en conf.
 	
-	pAccessibility->iUnhideDockDelay = cairo_dock_get_integer_key_value (pKeyFile, "Accessibility", "unhide delay", &bFlushConfFileNeeded, 0, NULL, NULL);
+	double fSensitivity = cairo_dock_get_double_key_value (pKeyFile, "Accessibility", "edge sensitivity", &bFlushConfFileNeeded, -1, NULL, NULL);  // replace "unhide delay"
+	if (fSensitivity < 0)  // old param
+	{
+		int iUnhideDockDelay = g_key_file_get_integer (pKeyFile, "Accessibility", "unhide delay", NULL);
+		fSensitivity = iUnhideDockDelay / 1500.;  // 0 -> 0 = sensitive, 1500 -> 1 = not sensitive
+		g_key_file_set_double (pKeyFile, "Accessibility", "edge sensitivity", fSensitivity);
+	}
+	pAccessibility->iUnhideDockDelay = fSensitivity * 1000;  // so we decreased the old delay by 1.5, since we handle mouse movements better.
 	
 	pAccessibility->bAutoHideOnFullScreen = cairo_dock_get_boolean_key_value (pKeyFile, "Accessibility", "auto quick hide", &bFlushConfFileNeeded, FALSE, "TaskBar", NULL);
 	//pAccessibility->bAutoHideOnFullScreen = FALSE;
@@ -1456,7 +1570,7 @@ static gboolean get_config (GKeyFile *pKeyFile, CairoDocksParam *pDocksParam)
 		pAccessibility->bShowSubDockOnClick = (cairo_dock_get_integer_key_value (pKeyFile, "Accessibility", "show_on_click", &bFlushConfFileNeeded, 0, NULL, NULL) == 1);
 	
 	//\____________________ lock
-	pAccessibility->bLockAll = cairo_dock_get_boolean_key_value (pKeyFile, "Accessibility", "lock all", &bFlushConfFileNeeded, FALSE, NULL, NULL);
+	///pAccessibility->bLockAll = cairo_dock_get_boolean_key_value (pKeyFile, "Accessibility", "lock all", &bFlushConfFileNeeded, FALSE, NULL, NULL);
 	pAccessibility->bLockIcons = pAccessibility->bLockAll || cairo_dock_get_boolean_key_value (pKeyFile, "Accessibility", "lock icons", &bFlushConfFileNeeded, FALSE, NULL, NULL);
 	
 	// system
@@ -1532,6 +1646,27 @@ static void load (void)
 		
 		g_pMainDock->fFlatDockWidth = - myIconsParam.iIconGap;  // car on ne le connaissait pas encore au moment de sa creation.
 		
+		// register a key binding
+		if (myDocksParam.iVisibility == CAIRO_DOCK_VISI_SHORTKEY)  // register a key binding
+		{
+			if (s_pPopupBinding == NULL)
+			{
+				s_pPopupBinding = cd_keybinder_bind (myDocksParam.cRaiseDockShortcut,
+					"Cairo-Dock",
+					_("Pop up the main dock"),
+					GLDI_SHARE_DATA_DIR"/"CAIRO_DOCK_ICON,
+					g_cConfFile,
+					"Accessibility",
+					"raise shortcut",
+					(CDBindkeyHandler) cairo_dock_raise_from_shortcut,
+					NULL);
+			}
+			else
+			{
+				cd_keybinder_rebind (s_pPopupBinding, myDocksParam.cRaiseDockShortcut, NULL);
+			}
+		}
+		
 		cairo_dock_set_dock_visibility (g_pMainDock, myDocksParam.iVisibility);
 	}
 }
@@ -1541,17 +1676,10 @@ static void load (void)
  /// RELOAD ///
 //////////////
 
-static void _reload_bg (const gchar *cDockName, CairoDock *pDock, gpointer data)
+static void _reload_bg (CairoDock *pDock, gpointer data)
 {
-	cairo_dock_update_dock_size (pDock);
-	if (pDock->iRefCount == 0)
-	{
-		cairo_dock_load_dock_background (pDock);  // pour forcer le chargement du fond.
-		cairo_dock_calculate_dock_icons (pDock);
-		gtk_widget_queue_draw (pDock->container.pWidget);
-	}
-	if (pDock->iRefCount == 0 && pDock->iVisibility == CAIRO_DOCK_VISI_RESERVE)
-		cairo_dock_reserve_space_for_dock (pDock, TRUE);
+	pDock->backgroundBuffer.iWidth ++;  // force the reload
+	cairo_dock_trigger_load_dock_background (pDock);
 }
 static void _init_hiding (CairoDock *pDock, gpointer data)
 {
@@ -1575,7 +1703,7 @@ static void reload (CairoDocksParam *pPrevDocksParam, CairoDocksParam *pDocksPar
 	CairoDock *pDock = g_pMainDock;
 	
 	// background
-	cairo_dock_foreach_docks ((GHFunc)_reload_bg, NULL);
+	cairo_dock_foreach_root_docks ((GFunc)_reload_bg, NULL);
 	
 	// position
 	if (pPosition->bUseXinerama)
@@ -1604,6 +1732,7 @@ static void reload (CairoDocksParam *pPrevDocksParam, CairoDocksParam *pDocksPar
 	if (pPosition->iScreenBorder != pPrevPosition->iScreenBorder)
 	{
 		cairo_dock_set_dock_orientation (pDock, pPosition->iScreenBorder);
+		_cairo_dock_draw_one_subdock_icon (NULL, g_pMainDock, NULL);  // container-icons may be drawn differently according to the orientation (ex.: box).
 	}
 	pDock->bExtendedMode = pBackground->bExtendedMode;
 	cairo_dock_update_dock_size (pDock);  // si l'ecran ou l'orientation a change, la taille max a change aussi.
@@ -1618,38 +1747,31 @@ static void reload (CairoDocksParam *pPrevDocksParam, CairoDocksParam *pDocksPar
 	gtk_widget_queue_draw (pDock->container.pWidget);
 	
 	// accessibility
-	if (! pDock)
+	//\_______________ Shortkey.
+	if (pAccessibility->iVisibility == CAIRO_DOCK_VISI_SHORTKEY)
 	{
-		if (pPrevAccessibility->cRaiseDockShortcut != NULL)
-			cd_keybinder_unbind (pPrevAccessibility->cRaiseDockShortcut, (CDBindkeyHandler) cairo_dock_raise_from_shortcut);
-		return ;
-	}
-	
-	//\_______________ Shortcut.
-	if (pPrevAccessibility->cRaiseDockShortcut != NULL)  // il y'a un ancien raccourci.
-	{
-		if (pAccessibility->cRaiseDockShortcut == NULL || strcmp (pAccessibility->cRaiseDockShortcut, pPrevAccessibility->cRaiseDockShortcut) != 0)  // le raccourci a change, ou n'est plus d'actualite => on le un-bind ou on le re-bind.
+		if (s_pPopupBinding == NULL)
 		{
-			cd_keybinder_unbind (pPrevAccessibility->cRaiseDockShortcut, (CDBindkeyHandler) cairo_dock_raise_from_shortcut);
-			
-			if (pAccessibility->cRaiseDockShortcut != NULL)  // le raccourci a seulement change, mais l'option est toujours d'actualite, le set_visibility ne verra pas la difference, donc on re-binde.
-			{
-				if (! cd_keybinder_bind (pAccessibility->cRaiseDockShortcut, (CDBindkeyHandler) cairo_dock_raise_from_shortcut, NULL))  // le bind n'a pas pu se faire.
-				{
-					g_free (pAccessibility->cRaiseDockShortcut);
-					pAccessibility->cRaiseDockShortcut = NULL;
-					pAccessibility->iVisibility = CAIRO_DOCK_VISI_KEEP_ABOVE;
-				}
-			}
+			s_pPopupBinding = cd_keybinder_bind (myDocksParam.cRaiseDockShortcut,
+				"Cairo-Dock",
+				_("Pop up the main dock"),
+				GLDI_SHARE_DATA_DIR"/"CAIRO_DOCK_ICON,
+				g_cCurrentThemePath,
+				"Accessibility",
+				"raise shortcut",
+				(CDBindkeyHandler) cairo_dock_raise_from_shortcut,
+				NULL);
+		}
+		else
+		{
+			cd_keybinder_rebind (s_pPopupBinding, myDocksParam.cRaiseDockShortcut, NULL);
 		}
 	}
-	
-	//\_______________ Max Size.
-	/**if (pAccessibility->iMaxAuthorizedWidth != pPrevAccessibility->iMaxAuthorizedWidth ||
-		pAccessibility->bExtendedMode != pPrevAccessibility->bExtendedMode)
+	else
 	{
-		cairo_dock_set_all_views_to_default (1);  // 1 <=> tous les docks racines. met a jour la taille et reserve l'espace.
-	}*/
+		cd_keybinder_unbind (s_pPopupBinding);
+		s_pPopupBinding = NULL;
+	}
 	
 	//\_______________ Hiding effect.
 	if (cairo_dock_strings_differ (pAccessibility->cHideEffect, pPrevAccessibility->cHideEffect))
@@ -1686,6 +1808,9 @@ static void unload (void)
 	
 	_cairo_dock_stop_polling_screen_edge ();
 	s_bQuickHide = FALSE;
+	
+	cd_keybinder_unbind (s_pPopupBinding);
+	s_pPopupBinding = NULL;
 }
 
 
@@ -1700,10 +1825,10 @@ static void init (void)
 		g_free,
 		NULL);  // donc on peut utiliser g_hash_table_remove plutot que g_hash_table_steal.
 	
-	cairo_dock_register_notification_on_object (&myDocksMgr,
-		NOTIFICATION_RENDER_DOCK,
+	/**cairo_dock_register_notification_on_object (&myDocksMgr,
+		NOTIFICATION_RENDER,
 		(CairoDockNotificationFunc) cairo_dock_render_dock_notification,
-		CAIRO_DOCK_RUN_FIRST, NULL);
+		CAIRO_DOCK_RUN_FIRST, NULL);*/
 	cairo_dock_register_notification_on_object (&myDocksMgr,
 		NOTIFICATION_LEAVE_DOCK,
 		(CairoDockNotificationFunc) cairo_dock_on_leave_dock_notification,
@@ -1752,6 +1877,7 @@ void gldi_register_docks_manager (void)
 	myDocksMgr.mgr.iSizeOfData = 0;
 	// signals
 	cairo_dock_install_notifications_on_object (&myDocksMgr, NB_NOTIFICATIONS_DOCKS);
+	gldi_object_set_manager (GLDI_OBJECT (&myDocksMgr), GLDI_MANAGER (&myContainersMgr));
 	// register
 	gldi_register_manager (GLDI_MANAGER(&myDocksMgr));
 }
