@@ -58,7 +58,6 @@ cairo_t *cairo_dock_create_drawing_context_on_container (CairoContainer *pContai
 cairo_t *cairo_dock_create_drawing_context_on_area (CairoContainer *pContainer, GdkRectangle *pArea, double *fBgColor);
 
 
-
 double cairo_dock_calculate_extra_width_for_trapeze (double fFrameHeight, double fInclination, double fRadius, double fLineWidth);
 
 /** Compute the path of a rectangle with rounded corners. It doesn't stroke it, use cairo_stroke or cairo_fill to draw the line or the inside.
@@ -96,7 +95,7 @@ void cairo_dock_render_decorations_in_frame (cairo_t *pCairoContext, CairoDock *
 
 void cairo_dock_set_icon_scale_on_context (cairo_t *pCairoContext, Icon *icon, gboolean bIsHorizontal, double fRatio, gboolean bDirectionUp);
 
-void cairo_dock_draw_icon_reflect_cairo (Icon *icon, CairoDock *pDock, cairo_t *pCairoContext);
+void cairo_dock_draw_icon_reflect_cairo (Icon *icon, CairoContainer *pContainer, cairo_t *pCairoContext);
 
 /** Draw an icon and its reflect on a dock. Only draw the icon's image and reflect, and nothing else.
 *@param icon the icon to draw.
@@ -117,7 +116,7 @@ gboolean cairo_dock_render_icon_notification (gpointer pUserData, Icon *pIcon, C
 void cairo_dock_render_one_icon (Icon *icon, CairoDock *pDock, cairo_t *pCairoContext, double fDockMagnitude, gboolean bUseText);
 void cairo_dock_render_icons_linear (cairo_t *pCairoContext, CairoDock *pDock);
 
-void cairo_dock_render_one_icon_in_desklet (Icon *icon, cairo_t *pCairoContext, gboolean bUseReflect, gboolean bUseText, int iWidth);
+void cairo_dock_render_one_icon_in_desklet (Icon *icon, CairoContainer *pContainer, cairo_t *pCairoContext, gboolean bUseText);
 
 
 /** Draw a string linking the center of all the icons of a dock.
@@ -157,7 +156,7 @@ void cairo_dock_render_hidden_dock (cairo_t *pCairoContext, CairoDock *pDock);
 #define cairo_dock_damage_container_area(pContainer, area) _cairo_dock_compute_areas_bounded_box (pContainer->damageArea, area)
 
 #define cairo_dock_damage_icon(pIcon, pContainer) do {\
-	GdkRectangle area;\
+	cairo_rectangle_int_t area;\
 	cairo_dock_compute_icon_area (icon, pContainer, &area);\
 	cairo_dock_damage_container_area(pContainer, area); } while (0)
 	
