@@ -23,17 +23,12 @@
 #include <glib/gstdio.h>
 #include <gtk/gtk.h>
 
-#ifdef HAVE_GLITZ
-#include <gdk/gdkx.h>
-#include <glitz-glx.h>
-#include <cairo-glitz.h>
-#endif
-
 #include "gldi-config.h"
 #include "cairo-dock-draw.h"
 #include "cairo-dock-opengl.h"
 #include "cairo-dock-draw-opengl.h"
 #include "cairo-dock-icon-factory.h"
+#include "cairo-dock-icon-facility.h"
 #include "cairo-dock-module-factory.h"
 #include "cairo-dock-config.h"
 #include "cairo-dock-log.h"
@@ -47,12 +42,21 @@
 #include "cairo-dock-dock-factory.h"
 #include "cairo-dock-gui-manager.h"
 #include "cairo-dock-X-manager.h"
-#include "cairo-dock-emblem.h"
+///#include "cairo-dock-emblem.h"
+#include "cairo-dock-overlay.h"
 #define _MANAGER_DEF_
 #include "cairo-dock-flying-container.h"
 
-/**#define HAND_WIDTH 80
-#define HAND_HEIGHT 50*/
+// 1/4 + 1/4
+//     + 1
+/**
+ ______
+|  ____|
+| |    |
+| |    |
+|_|____|
+
+*/
 #define EXPLOSION_NB_FRAMES 10
 
 CairoFlyingManager myFlyingsMgr;
@@ -65,11 +69,12 @@ extern gboolean g_bUseOpenGL;
 static cairo_surface_t *s_pExplosionSurface = NULL;
 static GLuint s_iExplosionTexture = 0;
 static double s_fExplosionWidth, s_fExplosionHeight;
-static CairoEmblem *s_pEmblem = NULL;
+static CairoDockImageBuffer *s_pEmblem = NULL;
+///static CairoEmblem *s_pEmblem = NULL;
 
 static void _cairo_dock_load_emblem (Icon *pIcon)
 {
-	cairo_dock_free_emblem (s_pEmblem);
+	///cairo_dock_free_emblem (s_pEmblem);
 	const gchar *cImage = NULL;
 	if (CAIRO_DOCK_ICON_TYPE_IS_APPLET (pIcon))
 	{
@@ -79,9 +84,14 @@ static void _cairo_dock_load_emblem (Icon *pIcon)
 	{
 		cImage = GTK_STOCK_DELETE;
 	}
-	gchar *cIcon = cairo_dock_search_icon_s_path (cImage);
-	s_pEmblem = cairo_dock_make_emblem (cIcon, pIcon);
-	cairo_dock_set_emblem_position (s_pEmblem, CAIRO_DOCK_EMBLEM_UPPER_LEFT);
+	
+	int iWidth, iHeight;
+	cairo_dock_get_icon_extent (pIcon, &iWidth, &iHeight);
+	gchar *cIcon = cairo_dock_search_icon_s_path (cImage, MAX (iWidth/2, iHeight/2));
+	/**s_pEmblem = cairo_dock_make_emblem (cIcon, pIcon);
+	cairo_dock_set_emblem_position (s_pEmblem, CAIRO_DOCK_EMBLEM_UPPER_LEFT);*/
+	cairo_dock_free_image_buffer (s_pEmblem);
+	s_pEmblem = cairo_dock_create_image_buffer (cIcon, iWidth/2, iHeight/2, 0);
 	g_free (cIcon);
 }
 static void _cairo_dock_load_explosion_image (int iWidth)
@@ -155,7 +165,16 @@ static gboolean _cairo_dock_render_flying_container_notification (gpointer pUser
 			
 			cairo_restore (pCairoContext);
 			
-			_cairo_dock_apply_emblem_surface (s_pEmblem, pFlyingContainer->container.iWidth, pFlyingContainer->container.iHeight, pCairoContext);
+			///_cairo_dock_apply_emblem_surface (s_pEmblem, pFlyingContainer->container.iWidth, pFlyingContainer->container.iHeight, pCairoContext);
+			
+			if (s_pEmblem && s_pEmblem->pSurface)
+			{
+				cairo_set_source_surface (pCairoContext,
+					s_pEmblem->pSurface,
+					0,
+					0);
+				cairo_paint (pCairoContext);
+			}
 		}
 		else if (pFlyingContainer->container.iAnimationStep > 0)
 		{
@@ -179,6 +198,13 @@ static gboolean _cairo_dock_render_flying_container_notification (gpointer pUser
 	}
 	else
 	{
+		glClear (GL_COLOR_BUFFER_BIT);
+		_cairo_dock_set_blend_source ();
+		_cairo_dock_set_alpha (1.);
+		
+		gldi_glx_apply_desktop_background (CAIRO_CONTAINER (pFlyingContainer));
+		glLoadIdentity ();
+		
 		if (pIcon != NULL)
 		{
 			glPushMatrix ();
@@ -188,17 +214,17 @@ static gboolean _cairo_dock_render_flying_container_notification (gpointer pUser
 
 			glPopMatrix ();
 			
-			glPushMatrix ();
-			glTranslatef (pFlyingContainer->container.iWidth/2,
-				pFlyingContainer->container.iHeight/2,
-				0);
 			_cairo_dock_enable_texture ();
 			_cairo_dock_set_blend_alpha ();
 			
-			_cairo_dock_apply_emblem_texture (s_pEmblem, pFlyingContainer->container.iWidth, pFlyingContainer->container.iHeight);
+			///_cairo_dock_apply_emblem_texture (s_pEmblem, pFlyingContainer->container.iWidth, pFlyingContainer->container.iHeight);
+			if (s_pEmblem && s_pEmblem->iTexture != 0)
+			{
+				glBindTexture (GL_TEXTURE_2D, s_pEmblem->iTexture);
+				_cairo_dock_apply_current_texture_at_size_with_offset (s_pEmblem->iWidth, s_pEmblem->iHeight, s_pEmblem->iWidth/3, pFlyingContainer->container.iHeight - s_pEmblem->iHeight/3);
+			}
 			
 			_cairo_dock_disable_texture ();
-			glPopMatrix ();
 		}
 		else if (pFlyingContainer->container.iAnimationStep > 0)
 		{
@@ -224,36 +250,27 @@ static gboolean _cairo_dock_render_flying_container_notification (gpointer pUser
 
 
 static gboolean on_expose_flying_icon (GtkWidget *pWidget,
+#if (GTK_MAJOR_VERSION < 3)
 	GdkEventExpose *pExpose,
+#else
+	cairo_t *ctx,
+#endif
 	CairoFlyingContainer *pFlyingContainer)
 {
 	if (g_bUseOpenGL)
 	{
-		GdkGLContext *pGlContext = gtk_widget_get_gl_context (pWidget);
-		GdkGLDrawable *pGlDrawable = gtk_widget_get_gl_drawable (pWidget);
-		if (!gdk_gl_drawable_gl_begin (pGlDrawable, pGlContext))
+		if (! gldi_glx_begin_draw_container (CAIRO_CONTAINER (pFlyingContainer)))
 			return FALSE;
 		
-		glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
-		glLoadIdentity ();
+		cairo_dock_notify_on_object (pFlyingContainer, NOTIFICATION_RENDER, pFlyingContainer, NULL);
 		
-		cairo_dock_apply_desktop_background_opengl (CAIRO_CONTAINER (pFlyingContainer));
-		
-		cairo_dock_notify_on_object (&myFlyingsMgr, NOTIFICATION_RENDER_FLYING_CONTAINER, pFlyingContainer, NULL);
-		cairo_dock_notify_on_object (pFlyingContainer, NOTIFICATION_RENDER_FLYING_CONTAINER, pFlyingContainer, NULL);
-		
-		if (gdk_gl_drawable_is_double_buffered (pGlDrawable))
-			gdk_gl_drawable_swap_buffers (pGlDrawable);
-		else
-			glFlush ();
-		gdk_gl_drawable_gl_end (pGlDrawable);
+		gldi_glx_end_draw_container (CAIRO_CONTAINER (pFlyingContainer));
 	}
  	else
 	{
 		cairo_t *pCairoContext = cairo_dock_create_drawing_context_on_container (CAIRO_CONTAINER (pFlyingContainer));
 		
-		cairo_dock_notify_on_object (&myFlyingsMgr, NOTIFICATION_RENDER_FLYING_CONTAINER, pFlyingContainer, pCairoContext);
-		cairo_dock_notify_on_object (pFlyingContainer, NOTIFICATION_RENDER_FLYING_CONTAINER, pFlyingContainer, pCairoContext);
+		cairo_dock_notify_on_object (pFlyingContainer, NOTIFICATION_RENDER, pFlyingContainer, pCairoContext);
 		
 		cairo_destroy (pCairoContext);
 	}
@@ -265,6 +282,7 @@ static gboolean on_configure_flying_icon (GtkWidget* pWidget,
 	GdkEventConfigure* pEvent,
 	CairoFlyingContainer *pFlyingContainer)
 {
+	//g_print ("%s (%dx%d / %dx%d)\n", __func__, pFlyingContainer->container.iWidth, pFlyingContainer->container.iHeight, pEvent->width, pEvent->height);
 	if (pFlyingContainer->container.iWidth != pEvent->width || pFlyingContainer->container.iHeight != pEvent->height)
 	{
 		pFlyingContainer->container.iWidth = pEvent->width;
@@ -272,41 +290,73 @@ static gboolean on_configure_flying_icon (GtkWidget* pWidget,
 		
 		if (g_bUseOpenGL)
 		{
-			GdkGLContext* pGlContext = gtk_widget_get_gl_context (pWidget);
-			GdkGLDrawable* pGlDrawable = gtk_widget_get_gl_drawable (pWidget);
 			GLsizei w = pEvent->width;
 			GLsizei h = pEvent->height;
-			if (!gdk_gl_drawable_gl_begin (pGlDrawable, pGlContext))
+			if (! gldi_glx_make_current (CAIRO_CONTAINER (pFlyingContainer)))
 				return FALSE;
 			
-			glViewport(0, 0, w, h);
-			
 			cairo_dock_set_ortho_view (CAIRO_CONTAINER (pFlyingContainer));
-			
-			gdk_gl_drawable_gl_end (pGlDrawable);
 		}
 	}
+	
+	gtk_widget_queue_draw (pWidget);
 	return FALSE;
 }
 
+static gboolean _cairo_flying_container_animation_loop (CairoContainer *pContainer)
+{
+	CairoFlyingContainer *pFlyingContainer = CAIRO_FLYING_CONTAINER (pContainer);
+	gboolean bContinue = FALSE;
+	
+	if (pFlyingContainer->pIcon != NULL)
+	{
+		gboolean bIconIsAnimating = FALSE;
+		
+		cairo_dock_notify_on_object (pFlyingContainer->pIcon, NOTIFICATION_UPDATE_ICON, pFlyingContainer->pIcon, pFlyingContainer, &bIconIsAnimating);
+		if (! bIconIsAnimating)
+			pFlyingContainer->pIcon->iAnimationState = CAIRO_DOCK_STATE_REST;
+		else
+			bContinue = TRUE;
+	}
+	
+	cairo_dock_notify_on_object (pFlyingContainer, NOTIFICATION_UPDATE, pFlyingContainer, &bContinue);
+	
+	if (! bContinue)
+	{
+		cairo_dock_free_flying_container (pFlyingContainer);
+		return FALSE;
+	}
+	else
+		return TRUE;
+}
+
 CairoFlyingContainer *cairo_dock_create_flying_container (Icon *pFlyingIcon, CairoDock *pOriginDock)
 {
 	g_return_val_if_fail (pFlyingIcon != NULL, NULL);
 	CairoFlyingContainer * pFlyingContainer = g_new0 (CairoFlyingContainer, 1);
 	pFlyingContainer->container.iType = CAIRO_DOCK_TYPE_FLYING_CONTAINER;
 	GtkWidget* pWindow = cairo_dock_init_container (CAIRO_CONTAINER (pFlyingContainer));
-	cairo_dock_install_notifications_on_object (pFlyingContainer, NB_NOTIFICATIONS_FLYING_CONTAINER);
+	gldi_object_set_manager (GLDI_OBJECT (pFlyingContainer), GLDI_MANAGER (&myFlyingsMgr));
 	gtk_window_set_keep_above (GTK_WINDOW (pWindow), TRUE);
 	gtk_window_set_title (GTK_WINDOW(pWindow), "cairo-dock-flying-icon");
 	
+	gtk_widget_set_double_buffered (pFlyingContainer->container.pWidget, ! g_bUseOpenGL);
+	
+	pFlyingContainer->container.iface.animation_loop = _cairo_flying_container_animation_loop;
+	
 	pFlyingContainer->pIcon = pFlyingIcon;
+	cairo_dock_set_icon_container (pFlyingIcon, pFlyingContainer);
+	
 	pFlyingContainer->container.bIsHorizontal = TRUE;
 	pFlyingContainer->container.bDirectionUp = TRUE;
 	pFlyingContainer->container.fRatio = 1.;
 	pFlyingContainer->container.bUseReflect = FALSE;
-	
 	g_signal_connect (G_OBJECT (pWindow),
+		#if (GTK_MAJOR_VERSION < 3)
 		"expose-event",
+		#else
+		"draw",
+		#endif
 		G_CALLBACK (on_expose_flying_icon),
 		pFlyingContainer);
 	g_signal_connect (G_OBJECT (pWindow),
@@ -316,34 +366,33 @@ CairoFlyingContainer *cairo_dock_create_flying_container (Icon *pFlyingIcon, Cai
 	
 	pFlyingContainer->container.bInside = TRUE;
 	pFlyingIcon->bPointed = TRUE;
-	pFlyingIcon->fScale = 1.;
 	pFlyingIcon->fAlpha = 1.;
 	
-	pFlyingContainer->container.iWidth = pFlyingIcon->fWidth * pFlyingIcon->fScale * 1.333;
-	pFlyingContainer->container.iHeight = pFlyingIcon->fHeight * pFlyingIcon->fScale * 1.333;
-	pFlyingIcon->fDrawX = pFlyingIcon->fWidth * pFlyingIcon->fScale / 3;
-	pFlyingIcon->fDrawY = pFlyingIcon->fHeight * pFlyingIcon->fScale / 3;
+	int iWidth = pFlyingIcon->fWidth * pFlyingIcon->fScale * 1.25;
+	int iHeight = pFlyingIcon->fHeight * pFlyingIcon->fScale * 1.25;
+	pFlyingIcon->fDrawX = pFlyingIcon->fWidth * pFlyingIcon->fScale / 4;
+	pFlyingIcon->fDrawY = pFlyingIcon->fHeight * pFlyingIcon->fScale / 4;
 	
 	if (pOriginDock->container.bIsHorizontal)
 	{
-		pFlyingContainer->container.iWindowPositionX = pOriginDock->container.iWindowPositionX + pOriginDock->container.iMouseX - pFlyingContainer->container.iWidth/2;
-		pFlyingContainer->container.iWindowPositionY = pOriginDock->container.iWindowPositionY + pOriginDock->container.iMouseY - pFlyingContainer->container.iHeight/2;
+		pFlyingContainer->container.iWindowPositionX = pOriginDock->container.iWindowPositionX + pOriginDock->container.iMouseX - iWidth/2;
+		pFlyingContainer->container.iWindowPositionY = pOriginDock->container.iWindowPositionY + pOriginDock->container.iMouseY - iHeight/2;
 	}
 	else
 	{
-		pFlyingContainer->container.iWindowPositionY = pOriginDock->container.iWindowPositionX + pOriginDock->container.iMouseX - pFlyingContainer->container.iWidth/2;
-		pFlyingContainer->container.iWindowPositionX = pOriginDock->container.iWindowPositionY + pOriginDock->container.iMouseY - pFlyingContainer->container.iHeight/2;
+		pFlyingContainer->container.iWindowPositionY = pOriginDock->container.iWindowPositionX + pOriginDock->container.iMouseX - iWidth/2;
+		pFlyingContainer->container.iWindowPositionX = pOriginDock->container.iWindowPositionY + pOriginDock->container.iMouseY - iHeight/2;
 	}
 	gtk_window_present (GTK_WINDOW (pWindow));
 	/*cd_debug ("%s (%d;%d %dx%d)\n", __func__ pFlyingContainer->container.iWindowPositionX,
 		pFlyingContainer->container.iWindowPositionY,
 		pFlyingContainer->container.iWidth,
 		pFlyingContainer->container.iHeight);*/
-	gdk_window_move_resize (pWindow->window,
+	gdk_window_move_resize (gldi_container_get_gdk_window (CAIRO_CONTAINER (pFlyingContainer)),
 		pFlyingContainer->container.iWindowPositionX,
 		pFlyingContainer->container.iWindowPositionY,
-		pFlyingContainer->container.iWidth,
-		pFlyingContainer->container.iHeight);
+		iWidth,
+		iHeight);
 	/*gtk_window_resize (GTK_WINDOW (pWindow),
 		pFlyingContainer->container.iWidth,
 		pFlyingContainer->container.iHeight);
@@ -352,7 +401,7 @@ CairoFlyingContainer *cairo_dock_create_flying_container (Icon *pFlyingIcon, Cai
 		pFlyingContainer->container.iWindowPositionY);*/
 	
 	_cairo_dock_load_emblem (pFlyingIcon);
-	_cairo_dock_load_explosion_image (pFlyingContainer->container.iWidth);
+	_cairo_dock_load_explosion_image (iWidth);
 	
 	struct timeval tv;
 	int r = gettimeofday (&tv, NULL);
@@ -382,14 +431,19 @@ void cairo_dock_drag_flying_container (CairoFlyingContainer *pFlyingContainer, C
 void cairo_dock_free_flying_container (CairoFlyingContainer *pFlyingContainer)
 {
 	cd_debug ("%s ()", __func__);
+	if (pFlyingContainer->pIcon != NULL)
+		cairo_dock_set_icon_container (pFlyingContainer->pIcon, NULL);
 	cairo_dock_finish_container (CAIRO_CONTAINER (pFlyingContainer));
 	g_free (pFlyingContainer);
+	cairo_dock_free_image_buffer (s_pEmblem);
+	s_pEmblem = NULL;
 }
 
 void cairo_dock_terminate_flying_container (CairoFlyingContainer *pFlyingContainer)
 {
 	Icon *pIcon = pFlyingContainer->pIcon;
 	pFlyingContainer->pIcon = NULL;
+	cairo_dock_set_icon_container (pIcon, NULL);
 	pFlyingContainer->container.iAnimationStep = EXPLOSION_NB_FRAMES+1;
 	cairo_dock_launch_animation (CAIRO_CONTAINER (pFlyingContainer));  // au cas ou pas d'animation.
 	
@@ -424,7 +478,8 @@ static void unload (void)
 	}
 	if (s_pEmblem != NULL)
 	{
-		cairo_dock_free_emblem (s_pEmblem);
+		///cairo_dock_free_emblem (s_pEmblem);
+		cairo_dock_free_image_buffer (s_pEmblem);
 		s_pEmblem = NULL;
 	}
 }
@@ -437,11 +492,11 @@ static void unload (void)
 static void init (void)
 {
 	cairo_dock_register_notification_on_object (&myFlyingsMgr,
-		NOTIFICATION_UPDATE_FLYING_CONTAINER,
+		NOTIFICATION_UPDATE,
 		(CairoDockNotificationFunc) _cairo_dock_update_flying_container_notification,
 		CAIRO_DOCK_RUN_AFTER, NULL);
 	cairo_dock_register_notification_on_object (&myFlyingsMgr,
-		NOTIFICATION_RENDER_FLYING_CONTAINER,
+		NOTIFICATION_RENDER,
 		(CairoDockNotificationFunc) _cairo_dock_render_flying_container_notification,
 		CAIRO_DOCK_RUN_AFTER, NULL);
 }
@@ -470,6 +525,7 @@ void gldi_register_flying_manager (void)
 	myFlyingsMgr.mgr.iSizeOfData = 0;
 	// signals
 	cairo_dock_install_notifications_on_object (&myFlyingsMgr, NB_NOTIFICATIONS_FLYING_CONTAINER);
+	gldi_object_set_manager (GLDI_OBJECT (&myFlyingsMgr), GLDI_MANAGER (&myContainersMgr));
 	// register
 	gldi_register_manager (GLDI_MANAGER(&myFlyingsMgr));
 }
