@@ -27,18 +27,14 @@
 
 #include <cairo.h>
 
-#ifdef HAVE_GLITZ
-#include <gdk/gdkx.h>
-#include <glitz-glx.h>
-#include <cairo-glitz.h>
-#endif
 
 #include "cairo-dock-struct.h"
+#include "cairo-dock-notifications.h"
 #include "cairo-dock-module-factory.h"
 #include "cairo-dock-dock-facility.h"
 #include "cairo-dock-dock-manager.h"
 #include "cairo-dock-indicator-manager.h"
-#include "cairo-dock-config.h"
+#include "cairo-dock-keyfile-utilities.h"
 #include "cairo-dock-log.h"
 #include "cairo-dock-gui-manager.h"  // cairo_dock_trigger_refresh_launcher_gui
 #include "cairo-dock-animations.h"  // CairoDockHidingEffect
@@ -177,27 +173,6 @@ Icon* cairo_dock_get_last_icon (GList *pIconList)
 	return (pListTail != NULL ? pListTail->data : NULL);
 }
 
-Icon *cairo_dock_get_first_drawn_icon (CairoDock *pDock)
-{
-	if (pDock->pFirstDrawnElement != NULL)
-		return pDock->pFirstDrawnElement->data;
-	else
-		return cairo_dock_get_first_icon (pDock->icons);
-}
-
-Icon *cairo_dock_get_last_drawn_icon (CairoDock *pDock)
-{
-	if (pDock->pFirstDrawnElement != NULL)
-	{
-		if (pDock->pFirstDrawnElement->prev != NULL)
-			return pDock->pFirstDrawnElement->prev->data;
-		else
-			return cairo_dock_get_last_icon (pDock->icons);
-	}
-	else
-		return cairo_dock_get_last_icon (pDock->icons);;
-}
-
 Icon* cairo_dock_get_first_icon_of_group (GList *pIconList, CairoDockIconGroup iGroup)
 {
 	GList* ic;
@@ -248,25 +223,6 @@ Icon* cairo_dock_get_last_icon_of_order (GList *pIconList, CairoDockIconGroup iG
 	}
 	return NULL;
 }
-Icon* cairo_dock_get_last_icon_until_order (GList *pIconList, CairoDockIconGroup iGroup)
-{
-	CairoDockIconGroup iGroupOrder = cairo_dock_get_group_order (iGroup);
-	GList* ic;
-	Icon *icon = NULL;
-	for (ic = pIconList; ic != NULL; ic = ic->next)
-	{
-		icon = ic->data;
-		if (cairo_dock_get_icon_order (icon) > iGroupOrder)
-		{
-			if (ic->prev != NULL)
-				icon = ic->prev->data;
-			else
-				icon = NULL;
-			break;
-		}
-	}
-	return icon;
-}
 Icon* cairo_dock_get_first_icon_of_true_type (GList *pIconList, CairoDockIconTrueType iType)
 {
 	GList* ic;
@@ -441,7 +397,7 @@ void cairo_dock_compute_icon_area (Icon *icon, CairoContainer *pContainer, GdkRe
 	double fReflectSize = 0;
 	if (pContainer->bUseReflect)
 	{
-		fReflectSize = myIconsParam.fReflectSize * icon->fScale * fabs (icon->fHeightFactor) + icon->fDeltaYReflection + myDocksParam.iFrameMargin;  // un peu moyen le iFrameMargin mais bon ...
+		fReflectSize = /**myIconsParam.fReflectSize*/icon->fHeight * myIconsParam.fReflectHeightRatio * icon->fScale * fabs (icon->fHeightFactor) + icon->fDeltaYReflection + myDocksParam.iFrameMargin;  // un peu moyen le iFrameMargin mais bon ...
 	}
 	if (! myIndicatorsParam.bIndicatorOnIcon)
 		fReflectSize = MAX (fReflectSize, myIndicatorsParam.fIndicatorDeltaY * icon->fHeight);
@@ -513,7 +469,7 @@ void cairo_dock_normalize_icons_order (GList *pIconList, CairoDockIconGroup iGro
 	g_string_free (sDesktopFilePath, TRUE);
 }
 
-void cairo_dock_move_icon_after_icon (CairoDock *pDock, Icon *icon1, Icon *icon2)
+void cairo_dock_move_icon_after_icon (CairoDock *pDock, Icon *icon1, Icon *icon2)  // move icon1 after icon2,or at the beginning of the dock/group if icon2 is NULL.
 {
 	//g_print ("%s (%s, %.2f, %x)\n", __func__, icon1->cName, icon1->fOrder, icon2);
 	if ((icon2 != NULL) && fabs (cairo_dock_get_icon_order (icon1) - cairo_dock_get_icon_order (icon2)) > 1)
@@ -546,14 +502,13 @@ void cairo_dock_move_icon_after_icon (CairoDock *pDock, Icon *icon1, Icon *icon2
 	cairo_dock_write_order_in_conf_file (icon1, icon1->fOrder);
 	
 	//\_________________ On change sa place dans la liste.
-	pDock->pFirstDrawnElement = NULL;
 	pDock->icons = g_list_remove (pDock->icons, icon1);
 	pDock->icons = g_list_insert_sorted (pDock->icons,
 		icon1,
 		(GCompareFunc) cairo_dock_compare_icons_order);
 
 	//\_________________ On recalcule la largeur max, qui peut avoir ete influencee par le changement d'ordre.
-	cairo_dock_update_dock_size (pDock);
+	cairo_dock_trigger_update_dock_size (pDock);
 	
 	if (icon1->pSubDock != NULL && icon1->cClass != NULL)
 	{
@@ -569,48 +524,10 @@ void cairo_dock_move_icon_after_icon (CairoDock *pDock, Icon *icon1, Icon *icon2
 		cairo_dock_normalize_icons_order (pDock->icons, icon1->iGroup);
 	
 	//\_________________ Notify everybody.
-	cairo_dock_notify_on_object (&myDocksMgr, NOTIFICATION_ICON_MOVED, icon1, pDock);
 	cairo_dock_notify_on_object (pDock, NOTIFICATION_ICON_MOVED, icon1, pDock);
 }
 
 
-Icon *cairo_dock_foreach_icons_of_type (GList *pIconList, CairoDockIconGroup iGroup, CairoDockForeachIconFunc pFuntion, gpointer data)
-{
-	//g_print ("%s (%d)\n", __func__, iGroup);
-	if (pIconList == NULL)
-		return NULL;
-
-	Icon *icon;
-	GList *ic = pIconList, *next_ic;
-	gboolean bOneIconFound = FALSE;
-	Icon *pSeparatorIcon = NULL;
-	while (ic != NULL)  // on parcourt tout, inutile de complexifier la chose pour gagner 3ns.
-	{
-		icon = ic->data;
-		next_ic = ic->next;
-		if (icon->iGroup == iGroup)
-		{
-			bOneIconFound = TRUE;
-			pFuntion (icon, NULL, data);
-		}
-		else
-		{
-			if (CAIRO_DOCK_IS_AUTOMATIC_SEPARATOR (icon))
-			{
-				if ( (bOneIconFound && pSeparatorIcon == NULL) || (! bOneIconFound) )
-					pSeparatorIcon = icon;
-			}
-		}
-		ic = next_ic;
-	}
-
-	if (bOneIconFound)
-		return pSeparatorIcon;
-	else
-		return NULL;
-}
-
-
 
 void cairo_dock_update_icon_s_container_name (Icon *icon, const gchar *cNewParentDockName)
 {
@@ -655,18 +572,16 @@ void cairo_dock_set_icon_name_printf (Icon *pIcon, CairoContainer *pContainer_us
 
 void cairo_dock_set_quick_info (Icon *pIcon, CairoContainer *pContainer, const gchar *cQuickInfo)
 {
-	g_return_if_fail (pIcon != NULL);  // le contexte sera verifie plus loin.
-
+	g_return_if_fail (pIcon != NULL);
+	
 	if (pIcon->cQuickInfo != cQuickInfo)
 	{
 		g_free (pIcon->cQuickInfo);
 		pIcon->cQuickInfo = g_strdup (cQuickInfo);
 	}
 	
-	double fMaxScale = cairo_dock_get_max_scale (pContainer);
 	cairo_dock_load_icon_quickinfo (pIcon,
-		&myIconsParam.quickInfoTextDescription,
-		fMaxScale);
+		&myIconsParam.quickInfoTextDescription);
 }
 
 void cairo_dock_set_quick_info_printf (Icon *pIcon, CairoContainer *pContainer, const gchar *cQuickInfoFormat, ...)
@@ -758,3 +673,67 @@ gchar *cairo_dock_cut_string (const gchar *cString, int iNbCaracters)  // gere l
 	//g_print (" -> etiquette : %s\n", cTruncatedName);
 	return cTruncatedName;
 }
+
+
+GdkPixbuf *cairo_dock_icon_buffer_to_pixbuf (Icon *icon)
+{
+	g_return_val_if_fail (icon != NULL, NULL);
+	GdkPixbuf *pixbuf = NULL;
+	int w = 24, h = w;
+	int iWidth, iHeight;
+	cairo_dock_get_icon_extent (icon, &iWidth, &iHeight);
+	if (iWidth > 0 && iHeight > 0 && icon->pIconBuffer != NULL)
+	{
+		cairo_surface_t *surface = cairo_image_surface_create (CAIRO_FORMAT_RGB24,
+			w,
+			h);
+		cairo_t *pCairoContext = cairo_create (surface);
+		cairo_scale (pCairoContext, (double)w/iWidth, (double)h/iHeight);
+		cairo_set_source_surface (pCairoContext, icon->pIconBuffer, 0., 0.);
+		cairo_paint (pCairoContext);
+		cairo_destroy (pCairoContext);
+		guchar *d, *data = cairo_image_surface_get_data (surface);
+		int r = cairo_image_surface_get_stride (surface);
+		
+		// on la convertit en un pixbuf.
+		pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
+			TRUE,
+			8,
+			w,
+			h);
+		guchar *p, *pixels = gdk_pixbuf_get_pixels (pixbuf);
+		int iNbChannels = gdk_pixbuf_get_n_channels (pixbuf);
+		int iRowstride = gdk_pixbuf_get_rowstride (pixbuf);
+		
+		int x, y;
+		int red, green, blue;
+		float fAlphaFactor;
+		for (y = 0; y < h; y ++)
+		{
+			for (x = 0; x < w; x ++)
+			{
+				p = pixels + y * iRowstride + x * iNbChannels;
+				d = data + y * r + x * 4;
+				
+				fAlphaFactor = (float) d[3] / 255;
+				if (fAlphaFactor != 0)
+				{
+					red = d[0] / fAlphaFactor;
+					green = d[1] / fAlphaFactor;
+					blue = d[2] / fAlphaFactor;
+				}
+				else
+				{
+					red = blue = green = 0;
+				}
+				p[0] = blue;
+				p[1] = green;
+				p[2] = red;
+				p[3] = d[3];
+			}
+		}
+		
+		cairo_surface_destroy (surface);
+	}
+	return pixbuf;
+}
\ No newline at end of file
