@@ -45,6 +45,11 @@ G_BEGIN_DECLS
 */
 #define cairo_dock_get_icon_order(icon) cairo_dock_get_group_order ((icon)->iGroup)
 
+#define cairo_dock_set_icon_container(_pIcon, _pContainer) (_pIcon)->pContainer = CAIRO_CONTAINER (_pContainer)
+
+#define cairo_dock_get_icon_max_scale(pIcon) (pIcon->fHeight != 0 && pIcon->pContainer ? (pIcon->pContainer->bIsHorizontal ? pIcon->iImageHeight : pIcon->iImageWidth) / pIcon->fHeight : 1.)
+
+
 /** Get the type of an icon according to its content (launcher, appli, applet). This can be different from its group.
 *@param icon the icon.
 *@return the type of the icon.
@@ -97,20 +102,6 @@ Icon *cairo_dock_get_first_icon (GList *pIconList);
 */
 Icon *cairo_dock_get_last_icon (GList *pIconList);
 
-/*
-*Renvoie la 1ere icon a etre dessinee dans un dock (qui n'est pas forcement la 1ere icon de la liste, si l'utilisateur a scrolle).
-*@param pDock le dock.
-*@return la 1ere icon a etre dessinee, ou NULL si la liste is vide.
-*/
-Icon *cairo_dock_get_first_drawn_icon (CairoDock *pDock);
-
-/*
-*Renvoie la derniere icon a etre dessinee dans un dock (qui n'est pas forcement la derniere icon de la liste, si l'utilisateur a scrolle).
-*@param pDock le dock.
-*@return la derniere icon a etre dessinee, ou NULL si la liste is vide.
-*/
-Icon *cairo_dock_get_last_drawn_icon (CairoDock *pDock);
-
 /** Get the first icon of a given group.
 *@param pIconList a list of icons.
 *@param iGroup the group of icon.
@@ -141,8 +132,6 @@ Icon* cairo_dock_get_first_icon_of_order (GList *pIconList, CairoDockIconGroup i
 */
 Icon* cairo_dock_get_last_icon_of_order (GList *pIconList, CairoDockIconGroup iGroup);
 
-Icon* cairo_dock_get_last_icon_until_order (GList *pIconList, CairoDockIconGroup iGroup);
-
 Icon* cairo_dock_get_first_icon_of_true_type (GList *pIconList, CairoDockIconTrueType iType);
 
 /** Get the currently pointed icon in a list of icons.
@@ -247,31 +236,23 @@ void cairo_dock_normalize_icons_order (GList *pIconList, CairoDockIconGroup iGro
 
 void cairo_dock_move_icon_after_icon (CairoDock *pDock, Icon *icon1, Icon *icon2);
 
-/** Run an action on all the icons of a given group. The action can even destroy or remove the icon from the list.
-*@param pIconList a list of icons.
-*@param iGroup the group.
-*@param pFuntion the callback.
-*@param data data passed as a parameter of the callback.
-*@return the first automatic separator with another group, or NULL if there is none.
-*/
-Icon *cairo_dock_foreach_icons_of_type (GList *pIconList, CairoDockIconGroup iGroup, CairoDockForeachIconFunc pFuntion, gpointer data);
-
 /** Update the container's name of an icon with the name of a dock. In the case of a launcher or an applet, the conf file is updated too.
 *@param icon an icon.
 *@param cNewParentDockName the name of its new dock.
 */
 void cairo_dock_update_icon_s_container_name (Icon *icon, const gchar *cNewParentDockName);
 
-/** Make an icon static. Static icon are not animated when mouse hovers them.
+/** Make an icon static or not. Static icons are not animated when mouse hovers them.
 *@param icon an icon.
+*@param bStatic static or not.
 */
-#define cairo_dock_set_icon_static(icon) ((icon)->bStatic = TRUE)
+#define cairo_dock_set_icon_static(icon, _bStatic) (icon)->bStatic = _bStatic
 
 /** Make an icon always visible, even when the dock is hidden.
 *@param icon an icon.
 *@param bAlwaysVisible whether the icon is always visible or not.
 */
-#define cairo_dock_set_icon_always_visible(icon, bAlwaysVisible) ((bAlwaysVisible)->bStatic = bAlwaysVisible)
+#define cairo_dock_set_icon_always_visible(icon, _bAlwaysVisible) (icon)->bAlwaysVisible = _bAlwaysVisible
 
 /** Set the label of an icon. If it has a sub-dock, it is renamed (the name is possibly altered to stay unique). The label buffer is updated too.
 *@param cIconName the new label of the icon. You can even pass pIcon->cName.
@@ -319,5 +300,8 @@ void cairo_dock_set_quick_info_printf (Icon *pIcon, CairoContainer *pContainer,
 gchar *cairo_dock_cut_string (const gchar *cString, int iNbCaracters);
 
 
+GdkPixbuf *cairo_dock_icon_buffer_to_pixbuf (Icon *icon);
+
+
 G_END_DECLS
 #endif
