@@ -23,14 +23,6 @@
 
 #include <gtk/gtk.h>
 
-#ifdef HAVE_GLITZ
-#include <gdk/gdkx.h>
-#include <glitz-glx.h>
-#include <cairo-glitz.h>
-#endif
-
-#include <gtk/gtkgl.h>
-
 #include "gldi-config.h"
 #include "cairo-dock-draw.h"  // cairo_dock_erase_cairo_context
 #include "cairo-dock-draw-opengl.h"
@@ -38,6 +30,7 @@
 #include "cairo-dock-module-factory.h"  // cairo_dock_reload_module_instance
 #include "cairo-dock-log.h"
 #include "cairo-dock-applications-manager.h"  // myTaskbarParam.iAppliMaxNameLength
+#include "cairo-dock-dock-facility.h"  // cairo_dock_update_dock_size
 #include "cairo-dock-dock-manager.h"  // cairo_dock_synchronize_one_sub_dock_orientation
 #include "cairo-dock-backends-manager.h"  // cairo_dock_get_icon_container_renderer
 #include "cairo-dock-icon-facility.h"
@@ -56,7 +49,9 @@ const gchar *s_cRendererNames[4] = {NULL, "Emblem", "Stack", "Box"};  // c'est j
 inline Icon *cairo_dock_new_icon (void)
 {
 	Icon *_icon = g_new0 (Icon, 1);
-	cairo_dock_install_notifications_on_object (_icon, NB_NOTIFICATIONS_ICON);
+	///cairo_dock_install_notifications_on_object (_icon, NB_NOTIFICATIONS_ICON);
+	_icon->object.ref = 1;
+	gldi_object_set_manager (GLDI_OBJECT (_icon), GLDI_MANAGER (&myIconsMgr));
 	return _icon;
 }
 
@@ -83,7 +78,7 @@ void cairo_dock_free_icon_buffers (Icon *icon)
 		g_strfreev (icon->pMimeTypes);
 	
 	cairo_surface_destroy (icon->pIconBuffer);
-	cairo_surface_destroy (icon->pReflectionBuffer);
+	///cairo_surface_destroy (icon->pReflectionBuffer);
 	cairo_surface_destroy (icon->pTextBuffer);
 	cairo_surface_destroy (icon->pQuickInfoBuffer);
 	
@@ -101,47 +96,21 @@ void cairo_dock_free_icon_buffers (Icon *icon)
 //////////////
 
 
-static void _set_icon_size_generic (CairoContainer *pContainer, Icon *icon)
-{
-	if (icon->fWidth == 0)
-		icon->fWidth = 48;
-	if (icon->fHeight == 0)
-		icon->fHeight = 48;
-}
-void cairo_dock_set_icon_size (CairoContainer *pContainer, Icon *icon)
-{
-	if (! pContainer)
-	{
-		cd_debug ("icone dans aucun container => pas chargee");
-		return;
-	}
-	// taille de l'icone dans le container (hors ratio).
-	if (pContainer->iface.set_icon_size)
-		pContainer->iface.set_icon_size (pContainer, icon);
-	else
-		_set_icon_size_generic (pContainer, icon);
-	// la taille que devra avoir la texture s'en deduit.
-	double fMaxScale = cairo_dock_get_max_scale (pContainer);
-	if (CAIRO_DOCK_ICON_TYPE_IS_SEPARATOR (icon) && myIconsParam.bRevolveSeparator)
-	{
-		icon->iImageWidth = icon->fWidth * fMaxScale;
-		icon->iImageHeight = icon->fHeight * fMaxScale;
-	}
-	else
-	{
-		icon->iImageWidth = (pContainer->bIsHorizontal ? icon->fWidth : icon->fHeight) * fMaxScale;
-		icon->iImageHeight = (pContainer->bIsHorizontal ? icon->fHeight : icon->fWidth) * fMaxScale;
-	}
-}
-
 void cairo_dock_load_icon_image (Icon *icon, CairoContainer *pContainer)
 {
+	if (icon->pContainer == NULL)
+	{
+		//g_print ("/!\\ Icon %s is not inside a container !!!\n", icon->cName);
+		return;
+	}
 	CairoDockModuleInstance *pInstance = icon->pModuleInstance;  // this is the only function where we destroy/create the icon's surface, so we must handle the cairo-context here.
 	if (pInstance && pInstance->pDrawContext != NULL)
 	{
 		cairo_destroy (pInstance->pDrawContext);
 		pInstance->pDrawContext = NULL;
 	}
+	
+	//g_print ("%s (%s, %dx%d)\n", __func__, icon->cName, (int)icon->fWidth, (int)icon->fHeight);
 	if (icon->fWidth < 0 || icon->fHeight < 0)  // on ne veut pas de surface.
 	{
 		if (icon->pIconBuffer != NULL)
@@ -150,27 +119,23 @@ void cairo_dock_load_icon_image (Icon *icon, CairoContainer *pContainer)
 		if (icon->iIconTexture != 0)
 			_cairo_dock_delete_texture (icon->iIconTexture);
 		icon->iIconTexture = 0;
-		if (icon->pReflectionBuffer != NULL)
+		/**if (icon->pReflectionBuffer != NULL)
 			cairo_surface_destroy (icon->pReflectionBuffer);
-		icon->pReflectionBuffer = NULL;
+		icon->pReflectionBuffer = NULL;*/
 		return;
 	}
 	
-	if (icon->fWidth == 0 || icon->iImageWidth <= 0)
-	{
-		cairo_dock_set_icon_size (pContainer, icon);
-	}
-	//g_print ("%s (%.2fx%.2f ; %dx%d)\n", __func__, icon->fWidth, icon->fHeight, icon->iImageWidth, icon->iImageHeight);
+	g_return_if_fail (icon->fWidth > 0 & icon->iImageWidth > 0);  // the renderer of the container must have set the size beforehand, when the icon has been inserted into the container.
 	
 	//\______________ on reset les buffers (on garde la surface/texture actuelle pour les emblemes).
 	cairo_surface_t *pPrevSurface = icon->pIconBuffer;
 	GLuint iPrevTexture = icon->iIconTexture;
 	
-	if (icon->pReflectionBuffer != NULL)
+	/**if (icon->pReflectionBuffer != NULL)
 	{
 		cairo_surface_destroy (icon->pReflectionBuffer);
 		icon->pReflectionBuffer = NULL;
-	}
+	}*/
 	
 	//\______________ on charge la surface/texture.
 	if (icon->iface.load_image)
@@ -190,7 +155,6 @@ void cairo_dock_load_icon_image (Icon *icon, CairoContainer *pContainer)
 			icon->iImageHeight);
 		g_free (cIconPath);
 	}
-	cd_debug ("%s (%s) -> %.2fx%.2f", __func__, icon->cName, icon->fWidth, icon->fHeight);
 	
 	//\_____________ On met le background de l'icone si necessaire
 	if (icon->pIconBuffer != NULL && g_pIconBackgroundBuffer.pSurface != NULL && ! CAIRO_DOCK_ICON_TYPE_IS_SEPARATOR (icon))
@@ -209,16 +173,10 @@ void cairo_dock_load_icon_image (Icon *icon, CairoContainer *pContainer)
 	}
 	
 	//\______________ le reflet en mode cairo.
-	if (! g_bUseOpenGL && myIconsParam.fAlbedo > 0 && icon->pIconBuffer != NULL && ! (CAIRO_DOCK_ICON_TYPE_IS_APPLET (icon) && icon->cFileName == NULL))
+	/**if (! g_bUseOpenGL && myIconsParam.fAlbedo > 0 && icon->pIconBuffer != NULL && ! (CAIRO_DOCK_ICON_TYPE_IS_APPLET (icon) && icon->cFileName == NULL))
 	{
-		icon->pReflectionBuffer = cairo_dock_create_reflection_surface (icon->pIconBuffer,
-			icon->iImageWidth,
-			icon->iImageHeight,
-			myIconsParam.fReflectSize * cairo_dock_get_max_scale (pContainer),
-			myIconsParam.fAlbedo,
-			pContainer ? pContainer->bIsHorizontal : TRUE,
-			pContainer ? pContainer->bDirectionUp : TRUE);
-	}
+		cairo_dock_add_reflection_to_icon (icon, pContainer);
+	}*/
 	
 	//\______________ on charge la texture si elle ne l'a pas ete.
 	if (g_bUseOpenGL && (icon->iIconTexture == iPrevTexture || icon->iIconTexture == 0))
@@ -276,7 +234,7 @@ void cairo_dock_load_icon_text (Icon *icon, CairoDockLabelDescription *pTextDesc
 	}
 }
 
-void cairo_dock_load_icon_quickinfo (Icon *icon, CairoDockLabelDescription *pTextDescription, double fMaxScale)
+void cairo_dock_load_icon_quickinfo (Icon *icon, CairoDockLabelDescription *pTextDescription)
 {
 	cairo_surface_destroy (icon->pQuickInfoBuffer);
 	icon->pQuickInfoBuffer = NULL;
@@ -288,12 +246,12 @@ void cairo_dock_load_icon_quickinfo (Icon *icon, CairoDockLabelDescription *pTex
 	if (icon->cQuickInfo == NULL)
 		return ;
 
-	double fQuickInfoXOffset, fQuickInfoYOffset;
+	double fMaxScale = cairo_dock_get_icon_max_scale (icon);
 	icon->pQuickInfoBuffer = cairo_dock_create_surface_from_text_full (icon->cQuickInfo,
 		pTextDescription,
 		fMaxScale,
 		icon->fWidth * fMaxScale,
-		&icon->iQuickInfoWidth, &icon->iQuickInfoHeight, NULL, NULL);
+		&icon->iQuickInfoWidth, &icon->iQuickInfoHeight);
 	
 	if (g_bUseOpenGL && icon->pQuickInfoBuffer != NULL)
 	{
@@ -304,8 +262,9 @@ void cairo_dock_load_icon_quickinfo (Icon *icon, CairoDockLabelDescription *pTex
 
 void cairo_dock_load_icon_buffers (Icon *pIcon, CairoContainer *pContainer)
 {
-	if (pIcon->iSidLoadImage != 0)
+	if (pIcon->iSidLoadImage != 0)  // if a load was sheduled, cancel it and do it now (we need to load the applets' buffer before initializing the module).
 	{
+		//g_print (" load %s immediately\n", pIcon->cName);
 		g_source_remove (pIcon->iSidLoadImage);
 		pIcon->iSidLoadImage = 0;
 	}
@@ -314,100 +273,55 @@ void cairo_dock_load_icon_buffers (Icon *pIcon, CairoContainer *pContainer)
 
 	cairo_dock_load_icon_text (pIcon, &myIconsParam.iconTextDescription);
 
-	double fMaxScale = cairo_dock_get_max_scale (pContainer);
-	cairo_dock_load_icon_quickinfo (pIcon, &myIconsParam.quickInfoTextDescription, fMaxScale);
+	cairo_dock_load_icon_quickinfo (pIcon, &myIconsParam.quickInfoTextDescription);
 }
 
 static gboolean _load_icon_buffer_idle (Icon *pIcon)
 {
-	//g_print ("%s (%s; %dx%d; %.2fx%.2f; %x)\n", __func__, pIcon->cName, pIcon->iImageWidth, pIcon->iImageHeight, pIcon->fWidth, pIcon->fHeight, pIcon->pContainerForLoad);
+	//g_print ("%s (%s; %dx%d; %.2fx%.2f; %x)\n", __func__, pIcon->cName, pIcon->iImageWidth, pIcon->iImageHeight, pIcon->fWidth, pIcon->fHeight, pIcon->pContainer);
 	pIcon->iSidLoadImage = 0;
 	
-	CairoContainer *pContainer = pIcon->pContainerForLoad;
+	CairoContainer *pContainer = pIcon->pContainer;
 	if (pContainer)
 	{
 		cairo_dock_load_icon_image (pIcon, pContainer);
-
-		double fMaxScale = cairo_dock_get_max_scale (pContainer);
-		cairo_dock_load_icon_quickinfo (pIcon, &myIconsParam.quickInfoTextDescription, fMaxScale);
+		
+		cairo_dock_load_icon_quickinfo (pIcon, &myIconsParam.quickInfoTextDescription);
 		
 		cairo_dock_redraw_icon (pIcon, pContainer);
 		//g_print ("icon-factory: do 1 main loop iteration\n");
-		//gtk_main_iteration_do (FALSE);  /// check the consequences...	
+		//gtk_main_iteration_do (FALSE);  /// "unforseen consequences" : if _redraw_subdock_content_idle is planned just after, the container-icon stays blank in opengl only. couldn't figure why exactly :-/
 	}
 	return FALSE;
 }
-void cairo_dock_trigger_load_icon_buffers (Icon *pIcon, CairoContainer *pContainer)
+void cairo_dock_trigger_load_icon_buffers (Icon *pIcon)
 {
-	cairo_dock_set_icon_size (pContainer, pIcon);
-	pIcon->pContainerForLoad = pContainer;
 	if (pIcon->iSidLoadImage == 0)
 	{
 		//g_print ("trigger load for %s (%x)\n", pIcon->cName, pContainer);
-		//cairo_dock_load_icon_buffers (pIcon, pContainer);
-		cairo_dock_load_icon_text (pIcon, &myIconsParam.iconTextDescription);  // la vue peut avoir besoin de connaitre la taille du texte.
+		if (!pIcon->pTextBuffer)
+			cairo_dock_load_icon_text (pIcon, &myIconsParam.iconTextDescription);  // la vue peut avoir besoin de connaitre la taille du texte.
 		pIcon->iSidLoadImage = g_idle_add ((GSourceFunc)_load_icon_buffer_idle, pIcon);
 	}
 }
 
 
-void cairo_dock_reload_buffers_in_dock (CairoDock *pDock, gboolean bReloadAppletsToo, gboolean bRecursive)
-{
-	cd_message ("%s (%d, %d)", __func__, bReloadAppletsToo, bRecursive);
-
-	double fFlatDockWidth = - myIconsParam.iIconGap;
-	pDock->iMaxIconHeight = 0;
-	Icon* icon;
-	GList* ic;
-	for (ic = pDock->icons; ic != NULL; ic = ic->next)
-	{
-		icon = ic->data;
-		
-		if (CAIRO_DOCK_IS_APPLET (icon))
-		{
-			if (bReloadAppletsToo)  /// modif du 23/05/2009 : utiliser la taille avec ratio ici. les applets doivent faire attention a utiliser la fonction get_icon_extent().
-			{
-				cairo_dock_reload_module_instance (icon->pModuleInstance, FALSE);
-			}
-		}
-		else
-		{
-			///cairo_dock_set_icon_size (CAIRO_CONTAINER (pDock), icon);
-			cairo_dock_trigger_load_icon_buffers (icon, CAIRO_CONTAINER (pDock));  // fait un set_icon_size
-			icon->fWidth *= pDock->container.fRatio;
-			icon->fHeight *= pDock->container.fRatio;
-			
-			if (bRecursive && icon->pSubDock != NULL)
-			{
-				cairo_dock_synchronize_one_sub_dock_orientation (icon->pSubDock, pDock, FALSE);
-				cairo_dock_reload_buffers_in_dock (icon->pSubDock, bReloadAppletsToo, bRecursive);
-			}
-		}
-		
-		//g_print (" =size <- %.2fx%.2f\n", icon->fWidth, icon->fHeight);
-		fFlatDockWidth += myIconsParam.iIconGap + icon->fWidth;
-		if (! CAIRO_DOCK_ICON_TYPE_IS_SEPARATOR (icon))
-			pDock->iMaxIconHeight = MAX (pDock->iMaxIconHeight, icon->fHeight);
-	}
-	pDock->fFlatDockWidth = (int) fFlatDockWidth;  /// (int) n'est plus tellement necessaire ...
-}
-
 void cairo_dock_reload_icon_image (Icon *icon, CairoContainer *pContainer)
 {
-	if (pContainer)
+	/**if (pContainer)
 	{
 		icon->fWidth /= pContainer->fRatio;
 		icon->fHeight /= pContainer->fRatio;
-	}
+	}*/
 	cairo_dock_load_icon_image (icon, pContainer);
-	if (pContainer)
+	/**if (pContainer)
 	{
 		icon->fWidth *= pContainer->fRatio;
 		icon->fHeight *= pContainer->fRatio;
-	}
+	}*/
 }
 
-void cairo_dock_add_reflection_to_icon (Icon *pIcon, CairoContainer *pContainer)
+/**void cairo_dock_add_reflection_to_icon (Icon *pIcon, CairoContainer *pContainer)
 {
 	if (g_bUseOpenGL)
 		return ;
@@ -426,11 +340,12 @@ void cairo_dock_add_reflection_to_icon (Icon *pIcon, CairoContainer *pContainer)
 	pIcon->pReflectionBuffer = cairo_dock_create_reflection_surface (pIcon->pIconBuffer,
 		iWidth,
 		iHeight,
-		myIconsParam.fReflectSize * cairo_dock_get_max_scale (pContainer),
+		myIconsParam.fReflectHeightRatio * iHeight,
+		///myIconsParam.fReflectSize * cairo_dock_get_max_scale (pContainer),
 		myIconsParam.fAlbedo,
 		pContainer->bIsHorizontal,
 		pContainer->bDirectionUp);
-}
+}*/
 
 
   ///////////////////////
@@ -517,8 +432,8 @@ void cairo_dock_draw_subdock_content_on_icon (Icon *pIcon, CairoDock *pDock)
 		//\______________ On finit le dessin.
 		if (g_bUseOpenGL)
 			cairo_dock_update_icon_texture (pIcon);
-		else
-			cairo_dock_add_reflection_to_icon (pIcon, CAIRO_CONTAINER (pDock));
+		/**else
+			cairo_dock_add_reflection_to_icon (pIcon, CAIRO_CONTAINER (pDock));*/
 		cairo_destroy (pCairoContext);
 	}
 }
