@@ -42,6 +42,8 @@ extern CairoIconsParam myIconsParam;
 extern CairoIconsManager myIconsMgr;
 #endif
 
+#define CAIRO_DOCK_DEFAULT_ICON_SIZE 128
+
 // params
 typedef enum {
 	CAIRO_DOCK_NORMAL_SEPARATOR,
@@ -52,6 +54,7 @@ typedef enum {
 
 struct _CairoIconsParam {
 	// icons
+	CairoDockIconGroup tIconTypeOrder[CAIRO_DOCK_NB_GROUPS];
 	gdouble fReflectHeightRatio;
 	gdouble fAlbedo;
 	gdouble fAmplitude;
@@ -60,19 +63,19 @@ struct _CairoIconsParam {
 	gint iStringLineWidth;
 	gdouble fStringColor[4];
 	gdouble fAlphaAtRest;
-	gdouble fReflectSize;
+	///gdouble fReflectSize;
 	gchar *cIconTheme;
 	gchar *cBackgroundImagePath;
-	gint tIconAuthorizedWidth[CAIRO_DOCK_NB_GROUPS];
-	gint tIconAuthorizedHeight[CAIRO_DOCK_NB_GROUPS];
+	gint iIconWidth;  // default icon size
+	gint iIconHeight;
+	// separators
 	CairoDockSpeparatorType iSeparatorType;
+	gint iSeparatorWidth;
+	gint iSeparatorHeight;
 	gchar *cSeparatorImage;
 	gboolean bRevolveSeparator;
 	gboolean bConstantSeparatorSize;
 	gdouble fSeparatorColor[4];
-	gint iSeparateIcons;
-	CairoDockIconGroup tIconTypeOrder[CAIRO_DOCK_NB_GROUPS];
-	gint iIconsTypesList[3];  // pour la conf simple.
 	// labels
 	CairoDockLabelDescription iconTextDescription;
 	CairoDockLabelDescription quickInfoTextDescription;
@@ -94,7 +97,7 @@ struct _CairoIconsManager {
 /// signals
 typedef enum {
 	/// notification called when an icon's sub-dock is starting to (un)fold. data : {Icon}
-	NOTIFICATION_UNFOLD_SUBDOCK,
+	NOTIFICATION_UNFOLD_SUBDOCK = NB_NOTIFICATIONS_OBJECT,
 	/// notification called when an icon is updated in the fast rendering loop.
 	NOTIFICATION_UPDATE_ICON,
 	/// notification called when an icon is updated in the slow rendering loop.
@@ -128,11 +131,23 @@ void cairo_dock_hide_show_launchers_on_other_desktops (CairoDock *pDock);
 
 void cairo_dock_set_specified_desktop_for_icon (Icon *pIcon, int iSpecificDesktop);
 
+/** Search the icon size of a GtkIconSize.
+ * @param iIconSize a GtkIconSize
+ * @return the maximum between the width and the height of the icon size in pixel (or 128 if there is a problem) 
+ */
+gint cairo_dock_search_icon_size (GtkIconSize iIconSize);
+
 /** Search the path of an icon into the defined icons themes. It also handles the '~' caracter in paths.
  * @param cFileName name of the icon file.
+ * @param iDesiredIconSize desired icon size if we use icons from user icons theme.
  * @return the complete path of the icon, or NULL if not found.
  */
-gchar *cairo_dock_search_icon_s_path (const gchar *cFileName);
+gchar *cairo_dock_search_icon_s_path (const gchar *cFileName, gint iDesiredIconSize);
+
+void cairo_dock_add_path_to_icon_theme (const gchar *cPath);
+
+void cairo_dock_remove_path_from_icon_theme (const gchar *cPath);
+
 
 void gldi_register_icons_manager (void);
 
