@@ -23,13 +23,7 @@
 
 #include <gtk/gtk.h>
 
-#ifdef HAVE_GLITZ
-#include <gdk/gdkx.h>
-#include <glitz-glx.h>
-#include <cairo-glitz.h>
-#endif
 
-#include <gtk/gtkgl.h>
 
 #include "cairo-dock-draw.h"
 #include "cairo-dock-surface-factory.h"
@@ -115,7 +109,7 @@ void cairo_dock_load_image_buffer_full (CairoDockImageBuffer *pImage, const gcha
 	if (cImageFile == NULL)
 		return;
 	gchar *cImagePath = cairo_dock_search_image_s_path (cImageFile);
-	double w, h;
+	double w=0, h=0;
 	pImage->pSurface = cairo_dock_create_surface_from_image (
 		cImagePath,
 		1.,
@@ -129,6 +123,34 @@ void cairo_dock_load_image_buffer_full (CairoDockImageBuffer *pImage, const gcha
 	pImage->iWidth = w;
 	pImage->iHeight = h;
 	
+	if ((iLoadModifier & CAIRO_DOCK_ANIMATED_IMAGE) && h != 0)
+	{
+		//g_print ("%dx%d\n", (int)w, (int)h);
+		if (w >= 2*h)  // we need at least 2 frames (Note: we assume that frames are wide).
+		{
+			if ((int)w % (int)h == 0)  // w = k*h
+			{
+				pImage->iNbFrames = w / h;
+			}
+			else if (w > 2 * h)  // if we're pretty sure this image is an animated one, try to be smart, to handle the case of non-square frames.
+			{
+				// assume we have wide frames => w > h
+				int w_ = h+1, h_ = h;
+				do
+				{
+					//g_print (" %d/%d\n", w_, (int)w);
+					if ((int)w % w_ == 0)
+					{
+						pImage->iNbFrames = w / w_;
+						break;
+					}
+					w_ ++;
+				} while (w_ < w / 2);
+			}
+		}
+		//g_print ("CAIRO_DOCK_ANIMATED_IMAGE -> %d frames\n", pImage->iNbFrames);
+	}
+	
 	if (fAlpha < 1 && pImage->pSurface != NULL)
 	{
 		cairo_surface_t *pNewSurfaceAlpha = cairo_dock_create_blank_surface (
@@ -161,6 +183,15 @@ void cairo_dock_load_image_buffer_from_surface (CairoDockImageBuffer *pImage, ca
 		pImage->iTexture = cairo_dock_create_texture_from_surface (pImage->pSurface);
 }
 
+void cairo_dock_load_image_buffer_from_texture (CairoDockImageBuffer *pImage, GLuint iTexture)
+{
+	pImage->iTexture = iTexture;
+	pImage->iWidth = 1.;
+	pImage->iHeight = 1.;
+	pImage->fZoomX = 1.;
+	pImage->fZoomY = 1.;
+}
+
 CairoDockImageBuffer *cairo_dock_create_image_buffer (const gchar *cImageFile, int iWidth, int iHeight, CairoDockLoadImageModifier iLoadModifier)
 {
 	CairoDockImageBuffer *pImage = g_new0 (CairoDockImageBuffer, 1);
@@ -191,3 +222,38 @@ void cairo_dock_free_image_buffer (CairoDockImageBuffer *pImage)
 	g_free (pImage);
 }
 
+void cairo_dock_apply_image_buffer_surface_with_offset (CairoDockImageBuffer *pImage, cairo_t *pCairoContext, double x, double y, double fAlpha)
+{
+	if (cairo_dock_image_buffer_is_animated (pImage))
+	{
+		cairo_save (pCairoContext);
+		cairo_translate (pCairoContext, x, y);
+		cairo_rectangle (pCairoContext, 0, 0, pImage->iHeight, pImage->iHeight);
+		cairo_clip (pCairoContext);
+		cairo_set_source_surface (pCairoContext, pImage->pSurface, - pImage->iCurrentFrame * pImage->iHeight, 0.);
+		cairo_paint_with_alpha (pCairoContext, fAlpha);
+		cairo_restore (pCairoContext);
+	}
+	else
+	{
+		cairo_set_source_surface (pCairoContext, pImage->pSurface, 0., 0.);
+		cairo_paint_with_alpha (pCairoContext, fAlpha);
+	}
+}
+
+void cairo_dock_apply_image_buffer_texture_with_offset (CairoDockImageBuffer *pImage, double x, double y)
+{
+	glBindTexture (GL_TEXTURE_2D, pImage->iTexture);
+	if (cairo_dock_image_buffer_is_animated (pImage))
+	{
+		int iFrameWidth = pImage->iWidth / pImage->iNbFrames;
+		_cairo_dock_apply_current_texture_portion_at_size_with_offset ((double)pImage->iCurrentFrame / pImage->iNbFrames, 0,
+			1. / pImage->iNbFrames, 1.,
+			iFrameWidth, pImage->iHeight,
+			x, y);
+	}
+	else
+	{
+		_cairo_dock_apply_current_texture_at_size_with_offset (pImage->iWidth, pImage->iHeight, x, y);
+	}
+}
