@@ -23,13 +23,7 @@
 
 #include <gtk/gtk.h>
 
-#ifdef HAVE_GLITZ
-#include <gdk/gdkx.h>
-#include <glitz-glx.h>
-#include <cairo-glitz.h>
-#endif
 
-#include <gtk/gtkgl.h>
 
 #include "gldi-config.h"  // GLDI_SHARE_DATA_DIR
 #include "cairo-dock-draw.h"
@@ -39,7 +33,7 @@
 #include "cairo-dock-dock-manager.h"  // cairo_dock_redraw_root_docks
 #include "cairo-dock-draw-opengl.h"
 #include "cairo-dock-container.h"
-#include "cairo-dock-emblem.h"
+///#include "cairo-dock-emblem.h"
 #include "cairo-dock-applications-manager.h"  // cairo_dock_foreach_applis
 #include "cairo-dock-image-buffer.h"
 #include "cairo-dock-icon-manager.h"
@@ -68,13 +62,13 @@ static gboolean cairo_dock_render_indicator_notification (gpointer pUserData, Ic
  /// RENDERING ///
 /////////////////
 
-static inline double _compute_delta_y (Icon *icon, double py, gboolean bOnIcon, double fRatio, gboolean bUseReflect, double fMaxScale)
+static inline double _compute_delta_y (Icon *icon, double py, gboolean bOnIcon, gboolean bUseReflect)
 {
 	double dy;
 	if (bOnIcon)  // decalage vers le haut et zoom avec l'icone.
 		dy = py * icon->fHeight * icon->fScale + icon->fDeltaYReflection / 2;
 	else  // decalage vers le bas sans zoom.
-		dy = - py * ((bUseReflect ? myIconsParam.fReflectSize * fRatio : 0.) + myDocksParam.iFrameMargin + .5*myDocksParam.iDockLineWidth);
+		dy = - py * ((bUseReflect ? /**myIconsParam.fReflectSize * fRatio*/icon->fHeight * myIconsParam.fReflectHeightRatio : 0.) + myDocksParam.iFrameMargin + .5*myDocksParam.iDockLineWidth);
 	return dy;
 }
 
@@ -89,9 +83,10 @@ static void _cairo_dock_draw_appli_indicator_opengl (Icon *icon, CairoDock *pDoc
 	//\__________________ On calcule l'offset et le zoom.
 	double w = s_indicatorBuffer.iWidth;
 	double h = s_indicatorBuffer.iHeight;
-	double fMaxScale = cairo_dock_get_max_scale (pDock);
-	double z = (myIndicatorsParam.bIndicatorOnIcon ? icon->fScale / fMaxScale : 1.) * fRatio;  // on divise par fMaxScale car l'indicateur est charge a la taille max des icones.
-	double fY = _compute_delta_y (icon, myIndicatorsParam.fIndicatorDeltaY, myIndicatorsParam.bIndicatorOnIcon, fRatio, pDock->container.bUseReflect, fMaxScale);
+	///double fMaxScale = cairo_dock_get_max_scale (pDock);
+	///double z = (myIndicatorsParam.bIndicatorOnIcon ? icon->fScale / fMaxScale : 1.) * fRatio;  // on divise par fMaxScale car l'indicateur est charge a la taille max des icones.
+	double z = icon->fWidth / w * (myIndicatorsParam.bIndicatorOnIcon ? icon->fScale : 1.);
+	double fY = _compute_delta_y (icon, myIndicatorsParam.fIndicatorDeltaY, myIndicatorsParam.bIndicatorOnIcon, pDock->container.bUseReflect);
 	fY += - icon->fHeight * icon->fScale/2 + h*z/2;  // a 0, le bas de l'indicateur correspond au bas de l'icone.
 	
 	//\__________________ On place l'indicateur.
@@ -166,9 +161,10 @@ static void _cairo_dock_draw_appli_indicator (Icon *icon, CairoDock *pDock, cair
 	//\__________________ On calcule l'offset et le zoom.
 	double w = s_indicatorBuffer.iWidth;
 	double h = s_indicatorBuffer.iHeight;
-	double fMaxScale = cairo_dock_get_max_scale (pDock);
-	double z = (myIndicatorsParam.bIndicatorOnIcon ? icon->fScale / fMaxScale : 1.) * fRatio;  // on divise par fMaxScale car l'indicateur est charge a la taille max des icones.
-	double fY = - _compute_delta_y (icon, myIndicatorsParam.fIndicatorDeltaY, myIndicatorsParam.bIndicatorOnIcon, fRatio, pDock->container.bUseReflect, fMaxScale);  // a 0, le bas de l'indicateur correspond au bas de l'icone.
+	///double fMaxScale = cairo_dock_get_max_scale (pDock);
+	///double z = (myIndicatorsParam.bIndicatorOnIcon ? icon->fScale / fMaxScale : 1.) * fRatio;  // on divise par fMaxScale car l'indicateur est charge a la taille max des icones.
+	double z = icon->fWidth / w * (myIndicatorsParam.bIndicatorOnIcon ? icon->fScale : 1.);
+	double fY = - _compute_delta_y (icon, myIndicatorsParam.fIndicatorDeltaY, myIndicatorsParam.bIndicatorOnIcon, pDock->container.bUseReflect);  // a 0, le bas de l'indicateur correspond au bas de l'icone.
 	
 	//\__________________ On place l'indicateur.
 	cairo_save (pCairoContext);
@@ -481,8 +477,8 @@ static inline void _load_task_indicator (const gchar *cIndicatorImagePath, doubl
 {
 	cairo_dock_unload_image_buffer (&s_indicatorBuffer);
 	
-	double fLauncherWidth = (myIconsParam.tIconAuthorizedWidth[CAIRO_DOCK_LAUNCHER] != 0 ? myIconsParam.tIconAuthorizedWidth[CAIRO_DOCK_LAUNCHER] : 48);
-	double fLauncherHeight = (myIconsParam.tIconAuthorizedHeight[CAIRO_DOCK_LAUNCHER] != 0 ? myIconsParam.tIconAuthorizedHeight[CAIRO_DOCK_LAUNCHER] : 48);
+	double fLauncherWidth = myIconsParam.iIconWidth;
+	double fLauncherHeight = myIconsParam.iIconHeight;
 	double fScale = (myIndicatorsParam.bIndicatorOnIcon ? fMaxScale : 1.) * fIndicatorRatio;
 	
 	cairo_dock_load_image_buffer (&s_indicatorBuffer,
@@ -495,12 +491,8 @@ static inline void _load_active_window_indicator (const gchar *cImagePath, doubl
 {
 	cairo_dock_unload_image_buffer (&s_activeIndicatorBuffer);
 	
-	int iWidth = MAX (myIconsParam.tIconAuthorizedWidth[CAIRO_DOCK_LAUNCHER], myIconsParam.tIconAuthorizedWidth[CAIRO_DOCK_APPLI]);
-	int iHeight = MAX (myIconsParam.tIconAuthorizedHeight[CAIRO_DOCK_APPLI], myIconsParam.tIconAuthorizedHeight[CAIRO_DOCK_APPLI]);
-	if (iWidth == 0)
-		iWidth = 48;
-	if (iHeight == 0)
-		iHeight = 48;
+	int iWidth = myIconsParam.iIconWidth;
+	int iHeight = myIconsParam.iIconHeight;
 	iWidth *= fMaxScale;
 	iHeight *= fMaxScale;
 	
@@ -546,8 +538,8 @@ static inline void _load_class_indicator (const gchar *cIndicatorImagePath, doub
 {
 	cairo_dock_unload_image_buffer (&s_classIndicatorBuffer);
 	
-	double fLauncherWidth = (myIconsParam.tIconAuthorizedWidth[CAIRO_DOCK_LAUNCHER] != 0 ? myIconsParam.tIconAuthorizedWidth[CAIRO_DOCK_LAUNCHER] : 48);
-	double fLauncherHeight = (myIconsParam.tIconAuthorizedHeight[CAIRO_DOCK_LAUNCHER] != 0 ? myIconsParam.tIconAuthorizedHeight[CAIRO_DOCK_LAUNCHER] : 48);
+	double fLauncherWidth = myIconsParam.iIconWidth;
+	double fLauncherHeight = myIconsParam.iIconHeight;
 	
 	cairo_dock_load_image_buffer (&s_classIndicatorBuffer,
 		cIndicatorImagePath,
