@@ -28,6 +28,7 @@
 #include <cairo.h>
 
 #include "gldi-config.h"
+#include "cairo-dock-notifications.h"
 #include "cairo-dock-dock-facility.h"  // cairo_dock_update_dock_size
 #include "cairo-dock-dock-manager.h"
 #include "cairo-dock-keyfile-utilities.h"  // cairo_dock_conf_file_needs_update
@@ -291,16 +292,16 @@ GKeyFile *cairo_dock_pre_read_module_instance_config (CairoDockModuleInstance *p
 	//\____________________ on recupere les parametres de l'icone.
 	if (pInstance->pModule->pVisitCard->iContainerType & CAIRO_DOCK_MODULE_CAN_DOCK)  // l'applet peut aller dans un dock.
 	{
-		gboolean bUseless;
-		cairo_dock_get_size_key_value_helper (pKeyFile, "Icon", "icon ", bUseless, pMinimalConfig->iDesiredIconWidth, pMinimalConfig->iDesiredIconHeight);
-		if (pMinimalConfig->iDesiredIconWidth == 0)
-			pMinimalConfig->iDesiredIconWidth = myIconsParam.tIconAuthorizedWidth[CAIRO_DOCK_APPLET];
+		gboolean bUnused;
+		cairo_dock_get_size_key_value_helper (pKeyFile, "Icon", "icon ", bUnused, pMinimalConfig->iDesiredIconWidth, pMinimalConfig->iDesiredIconHeight);  // for a dock, if 0, will just get the default size; for a desklet, unused.
+		/**if (pMinimalConfig->iDesiredIconWidth == 0)
+			pMinimalConfig->iDesiredIconWidth = myIconsParam.iIconWidth;
 		if (pMinimalConfig->iDesiredIconWidth == 0)
 			pMinimalConfig->iDesiredIconWidth = 48;
 		if (pMinimalConfig->iDesiredIconHeight == 0)
-			pMinimalConfig->iDesiredIconHeight = myIconsParam.tIconAuthorizedHeight[CAIRO_DOCK_APPLET];
+			pMinimalConfig->iDesiredIconHeight = myIconsParam.iIconHeight;
 		if (pMinimalConfig->iDesiredIconHeight == 0)
-			pMinimalConfig->iDesiredIconHeight = 48;
+			pMinimalConfig->iDesiredIconHeight = 48;*/
 		
 		pMinimalConfig->cLabel = cairo_dock_get_string_key_value (pKeyFile, "Icon", "name", NULL, NULL, NULL, NULL);
 		if (pMinimalConfig->cLabel == NULL && !pInstance->pModule->pVisitCard->bAllowEmptyTitle)
@@ -327,17 +328,33 @@ GKeyFile *cairo_dock_pre_read_module_instance_config (CairoDockModuleInstance *p
 			s_iMaxOrder = MAX (s_iMaxOrder, pMinimalConfig->fOrder);
 		}
 		pMinimalConfig->cDockName = cairo_dock_get_string_key_value (pKeyFile, "Icon", "dock name", NULL, NULL, NULL, NULL);
-		pMinimalConfig->bAlwaysVisible = g_key_file_get_boolean (pKeyFile, "Icon", "always visi", NULL);
-		if (pMinimalConfig->bAlwaysVisible)
+		int iBgColorType;
+		if (g_key_file_has_key (pKeyFile, "Icon", "always_visi", NULL))
+		{
+			iBgColorType = g_key_file_get_integer (pKeyFile, "Icon", "always_visi", NULL);
+		}
+		else  // old param
+		{
+			iBgColorType = (g_key_file_get_boolean (pKeyFile, "Icon", "always visi", NULL) ? 2 : 0);  // keep the existing custom color
+			g_key_file_set_integer (pKeyFile, "Icon", "always_visi", iBgColorType);
+		}
+		pMinimalConfig->bAlwaysVisible = (iBgColorType != 0);
+		pMinimalConfig->pHiddenBgColor = NULL;
+		if (iBgColorType == 2)  // custom bg color
 		{
 			gsize length;
 			pMinimalConfig->pHiddenBgColor = g_key_file_get_double_list (pKeyFile, "Icon", "bg color", &length, NULL);
-			if (length < 4 || pMinimalConfig->pHiddenBgColor[3] == 0.)
+			if (length < 4 || pMinimalConfig->pHiddenBgColor[3] == 0)
 			{
 				g_free (pMinimalConfig->pHiddenBgColor);
 				pMinimalConfig->pHiddenBgColor = NULL;
 			}
 		}
+		else if (iBgColorType == 1)  // default bg color
+		{
+			if (myDocksParam.fHiddenBg[3] != 0)
+				pMinimalConfig->pHiddenBgColor = g_memdup (myDocksParam.fHiddenBg, sizeof (myDocksParam.fHiddenBg));
+		}
 	}
 	
 	//\____________________ on recupere les parametres de son desklet.
@@ -449,13 +466,14 @@ CairoDockModuleInstance *cairo_dock_instanciate_module (CairoDockModule *pModule
 		pInstance->bCanDetach = pMinimalConfig->deskletAttribute.iDeskletWidth > 0;
 		pModule->bCanDetach = pInstance->bCanDetach;  // pas encore clair ...
 		
-		// on trouve/cree son container.
+		// create the icon.
+		pIcon = cairo_dock_create_icon_for_applet (pMinimalConfig,
+			pInstance);
+		
+		// create/find its container and insert the icon inside.
 		if (pModule->bCanDetach && pMinimalConfig->bIsDetached)
 		{
-			pDesklet = cairo_dock_create_desklet (NULL, &pMinimalConfig->deskletAttribute);
-			/*cd_debug ("transparence du desklet...\n");
-			while (gtk_events_pending ())  // pour la transparence initiale.
-				gtk_main_iteration ();*/
+			pDesklet = cairo_dock_create_desklet (pIcon, &pMinimalConfig->deskletAttribute);
 			pContainer = CAIRO_CONTAINER (pDesklet);
 		}
 		else
@@ -464,79 +482,36 @@ CairoDockModuleInstance *cairo_dock_instanciate_module (CairoDockModule *pModule
 			pDock = cairo_dock_search_dock_from_name (cDockName);
 			if (pDock == NULL)
 			{
-				pDock = cairo_dock_create_dock (cDockName, NULL);
+				pDock = cairo_dock_create_dock (cDockName);
 			}
 			pContainer = CAIRO_CONTAINER (pDock);
+			
+			if (pDock)
+			{
+				cairo_dock_insert_icon_in_dock (pIcon, pDock, ! cairo_dock_is_loading ());  // animate the icon if it's instanciated by the user, not during the initial loading.
+				
+				// we need to load the icon's buffer before we init the module, because the applet may need it. no ned to do it in desklet mode, since the desklet doesn't have a renderer yet (so buffer can't be loaded).
+				cairo_dock_load_icon_buffers (pIcon, pContainer);  // ne cree rien si w ou h < 0 (par exemple si l'applet est detachee).
+			}
 		}
-		
-		// on cree son icone.
-		pIcon = cairo_dock_create_icon_for_applet (pMinimalConfig,
-			pInstance,
-			pContainer);
-		if (pDesklet)
-		{
-			pDesklet->pIcon = pIcon;
-			gtk_window_set_title (GTK_WINDOW(pContainer->pWidget), pInstance->pModule->pVisitCard->cModuleName);
-		}
+
+		pInstance->pIcon = pIcon;
+		pInstance->pDock = pDock;
+		pInstance->pDesklet = pDesklet;
+		pInstance->pContainer = pContainer;
 	}
 	cairo_dock_free_minimal_config (pMinimalConfig);
-
-	//\____________________ On initialise l'instance.
-	if (pDock)  //  on met la taille qu'elle aura une fois dans le dock.
-	{
-		pIcon->fWidth *= pDock->container.fRatio;
-		pIcon->fHeight *= pDock->container.fRatio;
-	}
-	
-	pInstance->pIcon = pIcon;
-	pInstance->pDock = pDock;
-	pInstance->pDesklet = pDesklet;
-	pInstance->pContainer = pContainer;
 	
+	//\____________________ initialise the instance.
 	if (pKeyFile)
 		_cairo_dock_read_module_config (pKeyFile, pInstance);
 	
-	gboolean bCanInit = TRUE;
-	/**pInstance->pDrawContext = NULL;
-	if (pDock && pIcon)  // applet dans un dock (dans un desklet, il faut attendre que l'applet ait mis une vue pour que l'icone soit chargee).
-	{
-		if (pIcon->pIconBuffer == NULL)
-		{
-			cd_warning ("icon's buffer is NULL, applet won't be able to draw to it !");
-			pInstance->pDrawContext = NULL;
-			bCanInit = FALSE;
-		}
-		else
-		{
-			pInstance->pDrawContext = cairo_create (pIcon->pIconBuffer);
-			if (!pInstance->pDrawContext || cairo_status (pInstance->pDrawContext) != CAIRO_STATUS_SUCCESS)
-			{
-				cd_warning ("couldn't initialize drawing context, applet won't be able to draw itself !");
-				pInstance->pDrawContext = NULL;
-				bCanInit = FALSE;
-			}
-		}
-	}*/
-	
-	if (bCanInit && pModule->pInterface->initModule)
+	if (pModule->pInterface->initModule)
 		pModule->pInterface->initModule (pInstance, pKeyFile);
 	
-	if (pDock)
-	{
-		pIcon->fWidth /= pDock->container.fRatio;
-		pIcon->fHeight /= pDock->container.fRatio;
-		if (cairo_dock_is_loading ())
-		{
-			cairo_dock_insert_icon_in_dock (pIcon, pDock, ! CAIRO_DOCK_UPDATE_DOCK_SIZE, ! CAIRO_DOCK_ANIMATE_ICON);
-		}
-		else
-		{
-			cairo_dock_insert_icon_in_dock (pIcon, pDock, CAIRO_DOCK_UPDATE_DOCK_SIZE, CAIRO_DOCK_ANIMATE_ICON);
-			cairo_dock_launch_animation (CAIRO_CONTAINER (pDock));
-		}
-	}
-	else if (pDesklet && pDesklet->iDesiredWidth == 0 && pDesklet->iDesiredHeight == 0)  // peut arriver si le desklet a fini de se redimensionner avant l'init.
+	if (pDesklet && pDesklet->iDesiredWidth == 0 && pDesklet->iDesiredHeight == 0)  // peut arriver si le desklet a fini de se redimensionner avant l'init.
 		gtk_widget_queue_draw (pDesklet->container.pWidget);
+	
 	if (pKeyFile != NULL)
 		g_key_file_free (pKeyFile);
 	return pInstance;
@@ -583,7 +558,7 @@ static void _cairo_dock_stop_module_instance (CairoDockModuleInstance *pInstance
 	}
 }
 
-void cairo_dock_deinstanciate_module (CairoDockModuleInstance *pInstance)
+void cairo_dock_deinstanciate_module (CairoDockModuleInstance *pInstance)  // stop an instance of a module
 {
 	_cairo_dock_stop_module_instance (pInstance);
 	
@@ -660,7 +635,7 @@ void cairo_dock_reload_module_instance (CairoDockModuleInstance *pInstance, gboo
 				if (pNewDock == NULL)  // c'est un nouveau dock.
 				{
 					cairo_dock_add_root_dock_config_for_name (cDockName);
-					pNewDock = cairo_dock_create_dock (cDockName, NULL);
+					pNewDock = cairo_dock_create_dock (cDockName);
 				}
 				pNewContainer = CAIRO_CONTAINER (pNewDock);
 			}
@@ -670,7 +645,7 @@ void cairo_dock_reload_module_instance (CairoDockModuleInstance *pInstance, gboo
 			{
 				cd_message ("le container a change (%s -> %s)", pIcon->cParentDockName, pMinimalConfig->bIsDetached ? "desklet" : pMinimalConfig->cDockName);
 				cOldDockName = g_strdup (pIcon->cParentDockName);
-				cairo_dock_detach_icon_from_dock (pIcon, pCurrentDock, myIconsParam.iSeparateIcons);
+				cairo_dock_detach_icon_from_dock (pIcon, pCurrentDock);
 			}
 			
 			// on recupere son desklet (cree au besoin).
@@ -710,31 +685,23 @@ void cairo_dock_reload_module_instance (CairoDockModuleInstance *pInstance, gboo
 			g_key_file_free (pKeyFile);
 			pKeyFile = NULL;
 		}
-		pIcon->fWidth = pMinimalConfig->iDesiredIconWidth;
+		pIcon->fWidth = pMinimalConfig->iDesiredIconWidth;  // requested size
 		pIcon->fHeight = pMinimalConfig->iDesiredIconHeight;
-		
-		// on charge l'icone a la bonne taille.
-		cairo_dock_set_icon_size (pNewContainer, pIcon);
-		cairo_dock_load_icon_buffers (pIcon, pNewContainer);
+		pIcon->iImageWidth = 0;
+		pIcon->iImageHeight = 0;
 		
 		// on insere l'icone dans le dock ou on met a jour celui-ci.
-		if (pNewDock != pCurrentDock)  // on l'insere dans son nouveau dock.
+		if (pNewDock != pCurrentDock)  // insert in its new dock.
 		{
-			cairo_dock_insert_icon_in_dock (pIcon, pNewDock, CAIRO_DOCK_UPDATE_DOCK_SIZE, CAIRO_DOCK_ANIMATE_ICON);
+			cairo_dock_insert_icon_in_dock (pIcon, pNewDock, CAIRO_DOCK_ANIMATE_ICON);
 			pIcon->cParentDockName = g_strdup (pMinimalConfig->cDockName != NULL ? pMinimalConfig->cDockName : CAIRO_DOCK_MAIN_DOCK_NAME);
-			cairo_dock_start_icon_animation (pIcon, pNewDock);
+			cairo_dock_load_icon_buffers (pIcon, pNewContainer);  // do it now, since the applet may need it. no ned to do it in desklet mode, since the desklet doesn't have a renderer yet (so buffer can't be loaded).
 		}
-		else  // le dock n'a pas change, on le met a jour.
+		else  // same dock, just update its size.
 		{
-			pIcon->fWidth *= pNewContainer->fRatio;
-			pIcon->fHeight *= pNewContainer->fRatio;
-			
+			cairo_dock_resize_icon_in_dock (pIcon, pNewDock);
 			if (bReloadAppletConf)
-			{
-				cairo_dock_update_dock_size (pNewDock);
-				cairo_dock_calculate_dock_icons (pNewDock);
-				gtk_widget_queue_draw (pNewContainer->pWidget);
-			}
+				cairo_dock_load_icon_text (pIcon, &myIconsParam.iconTextDescription);
 		}
 	}
 	
@@ -745,38 +712,15 @@ void cairo_dock_reload_module_instance (CairoDockModuleInstance *pInstance, gboo
 		_cairo_dock_read_module_config (pKeyFile, pInstance);
 	}
 	
-	/**if (pInstance->pDrawContext != NULL)
-	{
-		cairo_destroy (pInstance->pDrawContext);
-		pInstance->pDrawContext = NULL;
-	}
-	if (pIcon && pIcon->pIconBuffer)  // applet, on lui associe un contexte de dessin avant le reload.
-	{
-		pInstance->pDrawContext = cairo_create (pIcon->pIconBuffer);
-		if (!pInstance->pDrawContext || cairo_status (pInstance->pDrawContext) != CAIRO_STATUS_SUCCESS)
-		{
-			cd_warning ("couldn't initialize drawing context, applet won't be reloaded !");
-			bCanReload = FALSE;
-			pInstance->pDrawContext = NULL;
-		}
-	}*/
-
 	//\_______________________ On recharge l'instance.
 	if (bCanReload && module->pInterface->reloadModule != NULL)
 		module->pInterface->reloadModule (pInstance, pCurrentContainer, pKeyFile);
 	
-	if (pNewContainer != pCurrentContainer && pNewDock != NULL && pCurrentDock != NULL && pIcon != NULL && pIcon->pSubDock != NULL)
-	{
-		cairo_dock_synchronize_one_sub_dock_orientation (pIcon->pSubDock, pNewDock, TRUE);
-	}
-	
 	if (pNewDock != NULL && pNewDock->iRefCount != 0)  // on redessine l'icone pointant sur le sous-dock contenant l'applet, au cas ou son image aurait change.
 	{
 		cairo_dock_redraw_subdock_content (pNewDock);
 	}
 	
-	///cairo_dock_trigger_refresh_launcher_gui ();  /// voir s'il y'a besoin de qqch ici...
-	
 	//\_______________________ On nettoie derriere nous.
 	cairo_dock_free_minimal_config (pMinimalConfig);
 	if (pKeyFile != NULL)
@@ -794,7 +738,6 @@ void cairo_dock_reload_module_instance (CairoDockModuleInstance *pInstance, gboo
 		else
 		{
 			cairo_dock_update_dock_size (pCurrentDock);
-			cairo_dock_calculate_dock_icons (pCurrentDock);
 			gtk_widget_queue_draw (pCurrentContainer->pWidget);
 		}
 	}
@@ -884,10 +827,6 @@ void cairo_dock_activate_module (CairoDockModule *module, GError **erreur)
 		{
 			gchar *cConfFilePath = g_strdup_printf ("%s/%s", cUserDataDirPath, module->pVisitCard->cConfFileName);
 			gboolean r = cairo_dock_copy_file (module->cConfFilePath, cConfFilePath);
-			/**gchar *command = g_strdup_printf ("cp \"%s\" \"%s\"", module->cConfFilePath, cConfFilePath);
-			int r = system (command);
-			g_free (command);*/
-			
 			if (! r)  // the copy failed.
 			{
 				g_set_error (erreur, 1, 1, "couldn't copy %s into %s; check permissions and file's existence", module->cConfFilePath, cUserDataDirPath);
@@ -911,7 +850,7 @@ void cairo_dock_activate_module (CairoDockModule *module, GError **erreur)
 	cairo_dock_notify_on_object (&myModulesMgr, NOTIFICATION_MODULE_ACTIVATED, module->pVisitCard->cModuleName, TRUE);
 }
 
-void cairo_dock_deactivate_module (CairoDockModule *module)
+void cairo_dock_deactivate_module (CairoDockModule *module)  // stop all instances of a module
 {
 	g_return_if_fail (module != NULL);
 	cd_debug ("%s (%s, %s)", __func__, module->pVisitCard->cModuleName, module->cConfFilePath);
