@@ -27,6 +27,7 @@
 #include <cairo.h>
 
 #include "gldi-config.h"
+#include "cairo-dock-notifications.h"
 #include "cairo-dock-dock-facility.h"  // cairo_dock_update_dock_size
 #include "cairo-dock-dock-manager.h"
 #include "cairo-dock-file-manager.h"  // cairo_dock_copy_file
@@ -173,9 +174,11 @@ CairoDockModule * cairo_dock_load_module (const gchar *cSoFilePath)  // cSoFileP
 	
 	g_hash_table_insert (s_hModuleTable, (gpointer)pModule->pVisitCard->cModuleName, pModule);
 	
-	if (cairo_dock_module_is_auto_loaded (pModule))  // c'est un module qui soit ne peut etre activer et/ou desactiver, soit etend un manager; on l'activera donc automatiquement.
+	if (cairo_dock_module_is_auto_loaded (pModule))  // c'est un module qui soit ne peut etre activer et/ou desactiver, soit etend un manager; on l'active donc automatiquement.
+	{
 		s_AutoLoadedModules = g_list_prepend (s_AutoLoadedModules, pModule);
-	
+	}
+		
 	cairo_dock_notify_on_object (&myModulesMgr, NOTIFICATION_MODULE_REGISTERED, pModule->pVisitCard->cModuleName, TRUE);
 	return pModule;
 }
@@ -273,16 +276,27 @@ void cairo_dock_activate_modules_from_list (gchar **cActiveModuleList)
 
 static void _cairo_dock_deactivate_one_module (gchar *cModuleName, CairoDockModule *pModule, gpointer data)
 {
-	cairo_dock_deactivate_module (pModule);
+	if (! cairo_dock_module_is_auto_loaded (pModule))
+		cairo_dock_deactivate_module (pModule);
 }
 void cairo_dock_deactivate_all_modules (void)
 {
+	// first deactivate applets
 	g_hash_table_foreach (s_hModuleTable, (GHFunc) _cairo_dock_deactivate_one_module, NULL);
 	if (s_iSidWriteModules != 0)
 	{
 		g_source_remove (s_iSidWriteModules);
 		s_iSidWriteModules = 0;
 	}
+	
+	// then deactivate auto-loaded modules (that have been loaded first)
+	CairoDockModule *pModule;
+	GList *m;
+	for (m = s_AutoLoadedModules; m != NULL; m = m->next)
+	{
+		pModule = m->data;
+		cairo_dock_deactivate_module (pModule);
+	}
 }
 
 
@@ -318,8 +332,8 @@ void cairo_dock_activate_module_and_load (const gchar *cModuleName)
 		pInstance = pElement->data;
 		if (pInstance->pDock)
 		{
-			cairo_dock_update_dock_size (pInstance->pDock);
-			gtk_widget_queue_draw (pInstance->pDock->container.pWidget);
+			cairo_dock_trigger_update_dock_size (pInstance->pDock);
+			///gtk_widget_queue_draw (pInstance->pDock->container.pWidget);
 		}
 	}
 	
@@ -493,7 +507,6 @@ void cairo_dock_detach_module_instance (CairoDockModuleInstance *pInstance)
 		if (pInstance->pDesklet)  // on a bien detache l'applet.
 			cairo_dock_zoom_out_desklet (pInstance->pDesklet);
 		//\__________________ On met a jour le panneau de conf s'il etait ouvert sur cette applet.
-		///cairo_dock_update_desklet_detached_state_in_gui (pInstance, !bIsDetached);
 		cairo_dock_notify_on_object (&myModulesMgr, NOTIFICATION_MODULE_INSTANCE_DETACHED, pInstance, !bIsDetached);
 	}
 }
@@ -527,8 +540,6 @@ void cairo_dock_detach_module_instance_at_position (CairoDockModuleInstance *pIn
 	g_key_file_free (pKeyFile);
 	
 	//\__________________ On met a jour le panneau de conf s'il etait ouvert sur cette applet.
-	///cairo_dock_update_desklet_position_in_gui (pInstance, iDeskletPositionX, iDeskletPositionY);
-	///cairo_dock_update_desklet_detached_state_in_gui (pInstance, TRUE);
 	cairo_dock_notify_on_object (&myModulesMgr, NOTIFICATION_MODULE_INSTANCE_DETACHED, pInstance, TRUE);  // inutile de notifier du changement de taille, le configure-event du desklet s'en chargera.
 	
 	//\__________________ On detache l'applet.
